("GNU" "GNU")
("LaTeX" "LaTeX")
("TeX" "TeX")
("a modifier" "A veranderaar")
("a series" "a reeks")
("abbreviation" "afkorting")
("about" "over")
("about menu" "over menu")
("above" "boven")
("abstract" "samenvatting")
("accent" "accent")
("accept" "accepteer")
("accept all scripts" "accepteer alle scripts")
("accept no scripts" "accepteer geen scripts")
("accept script" "accepteer script")
("acknowledgment" "dankwoord")
("acronym" "acronym")
("action" "aktie")
("activate" "aktiveer")
("activate once" "aktiveer eenmalig")
("activate hidden" "aktiveer en verberg")
("activation" "aktivering")
("acute" "naar rechts")
("add package" "gebruik pakket")
("add switch after" "voeg 'schakelaar hierna' in")
("add switch before" "voeg 'schakelaar hiervoor' in")
("address" "adres")
("address block" "adres blok")
("advanced" "geavanceerd")
("advanced layout features" "geavanceerde pagina-opmaak")
("after" "na")
("albanian" "albanees")
("algorithm" "algorithme")
("align cell to the bottom" "plaats inhoud cel aan de benedenkant")
("align cell to the left" "plaats inhoud cel aan de linkerkant")
("align cell to the right" "plaats inhoud cel aan de rechterkant")
("align cell to the top" "plaats inhoud cel aan de bovenkant")
("align table at base of left column" "plaats tabel op richtlijn van meest linkse kolom")
("align table at base of middle column" "plaats tabel op richtlijn van middelste kolom")
("align table at base of origin column" "plaats tabel op richtlijn van oorsprong-kolom")
("align table at base of right column" "plaats tabel op richtlijn van meest rechtse kolom")
("align table at fraction bar height" "plaats tabel op hoogte van de breukstreep")
("align table at the base of bottom row" "plaats tabel op richtlijn van laagste rij")
("align table at the base of middle row" "plaats tabel op richtlijn van middelste rij")
("align table at the base of origin row" "plaats tabel op de richtlijn van oorsprong-rij")
("align table at the base of top row" "plaats tabel op richtlijn van hoogste rij")
("align table at the bottom" "plaats tabel aan de benedenkant")
("align table at the center" "plaats tabel in het vertikale midden")
("align table at the horizontal center" "plaats tabel in het horizontale midden")
("align table at the left" "plaats tabel aan de linkerkant")
("align table at the top" "plaats tabel aan de bovenkant")
("align table to the right" "plaats tabel aan de rechterkant")
("align text to the left" "lijn tekst naar links uit")
("align text to the right" "lijn tekst naar rechts uit")
("aligned" "gericht")
("alignment" "plaatsing")
("all" "alles")
("allow" "sta toe")
("allow for macro definitions in preamble" "sta macro definities in preambule toe")
("allowed extension" "toegestane verlenging")
("allowed reduction" "toegestane reduktie")
("alpha release" "alpha versie")
("alt key" "alt toets")
("alternative specification" "alternatieve specificatie")
("always" "altijd")
("american" "amerikaans")
("and" "en")
("angular" "hoekig")
("appendix" "appendix")
("apply" "gebruik")
("apply macro" "gebruik macro")
("apply macro once" "gebruik macro eenmalig")
("april" "april")
("arab" "arabisch")
("arc cosine" "boogcosinus")
("arc sine" "boogsinus")
("arc tangent" "boogtangens")
("arch series" "arch reeks")
("archaic" "archaisch")
("architecture" "opzet")
("argument" "argument")
("arithmetic" "arithmetiek")
("armenian" "armeens")
("arrow" "pijl")
("arrows" "pijlen")
("article" "artikel")
("as on paper" "zoals op het papier")
("assign" "verander")
("at fraction bar height" "op hoogte breukstreep")
("at the baseline" "op de richtlijn")
("at the bottom" "onder")
("at the left" "links")
("at the middle" "in het midden")
("at the right" "rechts")
("at the top" "boven")
("attach" "koppel")
("attach buffer to which project" "koppel buffer aan welk projekt")
("august" "augustus")
("author" "auteur")
("authorize" "keur goed")
("automatic" "automatisch")
("automatically close brackets" "sluit haakjes automatisch")
("automatically generated content" "automatisch aangemaakte inhoud")
("automatic quotes" "automatische aanhalingstekens")
("autosave" "automatisch opslaan")
("axis" "as")
("axiom" "axioma")
("b series" "b reeks")
("background" "achtergrond")
("background color" "achtergrond kleur")
("background evaluations" "evaluaties op de achtergrond")
("backward replace" "vervang achteruit")
("backward search" "zoek achteruit")
("bad data format" "verkeerd bestandsformaat")
("bar" "streep")
("bard" "bardisch")
("base level" "basisniveau")
("baseline" "richtlijn")
("bashkirian" "bashkirisch")
("basic" "basis")
("basic communication using pipes" "elementaire kommunicatie over pijpen")
("basic data types" "fundamentele data types")
("basic types" "basis types")
("be useful" "help mee")
("become a TeXmacs developer" "word een TeXmacs ontwikkelaar")
("before" "voor")
("begin" "begin")
("below" "onder")
("beta release" "beta versie")
("bibliography" "bibliografie")
("bibliography file" "bibliografie bestand")
("bibliography style" "bibliografie stijl")
("big" "grote")
("big figure" "groot figuur")
("big operator" "grote operator")
("big skip" "grote sprong")
("big table" "grote tabel")
("binary operation" "binaire operatie")
("binary relation" "binaire relatie")
("black" "zwart")
("blackboard bold" "schoolbord vet")
("blackboard math" "schoolbord wiskunde")
("blackletter" "zwarte letter")
("blank column" "lege kolom")
("blank row" "lege rij")
("block content" "blokinhoud")
("blue" "blauw")
("body" "lichaam")
("bold" "vet")
("book" "boek")
("border" "rand")
("both" "beide")
("bottom" "bodem")
("bottom border" "onderrand")
("bottom cell border" "onderrand cel")
("bottom cell padding" "ruimte onderkant cel")
("bottom margin" "onderrand")
("bottom padding" "ruimte beneden")
("bottom table border" "onderrand tabel")
("bottom table padding" "ruimte onderkant tabel")
("branch" "tak")
("break" "breek af")
("breaking" "afbraak")
("breve" "breve")
("british" "brits")
("broken white" "gebroken wit")
("brown" "bruin")
("browse" "surfen")
("browse web" "surf over het web")
("browser" "surfprogramma")
("buffer" "buffer")
("bulgarian" "bulgaars")
("bullet" "rondje")
("bullets" "rondjes")
("by" "door")
("by typing" "ook verkrijgen via de toetsenkombinatie")
("bytes free" "bytes vrij")
("bytes used" "bytes gebruikt")
("calculus" "differentiaalrekening")
("calligraphic" "kaligrafisch")
("cancel" "annuleer")
("caps-lock key" "caps-lock toets")
("case" "kies")
("cell" "cel")
("cell background color" "achtergrondkleur cel")
("cell border" "rand cel")
("cell height" "hoogte van de cel")
("cell operation mode" "celmode")
("cell width" "breedte van de cel")
("cells" "cellen")
("center" "midden")
("center text" "centreer tekst")
("centered" "gecentreerd")
("centered block" "gencetreerd blok")
("centered tabular" "gecentreerde tabel")
("change border of cell" "verander celrand")
("change cell operation mode" "verander celmode")
("change log" "opgave van veranderingen")
("change the border of the table" "verander de rand van de tabel")
("changes" "veranderingen")
("chapter" "hoofdstuk")
("check" "check")
("check text for spelling errors" "zoek spelfouten in tekst")
("chinese" "chinees")
("choice" "keuze")
("citation" "citatie")
("cite" "citeer")
("class" "klasse")
("clear" "wis")
("clear selection" "wis selektie")
("clipped" "afgeknot")
("clipped and magnified" "afgeknot en uitvergroot")
("clipped and resized" "afgeknot en uitgerekt")
("clone" "kloon")
("clone window" "dupliceer venster")
("close" "sluit af")
("close TeXmacs" "sluit TeXmacs")
("close buffer" "sluit buffer")
("close document" "sluit dokument")
("close session" "sluit sessie")
("close window" "sluit venster")
("closing" "afsluiting")
("closing style" "sluitstijl")
("code" "code")
("color" "kleur")
("column" "kolom")
("column contains glued items" "kolom bevat aan elkaar gelijmde delen")
("column to the left" "kolom naar links")
("column to the right" "kolom naar rechts")
("columns" "kolommen")
("common formats" "algemene formaten")
("compact" "kompakt")
("compactification" "kompaktheid")
("compactify" "maak kompakt")
("compatibility with other formats" "uitwisselbaarheid met andere formaten")
("compile article" "stel artikel samen")
("compile book" "stel boek samen")
("compile buffer" "compileer buffer")
("compile project" "compileer projekt")
("completion" "kompletering")
("complex" "complex")
("concrete" "concreet")
("condensed" "nauw")
("condition" "voorwaarde")
("configuration" "configuratie")
("contact" "kontakt")
("content" "inhoud")
("content tag" "inhoudweergave")
("context dependent icons" "context afhankelijke iconen")
("continuation of" "vervolg van")
("contract logarithms" "logarithmes inkrimpen")
("contribute" "doe mee")
("control the evaluation of expressions" "kontroleer de evaluatie van expressies")
("convention" "conventie")
("converters" "uitwisselaars")
("converters to other data formats" "uitwisselaars met andere formaten")
("cosine" "cosinus")
("copy" "kopiëer")
("copy text" "kopiëer tekst")
("copy to" "kopiëer naar")
("copying" "copiëren")
("corollary" "gevolg")
("correct text" "korrigeer tekst")
("create a new document" "begin met een nieuw dokument")
("create web site" "website creëren")
("cursors" "cursor")
("curve" "kromme")
("custom tab" "gepersonnaliseerde tabulering")
("customizing TeXmacs" "personaliseer TeXmacs")
("customizing the TeXmacs styles" "personaliseer de TeXmacs stijlen")
("custum tab" "gepersonnaliseerde tabulering")
("cut" "wis")
("cut text" "wis tekst")
("cut to" "wis naar")
("cypriot" "cypriotisch")
("cyrillic" "cyrillisch")
("cyrillic input method" "cyrillisch via toetsenbord")
("cyrillic italic right" "cyrillisch scheef maar recht")
("czech" "tsjechisch")
("dancers" "dansers")
("danish" "deens")
("dark grey" "donkergrijs")
("dash" "streep")
("dashes" "strepen")
("data conversion" "gegevens omzetten")
("data format" "gegevensformaat")
("date" "datum")
("deactivate" "deaktiveer")
("deactivate table" "deaktiveer tabel")
("december" "december")
("decimal comma" "decimale komma")
("decimal dot" "decimale punt")
("decorate atoms" "versier atomen")
("default" "vooringesteld")
("default padding" "gebruik standaard ruimte")
("definition" "definitie")
("delete" "verwijder")
("delete column" "wis kolom")
("delete row" "wis rij")
("denominator" "noemer")
("description" "beschrijving")
("destination" "bestemming")
("detach" "ontkoppel")
("detailed" "gedetailleerd")
("detailed menus" "gedetailleerde menus")
("details in menus" "details in de menus")
("determinant" "determinant")
("devangari" "devangarisch")
("developers guide" "gids voor ontwikkelaars")
("differentiate" "afgeleide")
("directory" "dossier")
("disable" "uit")
("disable first indentation" "begin inspringen uit")
("disable first indentation after" "begin inspringen uit hierna")
("disable first indentation before" "begin inspringen uit hiervoor")
("disable indentation" "inspringen uit")
("disable indentation after" "inspringen uit hierna")
("disable indentation before" "inspringen uit hiervoor")
("disable last indentation" "eind inspringen uit")
("disable last indentation after" "eind inspringen uit hierna")
("disable last indentation before" "eind inspringen uit hiervoor")
("disactivate" "disaktiveer")
("disactivate once" "disaktiveer eenmalig")
("disallow" "verbied")
("distribute unused space" "verdeel ongebruikte ruimte")
("div" "div")
("do not remap" "niet doorsturen")
("document" "document")
("document format" "dokumentformaat")
("document style" "document stijl")
("document styles" "document stijlen")
("documentation" "dokumentatie")
("documents are trees" "dokumenten zijn bomen")
("dot" "rondje")
("dots" "rondjes")
("double stroke" "dubbele lijn")
("down" "omlaag")
("download" "haal van web")
("dpi" "punten per duim")
("draw image" "teken plaatje")
("draw over selection" "teken over selektie")
("duplicate" "dubbel")
("dutch" "nederlands")
("dynamic libraries" "dynamische bibliotheken")
("edit" "bewerken")
("edit source tree" "bewerk de boomstruktuur")
("editing tools" "bewerkingsgereedschap")
("eigenvalues" "eigenwaarden")
("email" "email")
("emphasize" "benadruk")
("emphasize text" "benadruk tekst")
("empty box" "lege rechthoek")
("enable first indentation" "begin inspringen aan")
("enable first indentation after" "begin inspringen aan hierna")
("enable first indentation before" "begin inspringen aan hiervoor")
("enable indentation" "inspringen aan")
("enable indentation after" "inspringen aan hierna")
("enable indentation before" "inspringen aan hiervoor")
("enable last indentation" "eind inspringen aan")
("enable last indentation after" "eind inspringen aan hierna")
("enable last indentation before" "eind inspringen aan hiervoor")
("encoding" "codering")
("end" "einde")
("english" "engels")
("enter a title" "geef een titel op")
("entire table" "hele tabel")
("enumerate" "nummeren")
("enumerate|Alpha" "Letters")
("enumerate|Roman" "Romeins")
("enumerate|alpha" "letters")
("enumerate|roman" "romeins")
("env" "omgeving")
("environment" "omgeving")
("epilogue" "nawoord")
("equal" "gelijk")
("equation" "vergelijking")
("equations" "vergelijkingen")
("equivalent for" "equivalent voor")
("error" "fout")
("esperanto" "esperanto")
("etruscan" "etruskisch")
("evaluate" "evalueer")
("evaluate scheme expression" "scheme expressie uitvoeren")
("evaluation switch" "evaluatie schakelaar")
("evaluation tag" "evaluatie veld")
("even page footer" "even pagina voet")
("even page header" "even pagina kop")
("even page margin" "even pagina rand")
("even page left margin" "even pagina linkerrand")
("even page shift" "even pagina verschuiving")
("even page text" "even pagina tekst")
("exact horizontal" "horizontaal exact")
("exact mode" "exakte mode")
("exam" "examen")
("example" "voorbeeld")
("exclusive" "exclusief")
("exclusive or" "ofwel")
("executable" "uitvoerbaar")
("execute" "voer uit")
("execute a TeXmacs command" "TeXmacs kommando uitvoeren")
("execute system command" "systeemkommando uitvoeren")
("executive" "executief")
("exercise" "oefening")
("expand" "ontwikkel")
("expand logarithms" "logarithmes uitbreiden")
("expand trigonometric functions" "trigonometrische funkties uitbreiden")
("expand unrecognized macros" "onbekende macros uitbreiden")
("expand user-defined macros" "gebruikersmacros uitbreiden")
("explained" "met uitleg")
("exponential" "exponentiele funktie")
("export" "exporteer")
("export as" "exporteer als")
("export bibliographies as links" "bibliografiën als verwijzingen exporteren")
("export formulas as images" "formules exporteren als plaatjes")
("extend or normal" "'extend' of 'normal'")
("extern " "extern")
("extra" "extra")
("extract format" "haal formaat eruit")
("factor" "ontbind")
("family" "familie")
("february" "februari")
("figure" "figuur")
("file" "bestand")
("file name" "bestandsnaam")
("file not found" "bestand niet gevonden")
("find next match" "vind volgende treffer")
("find text" "vind tekst")
("finnish" "fins")
("first contact" "eerste kontakt")
("first indentation" "eerste inspringing")
("first item" "eerste element")
("first page" "eerste pagina")
("first start" "eerste start")
("flat" "plat")
("flexibility" "flexibiliteit")
("flexibility factor for vertical spacing" "vertikale flexibiliteitsfaktor")
("floating figure" "drijvend figuur")
("floating object" "drijvend objekt")
("floating table" "drijvende tabel")
("fold" "vouw")
("fold input field" "vouw invoerveld")
("folded" "gevouwd")
("font" "lettertype")
("font base size" "basis lettergrootte")
("font dpi" "punten per duim")
("font family" "familie lettertype")
("font name" "lettertype")
("font series" "lettersoort")
("font shape" "lettervorm")
("font size" "lettergrootte")
("font type" "lettertype")
("font variant" "variant lettertype")
("fonts" "lettertypes")
("footer" "voet")
("footnote" "voetnoot")
("for" "voor")
("foreground" "voorgrond")
("foreign" "buitenlands")
("format" "opmaak")
("formatted and structured output" "geformateerde en gestruktureerde uitvoer")
("formatted date" "datum met formaat")
("formatting" "formatering")
("formula" "formule")
("formula style" "formule stijl")
("forward replace" "vervang vooruit")
("forward search" "zoek vooruit")
("fraction" "breuk")
("french" "frans")
("full manuals" "hele handleidingen")
("function" "funktie")
("functional" "funktioneel")
("further customization" "verdere personalisatie")
("future changes" "toekomstige veranderingen")
("gNU TeXmacs comes without any form of legal warranty" "GNU TeXmacs komt zonder enige vorm van juridische garantie")
("gNU TeXmacs falls under the GNU general public license" "GNU TeXmacs valt onder de ``GNU general public license''")
("gcd" "ggd")
("general architecture of TeXmacs" "algemene opbouw van TeXmacs")
("general negation" "algemene ontkenning")
("general note" "algemene noot")
("generating font file" "Aanmaak van het lettertype")
("generic" "algemeen")
("georgian" "georgisch")
("german" "duits")
("getting started" "starten")
("glossary" "symbolenlijst")
("glossary entry" "in symbolenlijst")
("glue" "lijm")
("glue decorations" "plak versiering")
("go" "ga")
("gothic" "gotisch")
("graphical interface" "grafische interface")
("grave" "naar links")
("greater" "groter")
("greater or equal" "groter of gelijk")
("greek" "grieks")
("greek letter" "griekse letter")
("green" "groen")
("grey" "grijs")
("group" "groep")
("h modifier" "H veranderaar")
("happy typing" "veel plezier met het typen")
("has been aborted" "is gestopt")
("has completed its task" "heeft zijn taak volbracht")
("hat" "dakje")
("header" "kop")
("header and footer" "kop en voet")
("hebrew" "hebreeuws")
("height" "hoogte")
("height is exactly the specified height" "gebruik gespecifieerde hoogte")
("height is maximum of specified height and box height" "gebruik maximum van gespecifieerde hoogte en hoogte van inhoud")
("height is minimum of specified height and box height" "gebruik minimum van gespecifieerde hoogte en hoogte van inhoud")
("height magnification" "vergroting hoogte")
("help" "help")
("help me" "help mee")
("help us" "help ons")
("here" "hier")
("hidden" "verborgen")
("home" "thuis")
("homepage" "thuispagina")
("horizontal" "horizontaal")
("horizontal alignment" "horizontale plaatsing")
("horizontal cell alignment" "horizontale plaatsing in cel")
("horizontal ink separation" "horizontale inkt afstand")
("horizontal part" "horizontaal deel")
("horizontal position" "horizontale positie")
("horizontal space" "horizontale spatie")
("horizontal table alignment" "horizontale tabelplaatsing")
("horizontally align cell to the base" "plaats inhoud cel horizontaal via richtlijn")
("horizontally center the cell" "centreer inhoud cel horizontaal")
("how much longer may pages become" "hoeveel langer mogen paginas worden")
("how much shorter may pages become" "hoeveel korter mogen paginas worden")
("huge" "enorm")
("hungarian" "hongaars")
("hybrid command" "meervoudig kommando")
("hyperlink" "hyper-verwijzing")
("hyphenation" "regelafbraak")
("icelandic" "ijslands")
("if" "als")
("if hyphenated" "als afgebroken")
("image" "plaatje")
("image links" "verbonden plaatjes")
("implementation" "implementatie")
("import" "importeer")
("improving the current implementation" "de huidige implementatie verbeteren")
("in popup windows" "in apparte vensters")
("inactive" "niet aktief")
("include" "opnemen")
("include a document" "een document opnemen")
("include file" "bestand opnemen")
("inclusion into TeXmacs" "opname in TeXmacs")
("inclusions" "opgenomen documenten")
("indent" "spring in")
("indentation flag" "inspringen")
("index" "woordenlijst")
("index entry" "in woordenlijst")
("index level" "index niveau")
("informative flags" "informatieve vlaggetjes")
("inline arguments" "regel-argumenten")
("input mode" "invoermode")
("insert" "invoegen")
("insert Pr" "voeg Pr in")
("insert a TeXmacs symbol" "TeXmacs symbool invoegen")
("insert a big operator" "voeg een grote operator in")
("insert a binary operation" "voeg een binaire operatie in")
("insert a binary relation" "voeg een binaire relatie in")
("insert a blackboard bold character" "voeg een vette schoolbord letter in")
("insert a bold calligraphic character" "voeg een vette kalligrafische letter in")
("insert a bold character" "voeg een vette letter in")
("insert a calligraphic character" "voeg een kalligrafische letter in")
("insert a condition" "voeg een voorwaarde in")
("insert a control flow instruction" "voeg een kontrolestroom-instruktie in")
("insert a fraction" "voeg een breuk in")
("insert a fraktur character" "voeg een Gotische letter in")
("insert a greek character" "voeg een griekse letter in")
("insert a large left delimiter" "voeg een grote linkerhaak in")
("insert a large right delimiter" "voeg een grote rechterhaak in")
("insert a large separator" "voeg een grote afscheider in")
("insert a left subscript" "linker index invoegen")
("insert a link" "maak een verwijzing")
("insert a miscellaneous symbol" "voeg een ander symbool in")
("insert a new branch" "maak een nieuwe tak")
("insert a new item" "voeg een nieuw element in")
("insert a numerical operation" "voeg een numerieke bewerking in")
("insert a picture" "voeg een plaatje in")
("insert a square root" "voeg een wortel in")
("insert a table" "voeg een tabel in")
("insert a table or a matrix" "voeg een tabel of een matrix in")
("insert a textual operation" "voeg een tekstuele bewerking in")
("insert a wide symbol above" "voeg een breed symbool hierboven in")
("insert a wide symbol below" "voeg een breed symbool hieronder in")
("insert an" "maak een")
("insert an accent" "voeg een accent in")
("insert an arrow" "voeg een pijl in")
("insert an environment" "voeg een nieuwe omgeving in")
("insert an n" "maak een n")
("insert an operation on tuples" "voeg een bewerking op tuples in")
("insert appendix" "begin een nieuw aanhangsel")
("insert arccos" "voeg arccos in")
("insert arcsin" "voeg arcsin in")
("insert arctan" "voeg arctan in")
("insert arg" "voeg arg in")
("insert argument" "nieuw argument")
("insert author" "voeg auteur in")
("insert automatically generated content" "voeg automatisch gegenereerd materiaal in")
("insert big contour integral" "voeg een grote cirkelintegraal in")
("insert big coproduct" "voeg een groot coprodukt in")
("insert big direct sum" "voeg een grote direkte som in")
("insert big dotted point" "voeg een grote cirkel met een punt erin in")
("insert big integral" "voeg een grote integraal in")
("insert big intersection" "voeg een grote doorsnijding in")
("insert big logical and" "voeg een grote logische en in")
("insert big logical or" "voeg een grote logische of in")
("insert big product" "voeg een groot produkt in")
("insert big square union" "voeg een grote hoekige vereniging in")
("insert big summation" "voeg een grote sommatie in")
("insert big tensor product" "voeg een groot tensor produkt in")
("insert big union" "voeg een grote vereniging in")
("insert big union sum" "voeg een grote vereningingssom in")
("insert big vertical space" "voeg grote vertikale spatie in")
("insert chapter" "begin een nieuw hoofdstuk")
("insert column" "voeg kolom in")
("insert column to the left" "voeg kolom naar links in")
("insert column to the right" "voeg kolom naar rechts in")
("insert cos" "voeg cos in")
("insert cosh" "voeg cosh in")
("insert cot" "voeg cot in")
("insert coth" "voeg coth in")
("insert csc" "voeg csc in")
("insert deg" "voeg deg in")
("insert det" "voeg det in")
("insert dim" "voeg dim in")
("insert dollar" "voeg dollar in")
("insert equation" "nieuwe vergelijking")
("insert executable markup" "voeg uitvoerbaar materiaal in")
("insert exp" "voeg exp in")
("insert fields" "voeg velden in")
("insert footnote" "voeg voetnoot in")
("insert formula" "voeg formule in")
("insert gcd" "voeg gcd in")
("insert hom" "voeg hom in")
("insert horizontal interword space" "voeg horizontale spatie in")
("insert image" "voeg plaatje in")
("insert inactive markup" "voer inaktief materiaal in")
("insert inf" "voeg inf in")
("insert input field above" "voeg invoerveld hierboven in")
("insert input field below" "voeg invoerveld hieronder in")
("insert ker" "voeg ker in")
("insert lg" "voeg lg in")
("insert lim" "voeg lim in")
("insert liminf" "voeg liminf in")
("insert limsup" "voeg limsup in")
("insert ln" "voeg ln in")
("insert log" "voeg log in")
("insert macro" "voeg macro in")
("insert mathematics" "voeg wiskunde in")
("insert max" "voeg max in")
("insert medium horizontal space" "voeg gemiddelde horizontale spatie in")
("insert medium vertical space" "voeg gewone vertikale spatie in")
("insert min" "voeg min in")
("insert negative horizontal space" "voeg kleine negatieve spatie in")
("insert new item or number" "voeg nieuw element of nummer in")
("insert new rows or columns" "voeg nieuwe rijen of kolommen in")
("insert or delete rows or columns" "invoegen of wissen van rijen of kolommen")
("insert row" "voeg rij in")
("insert row above" "voeg rij naar boven in")
("insert row below" "voeg rij naar beneden in")
("insert sec" "voeg sec in")
("insert section" "begin een nieuwe sectie")
("insert sharp" "voeg hekje in")
("insert sin" "voeg sin in")
("insert sinh" "voeg sinh in")
("insert small horizontal space" "voeg kleine horizontale spatie in")
("insert small vertical space" "voeg kleine vertikale spatie in")
("insert structural markup" "voeg structuur in")
("insert subsection" "begin een nieuwe deelsectie")
("insert subsubsection" "begin een nieuwe deeldeelsectie")
("insert sup" "voeg sup in")
("insert tan" "voeg tan in")
("insert tanh" "voeg tanh in")
("insert text field" "voeg een tekstveld in")
("insert thick horizontal space" "voeg dikke horizontale spatie in")
("insert title" "voeg titel in")
("insert value" "voeg waarde in")
("integrate" "integreer")
("interactive command" "interaktief kommando")
("interactive questions" "interaktieve vragen")
("interfacing" "nieuwe interfaces")
("interfacing TeXmacs with other systems" "TeXmacs aan andere systemen koppelen")
("interjection" "tussenwerpsel")
("interline separation" "scheiding tussen regels")
("interline space" "afstand tussen regels")
("internationalization" "internationalisering")
("interparagraph space" "afstand tussen paragrafen")
("interrupt execution" "onderbreek executie")
("introduction" "inleiding")
("invert" "inverse")
("invisible" "onzichtbaar")
("invisible citation" "onzichtbare citatie")
("irish" "iers")
("is running" "is bezig")
("is tuple" "is tupel")
("ispell does not respond" "ispell geeft geen antwoord")
("ispell is already busy" "ispell is al bezig")
("italian" "italiaans")
("italic" "schuin")
("italic right" "scheef maar recht")
("itemize" "lijst maken")
("january" "januari")
("japanese" "japans")
("july" "juli")
("june" "juni")
("justified" "uitgelijnd")
("justify text" "lijn tekst uit")
("keep evaluated expressions" "geëvalueerde expressies aanhouden")
("key" "toets")
("keyboard" "toetsenbord")
("keyboard equivalent" "via toetsenbord")
("keyboard modifiers" "toetsenbordwijzigingen")
("keyboard shorthand" "toetsenbordafkorting")
("keymaps" "toetsenbord")
("keyword" "sleutelwoord")
("keywords" "sleutelwoorden")
("kill" "dood")
("known bugs" "bekende bugs")
("label" "etiket")
("landscape" "landschap")
("language" "taal")
("large" "groot")
("large closing bracket" "grote rechter haak")
("large delimiter" "grote haak")
("large opening bracket" "grote linker haak")
("large separator" "grote separator")
("last indentation" "laatste inspringing")
("last page" "laatste pagina")
("latex command" "latex kommando")
("latin" "latijns")
("layout" "indeling")
("lcm" "kgv")
("left" "links")
("left arrow" "pijl naar links")
("left border" "linker rand")
("left cell border" "linker celrand")
("left cell padding" "ruimte linkerkant cel")
("left margin" "linker rand")
("left padding" "ruimte links")
("left prime" "linker priem")
("left ragged" "links")
("left slanted" "schuin naar links")
("left subscript" "linker index")
("left superscript" "linker exponent")
("left surrounding" "linker omgeving")
("left table border" "linker tabelrand")
("left table padding" "ruimte linkerkant tabel")
("legal" "legaal")
("lemma" "hulpstelling")
("length" "lengte")
("less" "kleiner")
("less or equal" "kleiner of gelijk")
("letter" "brief")
("license" "licentie")
("light" "dun")
("light grey" "lichtgrijs")
("limitations" "gebreken")
("limits" "limieten")
("line" "lijn")
("line break" "breek lijn af")
("line stretch" "regel uitrekbaarheid")
("linear algebra" "lineaire algebra")
("link" "verband")
("link image" "verbind plaatje")
("list" "lijst")
("list of figures" "figurenlijst")
("list of tables" "tabellenlijst")
("load" "laad")
("load LaTeX file" "laad LaTeX bestand")
("load TeX file" "laad TeX bestand")
("load TeXmacs file" "laad TeXmacs bestand")
("load TeXmacs style file" "laad TeXmacs stijl bestand")
("load a file" "laad een bestand")
("load buffer" "laad buffer")
("load file" "laad bestand")
("load html file" "laad html bestand")
("load image" "laad plaatje")
("load in new window" "laad in nieuw venster")
("load latex file" "laad latex bestand")
("load more recent autosave file" "het nieuwere veiligheidsbestand laden")
("load tex file" "laad tex bestand")
("load verbatim file" "laad verbatim bestand")
("loaded" "geladen")
("logarithm" "logarithme")
("logarithms" "logarithmes")
("long" "lang")
("look and feel" "gebruikersinterface")
("look up" "zoek op")
("lower border" "benedenrand")
("m modifier" "M veranderaar")
("macro" "macro")
("magenta" "paars")
("magic" "magie")
("magnification" "vergrotingsfaktor")
("magnified" "uitvergroot")
("main" "hoofd")
("main icon bar" "voornaamste iconen")
("major changes" "belangrijkste veranderingen")
("make a page insertion" "maak een pagina invoeging")
("make a subscript" "maak een index")
("make a superscript" "maak een exponent")
("make acute" "maak een streepje naar rechts")
("make bar" "maak een streep")
("make check" "maak een check")
("make fraction" "maak een breuk")
("make grave" "maak een streepje naar links")
("make hat" "maak een hoedje")
("make label" "maak etiket")
("make negation" "maak een negatie")
("make page reference" "maak pagina verwijzing")
("make reference" "maak verwijzing")
("make square root" "maak een vierkantswortel")
("make tilde" "maak een slangetje")
("make title" "maak titel")
("make vector" "maak een vector")
("making donations" "ons een gift doen")
("manual" "handleiding")
("map to" "stuur naar")
("march" "maart")
("margins" "randen")
("margins as on paper" "randen net als op papier")
("master" "meester")
("master file" "meester van bestand")
("math" "wiskunde")
("math font" "wiskunde lettertype")
("math font family" "familie wiskundig lettertype")
("math font series" "wiskundige lettersoort")
("math font shape" "wiskundige lettervorm")
("mathematical" "wiskundig")
("mathematical and customized input" "wiskundige en gepersonaliseerde invoer")
("mathematical font" "wiskundig lettertype")
("mathematical formulas" "wiskundige formules")
("mathematical input" "wiskundige invoer")
("mathematical typesetting" "wiskundige pagina-opmaak")
("mathematics" "wiskunde")
("matrices" "matrices")
("matrix" "matrix")
("maximal" "maximaal")
("maximal cell height" "maximale hoogte van de cel")
("maximal cell width" "maximale breedte van de cel")
("maximal number of columns" "maximaal aantal kolommen")
("maximal number of rows" "maximaal aantal rijen")
("maximal table height" "maximale hoogte van de tabel")
("maximal table width" "maximale breedte van de tabel")
("maximum mode" "maximum mode")
("may" "mei")
("medium" "gewoon")
("medium skip" "gewone sprong")
("meminfo" "geheugen info")
("merge" "voeg samen")
("messy" "warrig")
("meta key" "meta toets")
("middle" "midden")
("minimal" "minimaal")
("minimal cell height" "minimale hoogte van de cel")
("minimal cell width" "minimale breedte van de cel")
("minimal number of columns" "minimaal aantal kolommen")
("minimal number of rows" "minimaal aantal rijen")
("minimal table height" "minimale hoogte van de tabel")
("minimal table width" "minimale breedte van de tabel")
("minimum mode" "minimum mode")
("minus" "strepen")
("miscellaneous" "divers")
("miscellaneous features" "diverse funktionaliteiten")
("mod" "mod")
("mode" "mode")
("modifier" "veranderaar")
("modify cell alignment" "plaatsing in cel")
("modify cell border" "verander rand cel")
("modify cell padding" "verander ruimte om cel")
("modify cell size" "verander grootte cel")
("modify table border" "rand tabel veranderen")
("modify table padding" "ruimte om tabel veranderen")
("mongolian" "mongools")
("more information about GNU TeXmacs can be found in the Help" "meer informatie over GNU TeXmacs kunt U vinden in het Help")
("mosaic" "mozaïek")
("move down" "ga naar beneden")
("move object" "verplaats object")
("move to the right when finished" "ga naar rechts zodra klaar")
("move up" "ga naar boven")
("multiline input" "meerdere regel invoer")
("mutator tags" "constant veranderende tags")
("my documents" "mijn dokumenten")
("n-th" "n-de machts")
("n-th root" "n-de machts wortel")
("name" "naam")
("negation" "ontkenning")
("never" "nooit")
("new" "nieuw")
("new buffer" "nieuwe buffer")
("new column" "nieuwe kolom")
("new document" "nieuw dokument")
("new double page" "nieuwe dubbele pagina")
("new double page before" "nieuwe dubbele pagina hiervoor")
("new item" "nieuw element")
("new line" "nieuwe lijn")
("new number" "nieuw nummer")
("new page" "nieuwe pagina")
("new page before" "nieuwe pagina hiervoor")
("new paragraph" "nieuwe paragraaf")
("new roman" "nieuwe roman")
("new row" "nieuwe rij")
("new table" "nieuwe tabel")
("next" "volgende")
("next line" "volgende regel")
("no changes need to be saved" "geen veranderingen hoeven te worden opgeslagen")
("no dictionary for" "geen woordenboek voor")
("no first indentation" "geen eerste inspringing")
("no indentation" "geen inspringing")
("no line break" "breek lijn niet af")
("no matches found" "ik heb geen treffers gevonden")
("no matches found for" "geen treffers gevonden voor")
("no more matches for" "niet meer treffers voor")
("no more redo information available" "niet meer informatie beschikbaar voor ``opnieuw''")
("no more undo information available" "niet meer informatie beschikbaar om terug te gaan")
("no name" "zonder naam")
("no page break after" "geen nieuwe pagina hierna")
("no page break before" "geen nieuwe pagina hiervoor")
("no spelling errors found in text" "geen spelfouten ontdekt in tekst")
("no style" "geen dokumentstijl")
("none" "geen")
("normal" "normaal")
("norwegian" "noors")
("not" "niet")
("not equal" "ongelijk")
("not found" "niet gevonden")
("notation" "notatie")
("note" "aantekening")
("november" "november")
("number" "getal")
("number equation" "nummer de vergelijking")
("number of columns" "aantal kolommen")
("numbered equation" "genummerde vergelijking")
("numbered equations" "genummerde vergelijkingen")
("numerator" "teller")
("numeric" "numeriek")
("objectives" "doel")
("occurrences" "treffers vervangen")
("october" "oktober")
("odd page footer" "oneven pagina voet")
("odd page header" "oneven pagina kop")
("odd page margin" "oneven pagina rand")
("odd page left margin" "oneven pagina linkerrand")
("odd page right margin" "oneven pagina rechterrand")
("odd page shift" "oneven pagina verschuiving")
("odd page text" "oneven pagina tekst")
("off" "uit")
("ok" "ok")
("old english" "oud engels")
("old slavonic" "oud slavisch")
("old style" "oude interface")
("on" "aan")
("on cells" "per cel")
("on columns" "per kolom")
("on entire table" "per hele tabel")
("on rows" "per rij")
("on the footer" "op de statusbalk")
("one spelling error has been corrected" "één spelfout is gekorrigeerd")
("online documentation" "on-line documentatie")
("online help" "hulp van het web")
("only inline tags" "alleen regel-tags")
("open" "open")
("open a new buffer" "open een nieuwe buffer")
("open new window" "nieuw venster openen")
("opening" "aanvang")
("operation mode" "operatiemode")
("operation on named selection" "operatie op aan te geven selektie")
("operation on primary selection" "operatie op hoofdselektie")
("operation on secondary selection" "operatie op tweede selektie")
("operation on ternary selection" "operatie op derde selektie")
("optical" "optisch")
("option" "optie")
("options" "opties")
("or" "of")
("orange" "oranje")
("orientation" "oriëntatie")
("origin" "oorsprong")
("original welcome message" "oorspronkelijk welkomsbericht")
("other" "anders")
("other pages" "ander paginas")
("other projects" "andere projecten")
("other style" "andere stijl")
("outlined roman" "omlijnd")
("over" "gedeeld door")
("overbrace" "accolade over")
("overline" "overstreept")
("padding" "extra ruimte")
("page" "pagina")
("page bottom margin" "pagina beneden rand")
("page break" "breek pagina af")
("page break before" "breek pagina hiervoor af")
("page height" "pagina lengte")
("page insertion" "pagina invoeging")
("page layout" "pagina indeling")
("page number text" "paginanummer tekst")
("page numbering" "paginanummering")
("page reference" "pagina verwijzing")
("page right margin" "pagina rechter rand")
("page setup" "paginainstelling")
("page top margin" "pagina boven rand")
("page type" "pagina type")
("page width" "pagina breedte")
("paper" "papier")
("paper type" "papierformaat")
("papyrus" "papyrus")
("paragraph" "paragraaf")
("paragraph mode" "paragraaf soort")
("paragraph width" "paragraaf breedte")
("parametric curve" "geparametriseerde kromme")
("parametric surface" "geparametriseerd oppervlak")
("part" "deel")
("past changes" "gemaakte veranderingen")
("paste" "plak")
("paste from" "plak uit")
("paste text" "plak tekst")
("path" "pad")
("perform operations on cells" "doe operaties celsgewijze")
("perform operations on columns" "doe operaties kolomsgewijze")
("perform operations on entire table" "doe operaties op hele tabel")
("perform operations on rows" "doe operaties op rijen")
("permanent" "permanent")
("person" "persoon")
("personal dictionary has been modified" "Uw persoonlijke woordenboek is veranderd")
("phantasy" "fantasie")
("philosophy" "filosofie")
("phoenician" "phoeniciaans")
("phonetic" "fonetisch")
("pink" "rose")
("plain block" "standaard blok")
("plain tabular" "standaard tabel")
("planned changes" "geplande veranderingen")
("plans for the future" "plannen voor de toekomst")
("please" "graag")
("please wait" "een ogenblik a.u.b.")
("plot" "grafiek")
("plus" "plus")
("polish" "pools")
("porting TeXmacs to other platforms" "TeXmacs overzetten naar andere systemen")
("portrait" "portret")
("portuguese" "portugees")
("position" "positie")
("position float" "positioneer drijver")
("position floating object" "positioneer drijvend objekt")
("position the table" "plaats de tabel")
("postscript" "postscript")
("postscript image" "postscript plaatje")
("pre-alpha release" "pré-alpha versie")
("preamble" "preambule")
("preamble mode" "preambule mode")
("preferences" "voorkeuren")
("presentation" "presentatie")
("presentation mode" "presentatiemode")
("presentation tag" "visuele weergave")
("preview before printing" "zien voor het printen")
("preview command" "kommando om vooraf te zien")
("preview with ghostview" "vooraf zien met ghostview")
("previous" "vorige")
("primary" "eerste")
("prime" "priem")
("print" "druk af")
("print all" "druk alles af")
("print all to file" "druk alles af naar bestand")
("print page selection" "druk gedeelte af")
("print page selection to file" "druk gedeelte af naar bestand")
("print to postscript file" "naar postscript bestand")
("printer" "printer")
("printer dpi" "printers resolutie in punten per duim")
("printing command" "kommando om af te drukken")
("problem" "probleem")
("professional" "professioneel")
("prog" "programma")
("prog font" "programmeer lettertype")
("prog font family" "familie programmeer lettertype")
("prog font series" "programma lettersoort")
("prog font shape" "programma lettervorm")
("program" "programma")
("program font" "lettertype voor programmas")
("program mode" "programma mode")
("programming" "programma")
("progressive" "geleidelijk")
("project" "projekt")
("projects" "projecten")
("prologue" "voorwoord")
("prompt on scripts" "vraag bij elke script")
("prompts and default input" "prompts en invoer-suggesties")
("proof" "bewijs")
("proportional" "proportioneel")
("proportional typewriter" "proportionele schrijfmachine")
("proposition" "propositie")
("query replace" "tekst op gecontroleerde wijze vervangen")
("quit" "verlaat")
("quotation" "citaat")
("quote" "citeren")
("range" "deel")
("rank" "rang")
("really close it" "echt sluiten")
("really huge" "echt enorm")
("really quit" "wilt u echt ophouden")
("really tiny" "werkelijk miniem")
("recover autosave file" "herstel veiligheidsbestand")
("red" "rood")
("redo" "opnieuw")
("redo undone changes" "maak ongedane veranderingen opnieuw geldig")
("reduce bottom margin by" "verklein onder rand met")
("reduce left margin by" "verklein linker rand met")
("reduce margins" "randen verkleinen")
("reduce right margin by" "verklein rechter rand met")
("reduce top margin by" "verklein boven rand met")
("reduce trigonometric functions" "trigonometrische funkties inkorten")
("reduction even page margin" "verkleining even pagina rand")
("reduction odd page margin" "verkleining oneven pagina rand")
("reduction page bottom margin" "verkleining pagina beneden rand")
("reduction page right margin" "verkleining pagina rechter rand")
("reduction page top margin" "verkleining pagina boven rand")
("reference" "verwijzing")
("reference guide" "referentie gids")
("reference guide for the TeXmacs primitives" "gids voor de TeXmacs primitieven")
("reference guide for the standard TeXmacs styles" "gids voor de standaard TeXmacs stijlen")
("references" "verwijzingen")
("regular" "normaal")
("release" "versie")
("reload" "herlaad")
("remark" "opmerking")
("remove" "verwijder")
("remove all output fields" "verwijder alle uitvoervelden")
("remove fields" "velden verwijderen")
("remove input field" "verwijder invoerveld")
("remove input field above" "verwijder invoerveld hierboven")
("remove number" "nummer verwijderen")
("remove package" "schrap pakket")
("remove this switch" "schakelaar verwijderen")
("renumber this page" "hernummer deze pagina")
("repeat" "herhaal")
("repeat object" "herhaal object")
("replace" "vervang")
("replace by" "vervang door")
("replace unrecognized styles" "vervang onbekende stijlen")
("replaced" "ik heb")
("replaced one occurrence" "ik heb één treffer vervangen")
("reset" "herzet")
("resize object" "verander grootte object")
("resized" "uitgerekt")
("results of the search for" "resultaten voor het zoeken naar")
("return" "return")
("revert" "herlaad")
("revert buffer" "herlaad")
("right" "rechts")
("right arrow" "pijl naar rechts")
("right border" "rechter rand")
("right cell border" "rechter celrand")
("right cell padding" "ruimte rechterkant cel")
("right margin" "rechter rand")
("right padding" "ruimte rechts")
("right ragged" "rechts")
("right subscript" "rechter index")
("right superscript" "rechter exponent")
("right surrounding" "rechter omgeving")
("right table border" "rechter tabelrand")
("right table padding" "ruimte rechterkant tabel")
("rigid" "stijf")
("rigid horizontal" "stijf horizontaal")
("rigid space" "stijve spatie")
("romanian" "roemeens")
("root" "wortel")
("rotate backward" "terug draaien")
("rotate forward" "vooruit draaien")
("row" "rij")
("row above" "rij naar boven")
("row below" "rij naar beneden")
("row contains glued items" "rij bevat aan elkaar gelijmde delen")
("rows" "rijen")
("runic" "runen")
("running author" "lopende auteur")
("running title" "lopende titel")
("russian" "russisch")
("russian and Ukrainian users" "Russische en Ukraeense gebruikers")
("sample" "proef")
("sans serif" "sans serif")
("save" "sla op")
("save TeXmacs file" "berg TeXmacs bestand op")
("save TeXmacs style file" "berg TeXmacs stijl bestand op")
("save as" "sla op als")
("save as LaTeX" "sla op als LaTeX")
("save as TeX" "sla op als TeX")
("save buffer" "berg buffer op")
("save buffer as" "berg buffer op als")
("save file" "berg bestand op")
("save html file" "berg html bestand op")
("save image" "berg plaatje op")
("save latex file" "berg latex bestand op")
("save tex file" "berg tex bestand op")
("save this buffer" "berg deze buffer op")
("save verbatim file" "berg verbatim bestand op")
("saved" "opgeborgen")
("scheme command" "scheme kommando")
("screen layout" "scherm indeling")
("screen margins" "randen op het scherm")
("script" "index")
("script above" "index boven")
("script below" "index onder")
("script script size" "onderindex grootte")
("script size" "index grootte")
("scripts not accepted" "scripts worden niet geaccepteerd")
("search" "zoek")
("search words in my documents" "zoek woorden in mijn dokumenten")
("search words in the documentation" "zoek woorden in de dokumentatie")
("search words in the source code" "zoek woordern in de programmacode")
("searching" "zoek")
("secondary" "tweede")
("section" "sectie")
("security" "veiligheid")
("see" "zie")
("select a document style" "selekteer een dokumentstijl")
("select a foreground color" "selekteer een voorgrondkleur")
("select a language" "selekteer een taal")
("selected region" "geselecteerde streek")
("selection" "selektie")
("selections" "selekties")
("seminar" "voordracht")
("sending commands to TeXmacs" "kommandos naar TeXmacs zenden")
("separation between lines" "aftand tussen regels")
("separation between paragraphs" "afstand tussen paragrafen")
("separator" "afscheider")
("september" "september")
("serb" "servisch")
("series" "soort")
("series|right" "rechtop")
("session" "sessie")
("session name" "naam van de sessie")
("session type" "type sessie")
("sessions" "sessies")
("set" "zet")
("set background color of cell" "achtergrondskleur cel")
("set cell mode" "verander celmode")
("set extension center" "centrum voor uitbreidingen")
("set height" "verander hoogte")
("set height of cell" "verander hoogte cel")
("set margins" "verander randen")
("set n" "voer n in")
("set or get environment variables" "verander of lees omgevingsvariabelen")
("set paragraph margins" "verander paragraaf randen")
("set paragraph mode" "verander paragraaf soort")
("set printer dpi" "stel printers resolutie in")
("set printing command" "kommando om af te drukken")
("set span" "verander rijkwijdte")
("set special cell properties" "speciale celeigenschappen")
("set special table properties" "speciale tabeleigenschappen")
("set text width" "verander tekst breedte")
("set width" "verander breedte")
("set width of cell" "verander breedte cel")
("settings" "instelling")
("setup printer" "stel printer in")
("shape" "vorm")
("shift-down" "shift-onder")
("shift-right" "shift-rechts")
("short" "kort")
("show" "zie")
("show header and footer" "kop en voet laten zien")
("show hidden" "zie verborgen gedeelte")
("show on screen" "zien op het scherm")
("shrinking factor" "krimpfaktor")
("signature" "handtekening")
("simplify" "versimpel")
("simplification" "versimpelen")
("simplified evaluation of formulas" "versimpeld evalueren van formules")
("simplified menus" "versimpelde menus")
("sine" "sinus")
("size" "grootte")
("size limits" "limieten aan grootte")
("size tag" "grootte")
("slanted" "scheef")
("sloppy" "slobberig")
("slovene" "sloveens")
("small" "klein")
("small caps" "kleine hoofdletters")
("small figure" "klein figuur")
("small skip" "kleine sprong")
("small table" "kleine tabel")
("source code" "programmacode")
("source tags" "weergave bomen")
("space" "spatie")
("space between lines" "afstand tussen regels")
("space between paragraphs" "afstand tussen paragrafen")
("spacing" "afstanden")
("spanish" "spaans")
("special" "speciaal")
("special cell properties" "speciale celeigenschappen")
("special functions" "speciale funkties")
("special properties" "speciale eigenschappen")
("special table properties" "speciale tabeleigenschappen")
("specific" "specifiek")
("specify the size of the table" "grootte van de tabel bepalen")
("spell" "spel")
("spell checking complete" "klaar met het doornemen van spelfouten")
("spelling error" "spelfout")
("spelling errors have been corrected" "spelfouten zijn gekorrigeerd")
("split" "splijt")
("split session" "splijt sessie")
("square root" "vierkantswortel")
("standard environment variables" "standaard omgevingsvariabelen")
("standard formats" "standaard formaten")
("standard TeXmacs styles" "standaard TeXmacs stijlen")
("start" "start")
("start a new chapter" "begin met een nieuw hoofdstuk")
("start a new section" "begin met een nieuwe sectie")
("start an interactive session" "start een interaktieve sessie")
("status bar" "status balk")
("stretch" "strek uit")
("stretchable" "uitrekbaar")
("stretchable horizontal" "horizontaal uitrekbaar")
("stretchable vertical" "vertikaal uitrekbaar")
("stretched" "uitgerekt")
("stroke" "streep")
("strokes" "strepen")
("strong" "belangrijk")
("structural" "struktureel")
("style" "stijl")
("styles" "stijlen")
("stylesheet language" "taal voor stijlbestanden")
("sub" "onder")
("subparagraph" "deelparagraaf")
("subscript" "index")
("subsection" "deelsectie")
("subsub" "onder onder")
("subsubsection" "deeldeelsectie")
("subtable" "deeltabel")
("subtitle" "ondertitel")
("summary" "samenvatting")
("summary of the principal TeXmacs tags" "de voornaamste TeXmacs tags")
("superpose" "opeenstapelen")
("superposed" "opeengestapeld")
("superscript" "exponent")
("surface" "oppervlak")
("swedish" "zweeds")
("swiss" "zwitsers")
("switch" "schakelaar")
("switch to first" "schakel naar eerste")
("switch to formula style" "gebruik formule stijl")
("switch to last" "schakel naar laatste")
("switch to next" "schakel naar volgende")
("switch to previous" "schakel naar vorige")
("switching and folding" "schakelen en vouwen")
("symbol" "symbool")
("system command" "systeem kommando")
("system environment variables" "primitieve omgevingsvariabelen")
("tab" "tabulering")
("table" "tabel")
("table command" "tabel kommando")
("table height" "hoogte van tabel")
("table of contents" "inhoudsopgave")
("table width" "breedte van tabel")
("tables" "tabellen")
("tabular" "tabuleer")
("tabular material" "tabellen")
("tangent" "tangens")
("teXmacs as an interface" "TeXmacs als koppelprogramma")
("teXmacs fonts" "TeXmacs lettertypes")
("teXmacs notice" "dank aan TeXmacs")
("teXmacs style files" "teXmacs stijlbestanden")
("ternary" "derde")
("test" "test")
("tex" "tex")
("text" "tekst")
("text font" "tekst lettertype")
("text height correction" "hoogtekorrektie van tekst")
("text width" "tekst breedte")
("texts" "teksten")
("th century bc" "de eeuw voor C.")
("th root" "de machts wortel")
("thai" "thais")
("the TeXmacs authors" "de makers van TeXmacs")
("the TeXmacs format" "het TeXmacs formaat")
("the application did not send its usual startup banner" "de toepassing heeft niet zijn gebruikelijke opstartmelding verzonden")
("the application does not reply" "de toepassing geeft geen antwoord")
("the application is already running" "de toepassing is al bezig")
("the boxes produced by the typesetter" "de door de letterzetter geproduceerde rechthoeken")
("the buffer has not been saved" "de buffer is niet bewaard")
("the graphical user interface" "de graphische gebruikersinterface")
("the leaves of TeXmacs trees" "de bladeren van TeXmacs bomen")
("the mycas example" "het voorbeeld 'mycas'")
("the primitive TeXmacs constructs" "pritimitieve TeXmacs constructies")
("the standard TeXmacs styles" "de standaard TeXmacs stijlen")
("theorem" "stelling")
("there are unsaved files" "niet alle bestanden zijn bewaard")
("this buffer has not been saved" "deze buffer is niet bewaard")
("this column" "deze kolom")
("this document has been produced using" "dit dokument is tot stand gekomen met behulp van")
("this line" "deze regel")
("this page" "deze pagina")
("this page footer" "voet voor deze pagina")
("this page header" "kop voor deze pagina")
("this row" "deze rij")
("thumbnails" "serie van plaatjes")
("tilda" "slangetje")
("tilde" "slangetje")
("times" "maal")
("tiny" "miniem")
("title" "titel")
("to the left" "naar links")
("to the right" "naar rechts")
("today" "vandaag")
("tools" "gereedschap")
("top" "top")
("top border" "bovenrand")
("top cell border" "bovenrand cel")
("top cell padding" "ruimte bovenkant cel")
("top level" "topniveau")
("top margin" "bovenrand")
("top padding" "ruimte boven")
("top table border" "bovenrand tabel")
("top table padding" "ruimte bovenkant tabel")
("translate" "vertaak")
("transpose" "transposeer")
("tree" "boom")
("triangularize" "trianguleer")
("trigonometric" "trigonometrisch")
("trigonometry" "trigonometrie")
("tuple" "tupel")
("turkish" "turks")
("tutorial" "leerboek")
("type" "type")
("type a latex command followed by" "typ een latex kommando in, gevolgd door")
("type body" "typ lichaam")
("type denominator" "typ noemer")
("type numerator" "typ teller")
("type script" "typ index")
("typesetted boxes" "geletterzette rechthoeken")
("typewriter" "schrijfmachine")
("typewriter small caps" "schrijfmachine kleine hoofdletters")
("typing a simple text" "simpele tekst intypen")
("typing simple texts" "simpele tekst intypen")
("ugaritic" "ugaritisch")
("ukrainian" "ukraeens")
("underbrace" "accolade onder")
("underline" "onderstreept")
("undo" "terug")
("undo last changes" "maak laatste veranderingen ongedaan")
("up" "omhoog")
("update" "actualiseer")
("update from web" "haal nieuwe dokumentatie van het web")
("update image links" "aktualiseer verbonden plaatjes")
("upper border" "bovenrand")
("upright" "rechtop")
("use MathML" "gebruik MathML")
("use TeXmacs" "gebruik TeXmacs")
("use a black color" "gebruik zwarte kleur")
("use a blue color" "gebruik blauwe kleur")
("use a bold font" "gebruik een vet lettertype")
("use a brown color" "gebruik bruine kleur")
("use a calligraphic font" "gebruik een kalligrafisch lettertype")
("use a green color" "gebruik groene kleur")
("use a grey color" "gebruik grey kleur")
("use a magenta color" "gebruik paarse kleur")
("use a orange color" "gebruik oranje kleur")
("use a pink color" "gebruik roze kleur")
("use a red color" "gebruik rode kleur")
("use a sans serif font" "gebruik een lettertype zonder serifen")
("use a typewriter font" "gebruik een lettertype van een schrijfmachine")
("use a white color" "gebruik witte kleur")
("use a yellow color" "gebruik gele kleur")
("use blackboard bold font" "gebruik schoolbord letters")
("use bold font series" "gebruik vette letters")
("use calligraphical font" "gebruik kalligrafische letters")
("use catcode definitions in preamble" "gebruik catcodes in preambule")
("use footnote font size" "gebruik erg kleine letters")
("use fraktur font" "gebruik Gotische letters")
("use huge font size" "gebruik enorm grote letters")
("use italic font shape" "gebruik shuine letters")
("use large font size" "gebruik grote letters")
("use left hand column as border" "gebruik kolom links als rand")
("use medium font series" "gebruik gewone letterdikte")
("use normal font size" "gebruik normale lettergrootte")
("use package" "gebruik pakket")
("use paragraph width" "gebruik paragraafbreedte")
("use really huge font size" "gebruik echt enorme letters")
("use right font shape" "gebruik rechte letters")
("use right hand column as border" "gebruik kolom rechts als rand")
("use roman font family" "gebruik roman lettertype")
("use row above as border" "gebruik rij hierboven als rand")
("use row below as border" "gebruik rij hierbeneden als rand")
("use sans serif font family" "gebruik sans serif lettertype")
("use script font size" "gebruik erg kleine letters")
("use slanted font shape" "gebruik scheve letters")
("use small capitals" "gebruik kleine hoofdletters")
("use small font size" "gebruik kleine letters")
("use small-caps font shape" "gebruik kleine hoofdletters")
("use the blackboard bold font" "gebruik het vette schoolbord lettertype")
("use the fraktur font" "gebruig het fraktur lettertype")
("use tiny font font size" "gebruik minieme letters")
("use typewriter font family" "gebruik schrijfmachine lettertype")
("use very large font size" "gebruik erg grote letters")
("user manual" "gebruikersboek")
("user provided icons" "gebruikers gedefiniëerde iconen")
("users of Cyrillic languages" "gebruikers van Cyrillische talen")
("value" "waarde")
("variable" "variabele")
("variant" "variant")
("vector" "vektor")
("verbatim" "woordelijk")
("verse" "vers")
("vertical" "vertikaal")
("vertical after" "vertikaal achteraan")
("vertical alignment" "vertikale plaatsing")
("vertical before" "vertikaal vooraan")
("vertical cell alignment" "vertikale plaatsing in cel")
("vertical part" "vertikaal deel")
("vertical position" "vertikale positie")
("vertical space" "vertikale spatie")
("vertical table alignment" "vertikale plaatsing tabel")
("vertically align cell to the base" "plaats inhoud cel vertikaal via richtlijn")
("vertically center the cell" "centreer inhoud cel vertikaal")
("very large" "erg groot")
("very small" "erg klein")
("vietnamese" "vietnamees")
("view" "beeld")
("visible" "zichtbaar")
("warning" "waarschuwing")
("web" "web")
("welcome" "welkom")
("welcome to GNU TeXmacs" "welkom tot GNU TeXmacs")
("what is new" "nieuwigheden")
("when finished" "zodra klaar")
("while" "zolang")
("white" "wit")
("wide accent" "breed accent")
("wide bar" "brede streep")
("width" "breedte")
("width is exactly the specified width" "gebruik gespecifieerde breedte")
("width is maximum of specified width and box width" "gebruik maximum van gespecifieerde breedte en breedte van inhoud")
("width is minimum of specified width and box width" "gebruik minimum van gespecifieerde breedte en breedte van inhoud")
("width magnification" "vergroting breedte")
("window" "venster")
("window height" "venster hoogte")
("window width" "venster breedte")
("windows key" "windows toets")
("with" "met")
("wrap lines" "regels afbreken")
("write a macro" "schrijf een macro")
("write a name" "schrijf een naam")
("write bold mathematics" "schrijf vette wiskunde")
("write bold text" "schrijf vette tekst")
("write italic text" "schrijf schuine tekst")
("write sample text" "schrijf voorbeeld tekst")
("write strong text" "schrijf belangrijke tekst")
("write typewriter text" "schrijf typmachine tekst")
("write verbatim text" "schrijf woordelijke tekst")
("writing data converters" "schrijf data-uitwisselaars")
("writing your own style files" "scrijf uw eigen stijlbestanden")
("written with TeXmacs" "geschreven met TeXmacs")
("yellow" "geel")
("you can also obtain" "u kunt")
("your document is back in its original state" "Uw document is terug in zijn originele toestand")
("your first interface" "uw eerste interface")
("your language" "uw taal")
