("a modifier" "")
("a series" "")
("abbreviation" "大略")
("about" "關於")
("about menu" "關於功能表")
("above" "以上")
("abstract" "大綱")
("accent" "")
("accept" "")
("accept all scripts" "")
("accept no scripts" "")
("accept script" "")
("acknowledgment" "感謝辭")
("acronym" "")
("action" "")
("activate" "")
("activate once" "")
("activate hidden" "")
("activation" "")
("acute" "")
("add package" "增加程式庫")
("add switch after" "")
("add switch before" "")
("address" "住址")
("address block" "")
("advanced" "")
("advanced layout features" "")
("after" "之後")
("albanian" "")
("algorithm" "演算法")
("align cell to the bottom" "")
("align cell to the left" "")
("align cell to the right" "")
("align cell to the top" "")
("align table at base of left column" "表格對齊左邊")
("align table at base of middle column" "表格對齊中間")
("align table at base of origin column" "")
("align table at base of right column" "表格對齊右邊")
("align table at fraction bar height" "")
("align table at the base of bottom row" "")
("align table at the base of middle row" "")
("align table at the base of origin row" "")
("align table at the base of top row" "")
("align table at the bottom" "")
("align table at the center" "")
("align table at the horizontal center" "")
("align table at the left" "")
("align table at the top" "")
("align table to the right" "")
("align text to the left" "")
("align text to the right" "")
("aligned" "")
("alignment" "整列")
("all" "")
("allow" "")
("allowed extension" "")
("allowed reduction" "")
("alpha release" "")
("alt key" "alt 鍵")
("alternative specification" "")
("american" "美語")
("animation" "動畫")
("and" "")
("angular" "")
("appendix" "附錄")
("apply" "使用")
("apply macro" "")
("apply macro once" "")
("april" "")
("arab" "")
("arch series" "")
("archaic" "")
("architecture" "")
("argument" "")
("arithmetic" "")
("armenian" "")
("arrow" "")
("arrows" "")
("article" "論文")
("as on paper" "")
("assign" "")
("at fraction bar height" "")
("at the baseline" "")
("at the bottom" "")
("at the left" "")
("at the middle" "")
("at the right" "")
("at the top" "")
("attach" "")
("attach buffer to which project" "")
("august" "")
("author" "作者")
("authorize" "")
("automatic" "自動")
("automatically close brackets" "")
("automatically generated content" "")
("automatic quotes" "")
("autosave" "自動保存")
("axis" "")
("axiom" "公理")
("b series" "")
("background" "")
("background color" "")
("background evaluations" "")
("backward replace" "")
("backward search" "")
("bad data format" "")
("bar" "")
("bard" "")
("base level" "")
("bashkirian" "")
("basic" "")
("basic communication using pipes" "")
("basic data types" "基本資料型態")
("basic types" "基本型態")
("be useful" "")
("become a TeXmacs developer" "成為TeXmacs的開發者")
("before" "")
("begin" "")
("below" "")
("beta release" "β版")
("bibliography" "參考文獻")
("bibliography file" "參考文獻檔案")
("bibliography style" "參考文獻的格式檔")
("big" "大")
("big figure" "大圖")
("big operator" "大的運算符號")
("big skip" "大的間隔")
("big table" "大表格")
("binary operation" "2項運算")
("binary relation" "2項運算")
("black" "黑色")
("blackboard bold" "粗黑體")
("blackboard math" "")
("blackletter" "")
("blue" "藍色")
("body" "")
("bold" "粗黑")
("book" "書籍")
("border" "")
("both" "")
("bottom" "")
("bottom border" "")
("bottom cell border" "")
("bottom cell padding" "")
("bottom margin" "")
("bottom padding" "")
("bottom table border" "")
("bottom table padding" "")
("branch" "分支")
("break" "中斷")
("breaking" "")
("breve" "")
("british" "英語")
("broken white" "")
("brown" "")
("browse" "瀏覽")
("browse web" "")
("browser" "瀏覽程式")
("buffer" "")
("bulgarian" "保加力亞文")
("bullet" "")
("bullets" "")
("by" "")
("by typing" "")
("bytes free" "0 bytes ")
("bytes used" "使用 bytes 總數")
("calligraphic" "")
("cancel" "取消")
("caps-lock key" "")
("case" "")
("cc" "")
("cell" "")
("cell background color" "")
("cell border" "")
("cell height" "")
("cell operation mode" "")
("cell width" "")
("center" "中心")
("center text" "原文中間")
("centered" "")
("centered block" "")
("centered tabular" "")
("change border of cell" "")
("change cell operation mode" "")
("change log" "")
("changes" "")
("chapter" "章")
("check" "檢查")
("check text for spelling errors" "檢查拼字")
("chinese" "中文")
("choice" "選擇")
("citation" "引用")
("cite" "")
("class" "")
("clear" "清除")
("clear selection" "")
("clipped" "")
("clipped and magnified" "")
("clipped and resized" "")
("clone" "")
("clone window" "一樣的視窗")
("close" "結束")
("close TeXmacs" "結束 TeXmacs")
("close buffer" "結束暫存區")
("close session" "結束工作")
("close window" "關閉視窗")
("closing" "正在關閉中")
("closing style" "")
("code" "程式碼")
("color" "顏色")
("column" "列")
("column contains glued items" "")
("common formats" "")
("compact" "")
("compactification" "")
("compactify" "")
("compatibility with other formats" "")
("compile article" "文章編譯")
("compile book" "書籍編譯")
("compile buffer" "")
("compile project" "")
("completion" "完成")
("complex" "")
("concrete" "")
("condensed" "")
("condition" "")
("configuration" "設定")
("contact" "")
("content" "內容")
("content tag" "內容標籤")
("context dependent icons" "內容相關圖示")
("continuation of" "延續")
("contribute" "參與")
("control the evaluation of expressions" "式子表現的控制")
("convention" "")
("converters to other data formats" "轉換成其它格式")
("copy" "複製")
("copy text" "複製內文")
("copy to" "複製到")
("copying" "複製中")
("corollary" "系")
("correct text" "")
("create web site" "產生網站")
("cursors" "游標")
("custom tab" "")
("customizing TeXmacs" "調整TeXmacs ")
("customizing the TeXmacs styles" "調整TeXmacs格式")
("custum tab" "")
("cut" "剪下")
("cut text" "剪下內容")
("cut to" "剪下內容到")
("cypriot" "")
("cyrillic" "")
("cyrillic input method" "")
("cyrillic italic right" "")
("czech" "捷克語")
("dancers" "")
("danish" "丹麥語")
("dark grey" "")
("dash" "")
("dashes" "")
("data conversion" "")
("data format" "")
("date" "日期")
("deactivate" "")
("deactivate table" "")
("december" "")
("decorate atoms" "")
("default" "既定")
("default padding" "")
("definition" "定義")
("delete" "刪除")
("delete column" "刪除一行")
("delete row" "正在刪除中")
("denominator" "分母")
("description" "描述")
("destination" "目的")
("detach" "")
("detailed" "詳細")
("determinant" "行列式")
("devangari" "")
("developers guide" "開發者導引")
("directory" "目錄")
("disable" "取消")
("disable first indentation" "")
("disable first indentation after" "")
("disable first indentation before" "")
("disable indentation" "")
("disable indentation after" "")
("disable indentation before" "")
("disactivate" "")
("disactivate once" "")
("disallow" "")
("distribute unused space" "")
("div" "")
("do not remap" "")
("document" "文書")
("document format" "文書格式")
("document style" "文書")
("document styles" "文書格式")
("documentation" "")
("documents are trees" "樹狀文書")
("dot" "")
("dots" "")
("double stroke" "")
("down" "")
("download" "下載")
("dpi" "")
("duplicate" "")
("dutch" "荷蘭語")
("dynamic libraries" "")
("edit" "編輯")
("edit source tree" "編輯樹狀內容")
("editing tools" "編輯工具")
("email" "電子郵件")
("emphasize" "強調")
("emphasize text" "強調性內容")
("empty box" "空白的區域")
("enable first indentation" "")
("enable first indentation after" "")
("enable first indentation before" "")
("enable indentation" "")
("enable indentation after" "")
("enable indentation before" "")
("encl" "")
("end" "")
("english" "英語")
("enter a title" "輸入標題")
("enumerate" "有序列舉")
("enumerate|Alpha" "")
("enumerate|Roman" "")
("enumerate|alpha" "")
("enumerate|roman" "")
("env" "")
("environment" "環境")
("epilogue" "")
("equal" "等於")
("equation" "方程式")
("equations" "方程式群")
("equivalent for" "")
("error" "錯誤")
("esperanto" "")
("etruscan" "")
("evaluate" "評價")
("evaluate scheme expression" "scheme表現評價")
("even page footer" "偶數頁數註腳")
("even page header" "偶數頁數標題")
("even page margin" "")
("even page left margin" "")
("even page shift" "")
("even page text" "")
("exact horizontal" "")
("exact mode" "")
("exam" "試卷")
("example" "例")
("exclusive" "排他的")
("exclusive or" "")
("executable" "可執行")
("execute" "執行")
("execute a TeXmacs command" "執行 TeXmacs 指令")
("execute system command" "執行系統指令")
("executive" "")
("exercise" "練習")
("expand" "展開")
("explained" "")
("export" "轉換格式輸出")
("export as" "轉換成")
("extend or normal" "")
("extern " "")
("extra" "")
("extract format" "")
("family" "")
("february" "")
("figure" "圖")
("file" "檔案")
("file name" "檔名")
("file not found" "無此檔案")
("find next match" "尋找下一筆資料")
("find text" "尋找內容")
("finnish" "芬蘭語")
("first contact" "")
("first indentation" "首次對齊位置")
("first item" "")
("first page" "")
("first start" "開始第一次")
("flat" "")
("flexibility" "彈性")
("flexibility factor for vertical spacing" "")
("floating figure" "浮動的圖形")
("floating object" "浮動的物件")
("floating table" "浮動的表格")
("fold" "折疊")
("fold input field" "")
("folded" "可折疊的")
("font" "字體")
("font base size" "字體基本大小")
("font dpi" "")
("font family" "")
("font name" "")
("font series" "")
("font shape" "")
("font size" "")
("font type" "")
("font variant" "")
("fonts" "")
("footer" "")
("footnote" "注釋")
("for" "")
("foreground" "")
("foreign" "")
("format" "格式")
("formatted and structured output" "")
("formatted date" "")
("formatting" "格式中")
("formula" "公式")
("formula style" "")
("forward replace" "")
("forward search" "")
("fraction" "分數")
("french" "法文")
("full manuals" "完全手冊")
("full screen mode" "全螢幕模式")
("function" "")
("functional" "")
("further customization" "")
("future changes" "")
("gNU TeXmacs comes without any form of legal warranty" "GNU TeXmacs無任何保證")
("gNU TeXmacs falls under the GNU general public license" "GNU TeXmacs 使用 GNU 
general public license 授權.")
("general architecture of TeXmacs" "")
("general negation" "")
("general note" "")
("generating font file" "字型檔案製作中")
("generic" "一般")
("georgian" "")
("german" "德文")
("getting started" "")
("glossary" "詞彙")
("glossary entry" "")
("glue" "")
("glue decorations" "")
("go" "移到")
("gothic" "")
("graphical interface" "")
("grave" "")
("greater" "")
("greater or equal" "")
("greek" "")
("greek letter" "")
("green" "")
("grey" "")
("group" "")
("h modifier" "")
("happy typing" "")
("has been aborted" "")
("has completed its task" "")
("hat" "")
("header" "標題")
("header and footer" "標題與註釋")
("hebrew" "")
("height" "高度")
("height is exactly the specified height" "")
("height is maximum of specified height and box height" "")
("height is minimum of specified height and box height" "")
("height magnification" "")
("help" "說明")
("help me" "")
("help us" "請幫助我們")
("here" "")
("hidden" "")
("home" "")
("homepage" "首頁")
("horizontal" "")
("horizontal cell alignment" "")
("horizontal ink separation" "")
("horizontal part" "")
("horizontal position" "")
("horizontal space" "")
("horizontal table alignment" "")
("horizontally align cell to the base" "")
("horizontally center the cell" "")
("how much longer may pages become" "")
("how much shorter may pages become" "")
("huge" "")
("hungarian" "匈牙利語")
("hybrid command" "")
("hyperlink" "")
("hyphenation" "語法")
("icelandic" "")
("if" "")
("image" "圖")
("image links" "")
("implementation" "")
("import" "輸入其它格式檔案")
("improving the current implementation" "")
("inactive" "")
("include" "")
("include a document" "")
("include file" "")
("inclusion into TeXmacs" "")
("inclusions" "")
("indent" "")
("indentation flag" "對齊標籤")
("index" "索引")
("index entry" "")
("index level" "")
("informative flags" "")
("inline arguments" "")
("input mode" "")
("insert" "輸入")
("insert Pr" "")
("insert a TeXmacs symbol" "")
("insert a big operator" "")
("insert a binary operation" "")
("insert a binary relation" "")
("insert a blackboard bold character" "")
("insert a bold calligraphic character" "")
("insert a bold character" "")
("insert a calligraphic character" "")
("insert a condition" "")
("insert a control flow instruction" "")
("insert a fraction" "")
("insert a fraktur character" "")
("insert a greek character" "")
("insert a large left delimiter" "")
("insert a large right delimiter" "")
("insert a large separator" "")
("insert a left subscript" "")
("insert a link" "")
("insert a miscellaneous symbol" "")
("insert a new branch" "")
("insert a new item" "")
("insert a numerical operation" "")
("insert a picture" "")
("insert a square root" "")
("insert a table" "")
("insert a table or a matrix" "")
("insert a textual operation" "")
("insert a wide symbol above" "")
("insert a wide symbol below" "")
("insert an" "")
("insert an accent" "")
("insert an arrow" "")
("insert an environment" "")
("insert an n" "")
("insert an operation on tuples" "")
("insert appendix" "")
("insert arccos" "")
("insert arcsin" "")
("insert arctan" "")
("insert arg" "")
("insert argument" "")
("insert author" "")
("insert automatically generated content" "")
("insert big contour integral" "")
("insert big coproduct" "")
("insert big direct sum" "")
("insert big dotted point" "")
("insert big integral" "")
("insert big intersection" "")
("insert big logical and" "")
("insert big logical or" "")
("insert big product" "")
("insert big square union" "")
("insert big summation" "")
("insert big tensor product" "")
("insert big union" "")
("insert big union sum" "")
("insert big vertical space" "")
("insert chapter" "")
("insert column" "")
("insert column to the left" "")
("insert column to the right" "")
("insert cos" "")
("insert cosh" "")
("insert cot" "")
("insert coth" "")
("insert csc" "")
("insert deg" "")
("insert det" "")
("insert dim" "")
("insert dollar" "")
("insert equation" "")
("insert executable markup" "")
("insert exp" "")
("insert fields" "")
("insert footnote" "")
("insert formula" "")
("insert gcd" "")
("insert hom" "")
("insert horizontal interword space" "")
("insert image" "")
("insert inactive markup" "")
("insert inf" "")
("insert input field above" "")
("insert input field below" "")
("insert ker" "")
("insert lg" "")
("insert lim" "")
("insert liminf" "")
("insert limsup" "")
("insert ln" "")
("insert log" "")
("insert macro" "")
("insert mathematics" "")
("insert max" "")
("insert medium horizontal space" "")
("insert medium vertical space" "")
("insert min" "")
("insert negative horizontal space" "")
("insert new item or number" "")
("insert new rows or columns" "")
("insert or delete rows or columns" "")
("insert row" "")
("insert row above" "")
("insert row below" "")
("insert sec" "")
("insert section" "輸入節(section) ")
("insert sharp" "")
("insert sin" "")
("insert sinh" "")
("insert small horizontal space" "")
("insert small vertical space" "")
("insert structural markup" "")
("insert subsection" "")
("insert subsubsection" "")
("insert sup" "")
("insert tan" "")
("insert tanh" "")
("insert text field" "")
("insert thick horizontal space" "")
("insert title" "")
("insert value" "")
("interactive command" "")
("interfacing" "介面")
("interfacing TeXmacs with other systems" "")
("interjection" "")
("interline separation" "")
("interline space" "")
("internationalization" "")
("interparagraph space" "")
("interrupt execution" "")
("introduction" "")
("invisible" "")
("invisible citation" "")
("irish" "")
("is running" "")
("is tuple" "")
("ispell does not respond" "")
("ispell is already busy" "")
("italian" "意大利語")
("italic" "")
("italic right" "")
("itemize" "無序列舉")
("january" "")
("japanese" "日文")
("july" "")
("june" "")
("justified" "")
("justify text" "")
("key" "")
("keyboard" "鍵盤")
("keyboard equivalent" "")
("keyboard modifiers" "")
("keyboard shorthand" "")
("keymaps" "")
("keyword" "關鍵字")
("keywords" "關鍵語")
("kill" "")
("known bugs" "")
("label" "標籤 (lebel)")
("landscape" "")
("language" "言語")
("large" "")
("large closing bracket" "")
("large delimiter" "")
("large opening bracket" "")
("large separator" "")
("last page" "")
("latex command" "")
("latin" "")
("layout" "")
("left" "")
("left arrow" "")
("left border" "左端")
("left cell border" "")
("left cell padding" "")
("left margin" "左端空白")
("left padding" "")
("left prime" "")
("left ragged" "")
("left slanted" "")
("left subscript" "")
("left superscript" "")
("left surrounding" "")
("left table border" "")
("left table padding" "")
("legal" "")
("lemma" "引理")
("length" "")
("less" "")
("less or equal" "")
("letter" "書信")
("license" "")
("light" "")
("light grey" "")
("limitations" "")
("limits" "")
("line" "")
("line break" "")
("line stretch" "")
("link" "連結")
("link image" "")
("list" "")
("list of figures" "圖形目錄")
("list of tables" "表格目錄")
("load" "開啟")
("load LaTeX file" "")
("load TeX file" "")
("load TeXmacs file" "")
("load TeXmacs style file" "")
("load a file" "")
("load buffer" "")
("load file" "")
("load html file" "")
("load image" "")
("load in new window" "")
("load latex file" "")
("load more recent autosave file" "開啟最近自動保存的檔案")
("load tex file" "開啟 TeX 檔案")
("load verbatim file" "")
("loaded" "")
("long" "")
("look and feel" "")
("look up" "")
("lower border" "下端")
("m modifier" "")
("macro" "巨集命令")
("magenta" "")
("magic" "")
("magnification" "放大比率")
("magnified" "")
("main" "")
("main icon bar" "主選單")
("major changes" "")
("make a page insertion" "")
("make a subscript" "")
("make a superscript" "")
("make acute" "")
("make bar" "")
("make check" "")
("make fraction" "")
("make grave" "")
("make hat" "")
("make label" "")
("make negation" "")
("make page reference" "")
("make reference" "")
("make square root" "")
("make tilde" "")
("make title" "表題製作")
("make vector" "")
("making donations" "")
("manual" "說明書")
("map to" "")
("march" "")
("margins" "空白端")
("margins as on paper" "")
("master" "")
("master file" "")
("math" "")
("math font" "")
("math font family" "")
("math font series" "")
("math font shape" "")
("mathematical" "")
("mathematical and customized input" "")
("mathematical font" "")
("mathematical formulas" "")
("mathematical input" "")
("mathematical typesetting" "")
("mathematics" "數學")
("matrix" "行列")
("maximal" "")
("maximal number of columns" "")
("maximal number of rows" "")
("maximum mode" "")
("may" "")
("medium" "")
("medium skip" "")
("meminfo" "")
("merge" "")
("messy" "")
("meta key" "")
("middle" "")
("minimal" "")
("minimal number of columns" "")
("minimal number of rows" "")
("minimum mode" "")
("minus" "")
("miscellaneous" "")
("miscellaneous features" "")
("mod" "")
("mode" "")
("modifier" "")
("modify cell alignment" "")
("modify cell border" "")
("modify cell padding" "")
("modify cell size" "")
("modify table border" "")
("modify table padding" "")
("mongolian" "")
("more information about GNU TeXmacs can be found in the Help" "進一步有關GNU 
TeXmacs 的資料怎可參考說明")
("mosaic" "")
("move down" "")
("move object" "")
("move to the right when finished" "")
("move up" "")
("multiline input" "")
("mutator tags" "")
("my documents" "")
("n-th" "")
("n-th root" "")
("name" "字型檔名")
("negation" "")
("new" "新增")
("new buffer" "")
("new column" "新增一行")
("new double page" "")
("new double page before" "")
("new item" "")
("new line" "新增一行")
("new number" "")
("new page" "新增一頁")
("new page before" "")
("new paragraph" "")
("new roman" "")
("new row" "")
("new table" "")
("next" "下一個")
("next line" "")
("no changes need to be saved" "沒有任何變更需要保存")
("no dictionary for" "")
("no first indentation" "")
("no indentation" "")
("no line break" "")
("no matches found" "")
("no matches found for" "")
("no more matches for" "")
("no more redo information available" "")
("no more undo information available" "")
("no name" "未命名檔案")
("no page break after" "")
("no page break before" "")
("no spelling errors found in text" "")
("no style" "")
("none" "")
("normal" "")
("norwegian" "挪威文")
("not" "")
("not equal" "")
("not found" "")
("notation" "記法")
("note" "")
("november" "")
("number" "")
("number of columns" "列的數目")
("number equation" "")
("numbered equation" "")
("numbered equations" "")
("numerator" "分子")
("numeric" "")
("objectives" "")
("occurrences" "")
("october" "")
("odd page footer" "")
("odd page header" "")
("odd page margin" "")
("odd page left margin" "")
("odd page right margin" "")
("odd page shift" "")
("odd page text" "")
("off" "關")
("ok" "")
("old english" "")
("old slavonic" "")
("old style" "")
("on" "")
("on cells" "")
("on columns" "")
("on entire table" "")
("on rows" "")
("one spelling error has been corrected" "")
("online documentation" "")
("online help" "線上說明")
("only inline tags" "")
("open" "")
("open a new buffer" "")
("open new window" "新開視窗")
("opening" "")
("operation on named selection" "")
("operation on primary selection" "")
("operation on secondary selection" "")
("operation on ternary selection" "")
("optical" "")
("option" "")
("options" "")
("or" "")
("orange" "")
("orientation" "")
("origin" "")
("original welcome message" "")
("other" "其它")
("other pages" "")
("other projects" "")
("other style" "")
("outlined roman" "")
("over" "")
("overbrace" "")
("overline" "")
("padding" "")
("page" "頁")
("page bottom margin" "")
("page break" "")
("page break before" "")
("page height" "")
("page insertion" "新增頁")
("page layout" "")
("page number text" "")
("page numbering" "頁數")
("page reference" "")
("page right margin" "")
("page setup" "設定頁面")
("page top margin" "頁面上縁")
("page type" "頁面種類")
("page width" "頁面寬長")
("paper" "")
("paper type" "")
("papyrus" "")
("paragraph" "段落")
("paragraph mode" "")
("paragraph width" "")
("part" "部分")
("past changes" "")
("paste" "貼上")
("paste from" "從貼過來")
("paste text" "")
("path" "")
("perform operations on cells" "")
("perform operations on columns" "")
("perform operations on entire table" "")
("perform operations on rows" "")
("permanent" "")
("person" "")
("personal dictionary has been modified" "")
("phantasy" "")
("philosophy" "")
("phoenician" "")
("phonetic" "")
("pink" "")
("plain block" "")
("plain tabular" "")
("planned changes" "")
("plans for the future" "")
("please" "")
("please wait" "請稍後")
("plugin" "")
("plugins" "")
("plus" "")
("polish" "波蘭語")
("porting TeXmacs to other platforms" "")
("portrait" "")
("portuguese" "葡萄牙語")
("position" "")
("position float" "")
("position floating object" "")
("position the table" "")
("postscript" "")
("postscript image" "")
("pre-alpha release" "")
("preamble" "")
("preamble mode" "")
("preferences" "設定(preferences)")
("presentation" "演講")
("presentation mode" "演講模式")
("presentation tag" "演講標籤")
("preview before printing" "")
("preview command" "")
("preview with ghostview" "")
("previous" "")
("primary" "")
("prime" "")
("print" "列印")
("print all" "全部列印")
("print all to file" "全部印成檔案")
("print page selection" "選擇列印")
("print page selection to file" "")
("print to postscript file" "印成 postscript 檔案")
("printer" "印表機")
("printer dpi" "")
("printing command" "")
("problem" "問題")
("professional" "")
("prog" "")
("prog font" "")
("prog font family" "")
("prog font series" "")
("prog font shape" "")
("program" "")
("program font" "")
("program mode" "")
("programming" "")
("progressive" "")
("project" "專案")
("projects" "專案")
("prologue" "")
("prompt on scripts" "")
("prompts and default input" "")
("proof" "證明")
("proportional" "")
("proportional typewriter" "")
("proposition" "命題")
("query replace" "")
("quit" "")
("quotation" "引用")
("quote" "")
("range" "")
("really close it" "")
("really huge" "")
("really quit" "")
("recover autosave file" "復元自動保存檔案")
("red" "")
("redo" "恢復")
("redo undone changes" "")
("reduce bottom margin by" "")
("reduce left margin by" "")
("reduce margins" "")
("reduce right margin by" "")
("reduce top margin by" "")
("reduction even page margin" "")
("reduction odd page margin" "")
("reduction page bottom margin" "")
("reduction page right margin" "")
("reduction page top margin" "")
("reference" "參考")
("reference guide" "參考導引")
("reference guide for the TeXmacs primitives" "")
("reference guide for the standard TeXmacs styles" "")
("references" "參考")
("regular" "")
("release" "")
("reload" "重新載入")
("remark" "注意")
("remove all output fields" "")
("remove fields" "")
("remove input field" "")
("remove input field above" "")
("remove number" "")
("remove package" "取消程式庫")
("remove this switch" "")
("renumber this page" "")
("repeat" "")
("repeat object" "")
("replace" "取代")
("replace by" "被取代")
("replaced" "")
("replaced one occurrence" "")
("reset" "")
("resize object" "")
("resized" "")
("results of the search for" "")
("return" "")
("revert" "反向")
("revert buffer" "")
("right" "")
("right arrow" "")
("right border" "右端")
("right cell border" "")
("right cell padding" "")
("right margin" "右端")
("right padding" "")
("right ragged" "")
("right subscript" "")
("right superscript" "")
("right surrounding" "")
("right table border" "")
("right table padding" "")
("rigid" "")
("rigid horizontal" "")
("rigid space" "")
("romanian" "羅馬尼亞語")
("root" "根")
("rotate backward" "")
("rotate forward" "")
("row" "")
("row contains glued items" "")
("runic" "")
("running author" "")
("running title" "")
("russian" "蘇俄語")
("sample" "")
("sans serif" "")
("save" "存檔")
("save TeXmacs file" "存成 TeXmacs 檔")
("save TeXmacs style file" "存成 TeXmacs 格式檔")
("save as" "另存新檔")
("save as LaTeX" "另存為 LaTeX 檔案")
("save as TeX" "另存為 TeX 檔案")
("save buffer" "儲存暫存檔")
("save buffer as" "另存暫存檔為")
("save file" "儲存檔案")
("save html file" "存為 html 檔案")
("save image" "儲存圖形檔")
("save latex file" "儲存 LaTeX 檔案")
("save tex file" "儲存TeX檔案")
("save this buffer" "儲存暫存區資料")
("save verbatim file" "存為  verbatim 檔案")
("saved" "儲存完成")
("scheme command" "")
("scheme extensions" "scheme 擴充")
("screen layout" "")
("screen margins" "")
("script" "直譯程式")
("scripts" "直譯程式")
("script above" "")
("script below" "")
("script script size" "")
("script size" "")
("scripts not accepted" "")
("search" "搜尋")
("search words in my documents" "")
("search words in the documentation" "")
("search words in the source code" "")
("searching" "")
("secondary" "")
("section" "節")
("security" "")
("see" "")
("select a document style" "")
("select a foreground color" "")
("select a language" "")
("selected region" "")
("selection" "")
("selections" "選擇")
("seminar" "會議")
("sending commands to TeXmacs" "")
("separation between lines" "")
("separation between paragraphs" "")
("separator" "")
("september" "")
("serb" "")
("series" "種類")
("series|right" "")
("session" "行程")
("session name" "")
("session type" "")
("sessions" "行程")
("set" "")
("set background color of cell" "")
("set cell mode" "")
("set extension center" "")
("set height" "")
("set height of cell" "")
("set margins" "")
("set n" "")
("set or get environment variables" "")
("set paragraph margins" "")
("set paragraph mode" "")
("set printer dpi" "")
("set printing command" "")
("set span" "")
("set special cell properties" "")
("set special table properties" "")
("set text width" "")
("set width" "")
("set width of cell" "")
("settings" "")
("setup printer" "")
("shape" "型類")
("shift-down" "")
("shift-right" "")
("short" "")
("show" "")
("show header and footer" "")
("show hidden" "")
("show on screen" "")
("shrinking factor" "縮小比率")
("signature" "")
("size" "大小")
("size tag" "大小標籤")
("size limits" "")
("slanted" "")
("sloppy" "")
("slovene" "")
("small" "")
("small caps" "")
("small figure" "小圖")
("small skip" "")
("small table" "")
("source code" "內容碼")
("source tags" "")
("space" "空白")
("space between lines" "")
("space between paragraphs" "")
("spacing" "空白作成法")
("spanish" "西班牙語")
("special" "特定")
("special cell properties" "")
("special table properties" "")
("specific" "指定")
("spell" "拼字")
("spell checking complete" "拼字檢查完成")
("spelling error" "拼字錯誤")
("spelling errors have been corrected" "拼字錯誤更正完成")
("split" "")
("split session" "")
("square root" "平方根")
("standard environment variables" "")
("standard formats" "")
("standard TeXmacs styles" "")
("start" "開始")
("start a new chapter" "")
("start a new section" "")
("start an interactive session" "")
("status bar" "狀態選欄")
("stretch" "")
("stretchable" "")
("stretchable horizontal" "")
("stretchable vertical" "")
("stretched" "")
("stroke" "")
("strokes" "")
("strong" "")
("structural" "")
("style" "風格")
("styles" "")
("stylesheet language" "")
("sub" "")
("subparagraph" "小段落")
("subscript" "下添字")
("subsection" "小節(sub)")
("subsub" "")
("subsubsection" "小小節(subsub)")
("subtitle" "")
("summary" "")
("summary of the principal TeXmacs tags" "")
("superpose" "")
("superposed" "")
("superscript" "上添字")
("swedish" "瑞典語")
("swiss" "")
("switch" "")
("switch to first" "")
("switch to formula style" "")
("switch to last" "")
("switch to next" "")
("switch to previous" "")
("switching and folding" "")
("symbol" "")
("system command" "")
("system environment variables" "")
("tab" "")
("table" "表")
("table command" "")
("table of contents" "目次")
("tables" "")
("tabular" "")
("tabular material" "")
("taiwanese" "台灣話")
("teXmacs as an interface" "")
("teXmacs fonts" "")
("teXmacs notice" "")
("teXmacs plugins" "")
("teXmacs style files" "")
("ternary" "")
("test" "撿測")
("tex" "")
("text" "文書")
("text font" "")
("text height correction" "")
("text width" "")
("texts" "")
("th century bc" "")
("th root" "")
("thai" "")
("the TeXmacs authors" "")
("the TeXmacs format" "")
("the application did not send its usual startup banner" "")
("the application does not reply" "")
("the application is already running" "")
("the boxes produced by the typesetter" "")
("the buffer has not been saved" "暫存區資料尚未儲存")
("the graphical user interface" "")
("the leaves of TeXmacs trees" "")
("the mycas example" "")
("the primitive TeXmacs constructs" "")
("the standard TeXmacs styles" "")
("theorem" "定理")
("there are unsaved files" "尚有檔案尚未儲存保存")
("this buffer has not been saved" "現工作的暫存區資料尚未儲存")
("this document has been produced using" "此文件由製作")
("this line" "")
("this page" "")
("this page footer" "")
("this page header" "")
("tilda" "")
("tilde" "")
("times" "")
("tiny" "")
("title" "標題")
("to the left" "")
("to the right" "")
("today" "本日")
("tools" "工具")
("top" "")
("top border" "")
("top cell border" "")
("top cell padding" "")
("top level" "")
("top margin" "")
("top padding" "")
("top table border" "")
("top table padding" "")
("translate" "")
("tree" "")
("tuple" "")
("turkish" "土耳其文")
("tutorial" "入門說明")
("type" "")
("type a latex command followed by" "")
("type body" "")
("type denominator" "")
("type numerator" "")
("type script" "")
("typesetted boxes" "")
("typewriter" "")
("typewriter small caps" "")
("typing a simple text" "")
("typing simple texts" "")
("ugaritic" "")
("ukrainian" "烏克蘭語")
("underbrace" "")
("underline" "")
("undo" "放棄(undo)")
("undo last changes" "")
("up" "")
("update" "更新")
("update from web" "")
("update image links" "")
("upper border" "上端")
("upright" "")
("use TeXmacs" "")
("use a black color" "")
("use a blue color" "")
("use a bold font" "")
("use a brown color" "")
("use a calligraphic font" "")
("use a green color" "")
("use a grey color" "")
("use a magenta color" "")
("use a orange color" "")
("use a pink color" "")
("use a red color" "")
("use a sans serif font" "")
("use a typewriter font" "")
("use a white color" "")
("use a yellow color" "")
("use blackboard bold font" "")
("use bold font series" "")
("use calligraphical font" "")
("use footnote font size" "")
("use fraktur font" "")
("use huge font size" "")
("use italic font shape" "")
("use large font size" "")
("use left hand column as border" "")
("use medium font series" "")
("use normal font size" "")
("use package" "")
("use paragraph width" "")
("use really huge font size" "")
("use right font shape" "")
("use right hand column as border" "")
("use roman font family" "")
("use row above as border" "")
("use row below as border" "")
("use sans serif font family" "")
("use script font size" "")
("use slanted font shape" "")
("use small capitals" "")
("use small font size" "")
("use small-caps font shape" "")
("use the blackboard bold font" "")
("use the fraktur font" "")
("use tiny font font size" "")
("use typewriter font family" "")
("use very large font size" "")
("user manual" "")
("user provided icons" "使用者提供圖示")
("users of Cyrillic languages" "")
("value" "")
("variable" "")
("variant" "種類")
("vector" "")
("verbatim" "")
("verse" "韻文")
("vertical" "")
("vertical after" "")
("vertical before" "")
("vertical cell alignment" "")
("vertical part" "")
("vertical position" "")
("vertical space" "")
("vertical table alignment" "")
("vertically align cell to the base" "")
("vertically center the cell" "")
("very large" "")
("very small" "")
("vietnamese" "")
("view" "檢視")
("visible" "")
("warning" "警告")
("web" "網路")
("welcome" "歡迎")
("welcome to GNU TeXmacs" "歡迎使用GNU TeXmacs")
("what is new" "")
("when finished" "")
("while" "")
("white" "")
("wide accent" "")
("wide bar" "")
("width" "寬")
("width is exactly the specified width" "")
("width is maximum of specified width and box width" "")
("width is minimum of specified width and box width" "")
("width magnification" "")
("window" "")
("window height" "")
("window width" "")
("windows key" "")
("with" "")
("write a macro" "")
("write a name" "")
("write bold mathematics" "")
("write bold text" "")
("write italic text" "")
("write sample text" "")
("write strong text" "")
("write typewriter text" "")
("write verbatim text" "")
("writing data converters" "")
("writing your own style files" "")
("written with TeXmacs" "使用 TeXmacs 製作")
("yellow" "")
("you can also obtain" "")
("your document is back in its original state" "")
("your first interface" "")
("your language" "使用語言")

