("A modifier" "modificatore A")
("GNU" "GNU")
("H modifier" "modificatore H")
("LaTeX" "LaTeX")
("M modifier" "modificare M")
("TeX" "TeX")
("abbreviation" "abbreviazione")
("about" "informazioni su TeXmacs")
("about menu" "menų informazioni")
("above" "sopra")
("abstract" "riassunto")
("accent" "accento")
("accept" "accetta")
("accept all scripts" "accetta tutti gli script")
("accept no scripts" "non accettare gli scripts")
("accept script" "accetta script")
("acknowledgment" "riconoscimento")
("acronym" "acronimo")
("action" "azione")
("activate" "attiva")
("acute" "acuto")
("add switch after" "aggiungi switch dopo")
("add switch before" "aggiungi switch prima")
("address" "indirizzo")
("address block" "blocco indirizzo")
("advanced" "avanzato")
("advanced layout features" "caratteristiche di impaginazione avanzate")
("after" "dopo")
("albanian" "albanese")
("algorithm" "algoritmo")
("align cell to the bottom" "allinea la cella sul fondo")
("align cell to the left" "allinea la cella a sinistra")
("align cell to the right" "allinea la cella a destra")
("align cell to the top" "alinea la cella in alto")
("align table at base of left column" "allinea la tabella alla base della colonna sinistra")
("align table at base of middle column" "allinea la tabella alla base della colonna centrale")
("align table at base of origin column" "allinea la tabella alla base della colonna d'origine")
("align table at base of right column" "allinea la tabella alla base della colonna di destra")
("align table at fraction bar height" "allinea la tabella all'altezza della linea di frazione")
("align table at the base of bottom row" "allinea la tabella alla base della riga inferiore")
("align table at the base of middle row" "allinea la tabella alla base della riga centrale")
("align table at the base of origin row" "allinea la tabella alla base della riga d'origine")
("align table at the base of top row" "allinea la tabella alla base della riga superioree")
("align table at the bottom" "allinea la tabella al fondo")
("align table at the center" "allinea la tabella al centro")
("align table at the horizontal center" "allinea la tabella al centro orizzontale")
("align table at the left" "allinea la tabella a sinistra")
("align table at the top" "allinea la tabella in alto")
("align table to the right" "allinea la tabella a destra")
("align text to the left" "allinea il testo a sinistra")
("align text to the right" "allinea il testo a destra")
("alignment" "allineamento")
("all" "tutto")
("allow" "permetti")
("allowed extension" "estensione permessa")
("allowed reduction" "riduzione permessa")
("alpha release" "versione alfa")
("alt key" "tasto Alt")
("and" "e")
("appendix" "appendice")
("apply" "applica")
("april" "aprile")
("arab" "arabo")
("archaic" "arcaico")
("architecture" "architettura")
("argument" "argomento")
("arithmetic" "aritmetica")
("armenian" "armeno")
("arrow" "freccia")
("arrows" "freccie")
("article" "articolo")
("as on paper" "come sul foglio")
("assign" "assegna")
("at fraction bar height" "all'altezza della linea di frazione")
("at the baseline" "in linea con la base")
("at the bottom" "sul fondo")
("at the left" "a sinistra")
("at the middle" "al centro")
("at the right" "a destra")
("at the top" "in alto")
("attach" "collega")
("attach buffer to which project" "collega il buffer a quale progetto")
("august" "agosto")
("author" "autore")
("authorize" "autorizza")
("automatic" "automatico")
("automatically generated content" "contenuto generato in modo automatico")
("axiom" "assioma")
("background" "sfondo")
("background color" "colore di sfondo")
("backward replace" "sostituisci all'indietro")
("backward search" "cerca all'indietro")
("bad data format" "formato dei dati errato")
("bar" "barra")
("bard" "bardo")
("base level" "livello della base")
("bashkirian" "bashkirian")
("basic" "fondamentale")
("basic data types" "tipi di dato base")
("basic types" "tipi fondamentali")
("be useful" "rendersi utile")
("become a TeXmacs developer" "diventare uno sviluppatore TeXmacs")
("before" "prima")
("begin" "inizio")
("below" "sotto")
("beta release" "versione beta")
("bibliography" "bibliografia")
("bibliography file" "file della bibliografia")
("bibliography style" "stile della bibliografia")
("big" "grande")
("big figure" "figura grande")
("big operator" "operatore grande")
("big skip" "salto grande")
("big table" "tabella grande")
("binary operation" "operazione binaria")
("binary relation" "relazione binaria")
("black" "nero")
("blackboard bold" "grassetta da lavagna")
("blackboard math" "matematica da lavagna")
("blackletter" "blackletter")
("blue" "blu")
("body" "corpo")
("bold" "grassetto")
("book" "libro")
("border" "bordo")
("both" "entrambi")
("bottom" "fondo")
("bottom border" "bordo inferiore")
("bottom margin" "margine inferiore")
("bottom padding" "spaziatura inferiore")
("branch" "ramo")
("break" "salto")
("breaking" "salti")
("breve" "breve")
("british" "britannico")
("brown" "marrone")
("browse" "naviga")
("browse web" "naviga sul web")
("browser" "browser")
("buffer" "buffer")
("bulgarian" "bulgaro")
("bullet" "puntina")
("bullets" "puntine")
("by" "di")
("by typing" "scrivendo")
("bytes free" "byte liberi")
("bytes used" "byte usati")
("calligraphic" "calligrafico")
("cancel" "cancella")
("caps-lock key" "tasto Caps-lock")
("case" "caso")
("cc" "p.c.")
("cell" "cella")
("cell background color" "colore di sfondo della cella")
("cell border" "bordo della cella")
("cell height" "altezza della cella")
("cell operation mode" "modo operativo della cella")
("cell width" "larghezza della cella")
("center" "centro")
("center text" "centra il testo")
("centered" "centrato")
("centered block" "blocco centrato")
("centered tabular" "tabella centrata")
("change border of cell" "imposta il bordo della cella")
("change cell operation mode" "cambia il modo operativo della cella")
("change log" "change log")
("changes" "cambiamenti")
("chapter" "capitolo")
("check" "controlla")
("check text for spelling errors" "controllo ortografico")
("chinese" "cinese")
("choice" "scelta")
("citation" "citazione")
("cite" "citazione")
("class" "classe")
("clear" "elimina")
("clear selection" "elimina selezione")
("clipped" "ancorato")
("clipped and magnified" "ancorato e ampliato")
("clipped and resized" "ancorato e ridimensionato")
("clone" "duplica")
("clone window" "duplica la finestra")
("close" "chiudi")
("close TeXmacs" "chiudi TeXmacs")
("close buffer" "chiudi il buffer")
("close session" "chiudi la sessione")
("close window" "chiudi la finestra")
("closing" "chiusura")
("code" "codice")
("color" "colore")
("column" "colonna")
("column contains glued items" "la colonna contiene elementi appiccicati")
("compatibility with other formats" "compatibilitā con altri formati")
("compile article" "compila articolo")
("compile book" "compila libro")
("compile buffer" "compila il buffer")
("compile project" "compila il progetto")
("complex" "complesso")
("concrete" "concreto")
("condensed" "condensato")
("condition" "condizione")
("configuration" "configurazione")
("content tag" "content tag")
("context dependent icons" "icone dipendenti dal contesto")
("continuation of" "continuazione di")
("convention" "convenzione")
("converters to other data formats" "convertitori per altri formati")
("copy" "copia")
("copy text" "copia testo")
("copy to" "copia in")
("copying" "copiando")
("corollary" "corollario")
("correct text" "correggi il testo")
("cursors" "cursori")
("custom tab" "tabulazione personalizzata")
("customizing TeXmacs" "personalizzare TeXmacs")
("customizing the TeXmacs styles" "personalizzare gli stili di TeXmacs")
("custum tab" "tabulazione personalizzata")
("cut" "taglia")
("cut text" "taglia testo")
("cut to" "taglia in")
("cypriot" "cipriota")
("cyrillic" "cirillico")
("cyrillic input method" "metodo di input cirillico")
("cyrillic italic right" "cirillico italico a destra")
("czech" "ceco")
("dancers" "ballerini")
("danish" "danese")
("dash" "trattino")
("dashes" "trattini")
("data conversion" "conversione di dati")
("data format" "formato dei dati")
("date" "data")
("deactivate" "disattiva")
("deactivate table" "disattiva tabella")
("december" "dicembre")
("decorate atoms" "decora atomi")
("default" "predefinito")
("default padding" "spaziatura predefinita")
("definition" "definizione")
("delete" "cancella")
("delete column" "cancella colonna")
("delete row" "cancella riga")
("denominator" "denominatore")
("description" "descrizione")
("destination" "destinazione")
("detach" "scollega")
("determinant" "determinante")
("devangari" "devangari")
("developers guide" "guida per gli sviluppatori")
("directory" "directory")
("disable first indentation" "disabilita prima indentazione")
("disable first indentation after" "disabilita prima indentazione successiva")
("disable first indentation before" "disabilita prima indentazione precedente")
("disable indentation" "disabilita indentazione")
("disable indentation after" "disabilita indentazione successiva")
("disable indentation before" "disabilita indentazione precedente")
("disable last indentation" "disabilita l'ultima indentazione")
("disable last indentation after" "disabilita l'ultima indentazione successiva")
("disable last indentation before" "disabilita l'ultima indentazione precedente")
("disallow" "proibisci")
("distribute unused space" "distribuisci spazio inutilizzato")
("div" "div")
("document" "documento")
("document format" "formato del documento")
("document style" "stile del documento")
("document styles" "stili di documento")
("documentation" "documentazione")
("documents are trees" "i documenti sono alberi")
("dot" "punto")
("dots" "puntini")
("double stroke" "tratto doppio")
("down" "sotto")
("dpi" "dpi")
("duplicate" "duplica")
("dutch" "olandese")
("dynamic libraries" "librerie dinamiche")
("edit" "modifica")
("editing tools" "strumenti di editing")
("email" "email")
("emphasize" "enfatizza")
("emphasize text" "scrivi testo enfatizzato")
("empty box" "box vuoto")
("enable first indentation" "abilita prima indentazione")
("enable first indentation after" "abilita prima indentazione successiva")
("enable first indentation before" "abilita prima indentazione precedente")
("enable indentation" "abilita indentazione")
("enable indentation after" "abilita indentazione precedente")
("enable indentation before" "abilita indentazione successiva")
("enable last indentation" "abilita l'ultima indentazione")
("enable last indentation after" "abilita l'ultima indentazione successiva")
("enable last indentation before" "abilita l'ultima indentazione precedente")
("encl" "encl")
("end" "fine")
("english" "inglese")
("enter a title" "inserisci un titolo")
("enumerate" "lista numerata")
("enumerate|Alpha" "numerazione|Alfa")
("enumerate|Roman" "numerazione|Romano")
("enumerate|alpha" "numerazione|alfa")
("enumerate|roman" "numerazione|romano")
("env" "env")
("environment" "ambiente")
("epilogue" "epilogo")
("equal" "uguale")
("equation" "equazione")
("equations" "equazioni")
("equivalent for" "equivalente per")
("error" "errore")
("esperanto" "esperanto")
("etruscan" "etrusco")
("evaluate" "valuta")
("evaluate scheme expression" "valuta espressione scheme")
("even page footer" "pič di pagina (pagine pari)")
("even page header" "intestazione (pagine pari)")
("even page margin" "margine pagine pari")
("even page text" "testo (pagine pari)")
("exact horizontal" "orizzontale esatta")
("exact mode" "modo esatto")
("exam" "esame")
("example" "esempio")
("exclusive" "esclusivo")
("exclusive or" "o esclusivo")
("executable" "eseguibile")
("execute" "esegui")
("execute a TeXmacs command" "esegui un comando di TeXmacs")
("execute system command" "esegui un comando di sistema")
("executive" "esecutivo")
("exercise" "esercizio")
("expand" "espandi")
("explained" "spiegato")
("export" "esporta")
("export as" "esporta come")
("extend or normal" "esteso o normale")
("extern " "esterno")
("extra" "extra")
("extract format" "estrai formato")
("family" "famiglia")
("february" "febbraio")
("figure" "figura")
("file" "file")
("file name" "nome del file")
("file not found" "file non trovato")
("find next match" "trova il prossimo match")
("find text" "trova testo")
("finnish" "finlandese")
("first contact" "primo contatto")
("first indentation" "prima indentazione")
("first item" "primo elemento")
("first page" "prima pagina")
("first start" "primo avvio")
("flat" "piatto")
("flexibility" "flessibilitā")
("flexibility factor for vertical spacing" "fattore di flessibilitā per la spaziatura verticale")
("floating figure" "figura fluttuante")
("floating object" "oggetto fluttuante")
("floating table" "tabella fluttuante")
("fold" "arrotola")
("folded" "arrotolato")
("font" "carattere")
("font base size" "dimensione di base del carattere")
("font dpi" "risoluzione del carattere (dpi)")
("font family" "famiglia del carattere")
("font name" "nome del carattere")
("font series" "serie del carattere")
("font shape" "forma del carattere")
("font size" "dimensione del carattere")
("font variant" "variante del carattere")
("fonts" "tipi di carattere")
("footer" "pič di pagina")
("footnote" "nota a pič di pagina")
("for" "per")
("foreground" "primo piano")
("foreign" "straniero")
("format" "formato")
("formatted date" "data formattata")
("formula" "formula")
("formula style" "stile della formula")
("forward replace" "sostituisci in avanti")
("forward search" "cerca in avanti")
("fraction" "frazione")
("french" "francese")
("full manuals" "manuali completi")
("function" "funzione")
("further customization" "ulteriore personalizzazione")
("future changes" "cambiamenti futuri")
("gNU TeXmacs comes without any form of legal warranty" "gNU TeXmacs viene distribuito senza alcuna forma di garanzia legale")
("gNU TeXmacs falls under the GNU general public license" "gNU TeXmacs č distribuito sotto i termini della GNU general public license (GNU GPL)")
("general architecture of TeXmacs" "architettura generale di TeXmacs")
("general negation" "negazione generale")
("generating font file" "sto generando il file del font")
("generic" "generico")
("georgian" "georgiano")
("german" "tedesco")
("getting started" "primi passi")
("glossary" "glossario")
("glossary entry" "voce del glossario")
("glue" "appiccica")
("glue decorations" "appiccica le decorazioni")
("go" "vai")
("gothic" "gotico")
("graphical interface" "interfaccia grafica")
("grave" "grave")
("greater" "maggiore")
("greater or equal" "maggiore o uguale")
("greek" "greco")
("greek letter" "lettera greca")
("green" "verde")
("grey" "grigio")
("group" "gruppo")
("happy typing" "buona scrittura")
("has been aborted" "č stato abortito")
("has completed its task" "ha completato il suo compito")
("hat" "cappello")
("header" "intestazione")
("header and footer" "intestazione e pič di pagina")
("hebrew" "ebreo")
("height" "altezza")
("height is exactly the specified height" "l'altezza č esattamente quella specificata")
("height is maximum of specified height and box height" "l'altezza č la massima tra l'altezza specificata e l'altezza del box")
("height is minimum of specified height and box height" "l'altezza č la minima tra l'altezza specificata e l'altezza del box")
("height magnification" "moltiplicatore dell'altezza")
("help" "aiuto")
("help me" "aiutami")
("help us" "aiutaci")
("here" "qui")
("home" "inizio")
("horizontal" "orizzontale")
("horizontal cell alignment" "allineamento orizzontale della cella")
("horizontal ink separation" "separazione orizzontale dell'inchiostro")
("horizontal part" "parte orizzontale")
("horizontal position" "posizione orizzontale")
("horizontal space" "spazio orizzontale")
("horizontal table alignment" "allineamento orizzontale della tabella")
("horizontally align cell to the base" "allinea la cella orizzontalmente con la base")
("horizontally center the cell" "centra la cella orizzontalmente")
("how much longer may pages become" "quanto pių lunghe potrebbero diventare le pagine")
("how much shorter may pages become" "quanto pių corte potrebbero diventare le pagine")
("huge" "enorme")
("hungarian" "ungherese")
("hyperlink" "collegamento ipertestuale")
("hyphenation" "ritorno a capo")
("icelandic" "islandese")
("if" "se")
("image" "immagine")
("image links" "inserisci un'immagine (collegamento)")
("implementation" "implementazione")
("import" "importa")
("improving the current implementation" "migliorare l'implementazione attuale")
("inactive" "inattivo")
("include" "includi")
("include a document" "includi un documento")
("include a file" "includi un file")
("include file" "includi file")
("inclusion into TeXmacs" "inclusione in TeXmacs")
("inclusions" "inclusioni")
("indent" "indenta")
("indentation flag" "simbolo di indentazione")
("index" "indice analitico")
("index entry" "voce dell'indice analitico")
("index level" "livello dell'indice analitico")
("insert" "inserisci")
("insert Pr" "inserisci Pr")
("insert a TeXmacs symbol" "inserisci un simbolo di TeXmacs")
("insert a big operator" "inserisci un operatore grande")
("insert a binary operation" "inserisci un'operazione binaria")
("insert a binary relation" "inserisci una relazione binaria")
("insert a blackboard bold character" "inserisci un carattere lavagna grassetto")
("insert a bold calligraphic character" "inserisci un carattere calligrafico grassetto")
("insert a bold character" "inserisci un carattere grassetto")
("insert a calligraphic character" "inserisci un carattere calligrafico")
("insert a fraction" "inserisci una frazione")
("insert a fraktur character" "inserisci un carattere fraktur")
("insert a greek character" "inserisci un carattere greco")
("insert a large left delimiter" "inserisci un delimitatore sinistro grande")
("insert a large right delimiter" "inserisci un delimitatore destro grande")
("insert a large separator" "inserisci un separatore grande")
("insert a left subscript" "inserisci pedice sinistro")
("insert a link" "inserisci un collegamento")
("insert a miscellaneous symbol" "inserisci un simbolo (altro...)")
("insert a new branch" "inserisci un nuovo ramo")
("insert a new item" "inserisci un nuovo elemento")
("insert a picture" "inserisci un'immagine")
("insert a square root" "inserisci una radice quadrata")
("insert a table" "inserisci una tabella")
("insert a table or a matrix" "inserisci una tabella o una matrice")
("insert an" "inserisci un/a")
("insert an accent" "inserisci un accento")
("insert an arrow" "inserisci una freccia")
("insert an environment" "inserisci un nuovo ambiente")
("insert an n" "inserisci un/a n")
("insert appendix" "inserisci appendice")
("insert arccos" "inserisci arccos")
("insert arcsin" "inserisci arcsin")
("insert arctan" "inserisci arctan")
("insert arg" "inserisci arg")
("insert argument" "inserisci argomento")
("insert automatically generated content" "inserisci contenuto generato automaticamente")
("insert big contour integral" "inserisci integrale di curva grande")
("insert big coproduct" "inserisci coproduttoria grande")
("insert big direct sum" "inserisci somma diretta grande")
("insert big dotted point" "inserisci punto-prodotto grande")
("insert big integral" "inserisci integrale grande")
("insert big intersection" "inserisci intersezione grande")
("insert big logical and" "inserisci AND logico grande")
("insert big logical or" "inserisci OR logico grande")
("insert big product" "inserisci produttoria grande")
("insert big square union" "inserisci unione quadrata grande")
("insert big summation" "inserisci grande sommatoria")
("insert big tensor product" "inserisci prodotto tensoriale grande")
("insert big union" "inserisci unione grande")
("insert big union sum" "inserisci somma unione grande")
("insert big vertical space" "inserisci spazio verticale grande")
("insert chapter" "inserisci capitolo")
("insert column" "inserisci colonna")
("insert column to the left" "inserisci colonna a sinistra")
("insert column to the right" "inserisci colonna a destra")
("insert cos" "inserisci cos")
("insert cosh" "inserisci cosh")
("insert cot" "inserisci cot")
("insert coth" "inserisci coth")
("insert csc" "inserisci csc")
("insert deg" "inserisci deg")
("insert det" "inserisci det")
("insert dim" "inserisci dim")
("insert dollar" "inserisci dollaro")
("insert equation" "inserisci ecuazione")
("insert executable markup" "inserisci marcatura eseguibile")
("insert exp" "inserisci exp")
("insert footnote" "inserisci nota a pič di pagina")
("insert formula" "inserisci formula")
("insert gcd" "inserisci gcd")
("insert hom" "inserisci hom")
("insert horizontal interword space" "inserisci spazio orizzontale tra le parole")
("insert image" "inserisci immagine")
("insert inactive markup" "inserisci marcatura inattiva")
("insert inf" "inserisci inf")
("insert ker" "inserisci ker")
("insert lg" "inserisci lg")
("insert lim" "inserisci lim")
("insert liminf" "inserisci liminf")
("insert limsup" "inserisci limsup")
("insert ln" "inserisci ln")
("insert log" "inserisci log")
("insert mathematics" "inserisci matematica")
("insert max" "inserisci max")
("insert medium horizontal space" "inserisci spazio orizzontale medio")
("insert medium vertical space" "inserisci spazio verticale medio")
("insert min" "inserisci min")
("insert negative horizontal space" "inserisci spazio orizzontale negativo")
("insert new item or number" "inserisci nuovo elemento o numero")
("insert new rows or columns" "inserisci nuove righe o colonne")
("insert or delete rows or columns" "inserisci o cancella righe o colonne")
("insert row" "inserisci riga")
("insert row above" "inserisci riga sopra")
("insert row below" "inserisci riga sotto")
("insert sec" "inserisci sec")
("insert section" "inserisci sezione")
("insert sharp" "inserisci acuto")
("insert sin" "inserisci sin")
("insert sinh" "inserisci sinh")
("insert small horizontal space" "inserisci spazio orizzontale piccolo")
("insert small vertical space" "inserisci spazio verticale piccolo")
("insert structural markup" "inserisci marcatura strutturale")
("insert subsection" "inserisci sotto-sezione")
("insert subsubsection" "inserisci sotto-sotto-sezione")
("insert sup" "inserisci sup")
("insert tan" "inserisci tan")
("insert tanh" "inserisci tanh")
("insert thick horizontal space" "inserisci spazio orizzontale spesso")
("interfacing" "interfacciare")
("interfacing TeXmacs with other systems" "interfacciare TeXmacs con altri sistemi")
("interjection" "interiezione")
("interline separation" "separazione inter linea")
("interline space" "spazio inter linea")
("internationalization" "internazionalizzazione")
("interparagraph space" "spazio inter paragrafo")
("interrupt execution" "interrompi esecuzione")
("introduction" "introduzione")
("invisible" "invisible")
("invisible citation" "citazione invisible")
("irish" "irlandese")
("is running" "č in esecuzione")
("is tuple" "č una tupla")
("ispell does not respond" "ispell non risponde")
("ispell is already busy" "ispell č giā in esecuzione")
("italian" "italiano")
("italic" "corsivo")
("italic right" "corsivo a destra")
("itemize" "lista puntata")
("january" "gennaio")
("japanese" "giapponese")
("july" "luglio")
("june" "giugno")
("justified" "giustificato")
("justify text" "giustifica il testo")
("key" "chiave")
("keyboard" "tastiera")
("keyboard equivalent" "equivalente da tastiera")
("keyboard modifiers" "modificatori da tastiera")
("keyboard shorthand" "scorciatoia da tastiera")
("keymaps" "mappe di tastiera")
("keyword" "parola chiave")
("keywords" "parole chiave")
("kill" "uccidi")
("known bugs" "bug noti")
("label" "etichetta")
("landscape" "orizzontale")
("language" "lingua")
("large" "grande")
("large closing bracket" "parentesi chiusa grande")
("large delimiter" "delimitatore grande")
("large opening bracket" "parentesi aperta grande")
("large separator" "separatore grande")
("last indentation" "ultima indentazione")
("last page" "ultima pagina")
("latex command" "comando LaTeX")
("latin" "latino")
("layout" "impaginazione")
("left" "sinistra")
("left border" "bordo sinistro")
("left margin" "margine sinistro")
("left padding" "spaziatura sinistra")
("left prime" "prima sinistra")
("left ragged" "allineato a sinistra")
("left slanted" "inclinato a sinistra")
("left subscript" "pedice a sinistra")
("left superscript" "apice a sinistra")
("left surrounding" "surrounding a sinistra")
("legal" "legale")
("lemma" "lemma")
("length" "lunghezza")
("less" "minore")
("less or equal" "minore o uguale")
("letter" "lettera")
("license" "licenza")
("light" "leggero")
("limitations" "limitazioni")
("limits" "limiti")
("line" "linea")
("line break" "salto di linea")
("line stretch" "elasticitā della linea")
("link" "collegamento")
("link image" "inserisci immagine (collegamento)")
("list" "lista")
("list of figures" "lista delle figure")
("list of tables" "lista delle tabelle")
("load" "carica")
("load LaTeX file" "carica file LaTeX")
("load TeX file" "carica file TeX")
("load TeXmacs file" "carica file TeXmacs")
("load TeXmacs style file" "carica file di stile di TeXmacs")
("load a file" "carica un file")
("load buffer" "carica buffer")
("load file" "carica file")
("load html file" "carica file html")
("load image" "carica immagine")
("load in new window" "carica in una nuova finestra")
("load latex file" "carica file latex")
("load more recent autosave file" "carica ultimo file salvato automaticamente")
("load tex file" "carica file tex")
("load verbatim file" "carica file di testo semplice")
("loaded" "caricato")
("long" "lungo")
("look and feel" "aspetto")
("look up" "cerca")
("lower border" "bordo inferiore")
("macro" "macro")
("magenta" "magenta")
("magic" "magico")
("magnification" "ingrandimento")
("magnified" "ingrandito")
("main" "principale")
("main icon bar" "barra delle icone principale")
("major changes" "cambiamenti principali")
("make a page insertion" "fai un inserimento di pagina")
("make a subscript" "crea un pedice")
("make a superscript" "crea un apice")
("make acute" "crea un acuto")
("make bar" "crea una barra")
("make check" "crea controllo")
("make fraction" "crea frazione")
("make grave" "crea grave")
("make hat" "crea cappello")
("make label" "crea etichetta")
("make negation" "crea negazione")
("make page reference" "crea referenza ad una pagina")
("make reference" "crea referenza")
("make square root" "crea radice quadrata")
("make tilde" "crea tilde")
("make title" "crea titolo")
("make vector" "crea vettore")
("making donations" "come fare donazioni")
("manual" "manuale")
("map to" "rimappa in")
("march" "marzo")
("margins" "margini")
("margins as on paper" "margini come sul foglio")
("master" "master")
("master file" "file master")
("math" "math")
("math font" "carattere matematico")
("math font family" "famiglia del carattere matematico")
("math font series" "serie del carattere matematico")
("math font shape" "forma del carattere matematico")
("mathematical" "matematico")
("mathematical formulas" "formule matematiche")
("mathematical typesetting" "composizione tipografica matematica")
("mathematics" "matematica")
("matrix" "matrice")
("maximal number of columns" "numero massimo di colonne")
("maximal number of rows" "numero massimo di righe")
("maximum mode" "modo massimo")
("may" "maggio")
("medium" "medio")
("medium skip" "salto medio")
("meminfo" "meminfo")
("merge" "unisci")
("messy" "confusa")
("meta key" "tasto Meta")
("middle" "mediano")
("minimal number of columns" "numero minimo di colonne")
("minimal number of rows" "numero minimo di righe")
("minimum mode" "modo minimo")
("minus" "meno")
("miscellaneous" "miscellanea")
("mod" "mod")
("mode" "modo")
("modifier" "modificatore")
("modify cell alignment" "modifica l'allineamento della cella")
("modify cell border" "modifica il bordo della cella")
("modify cell padding" "modifica la spaziatura della cella")
("modify cell size" "modifica le dimensioni della cella")
("modify table border" "modifica il bordo della tabella")
("modify table padding" "modifica la spaziatura della tabella")
("mongolian" "mongolo")
("more information about GNU TeXmacs can be found in the help" "maggiori informazioni su GNU TeXmacs possono essere trovate nel menu aiuto")
("mosaic" "mosaico")
("move down" "sposta in basso")
("move object" "sposta oggetto")
("move to the right when finished" "muovi a destra quando hai finito")
("move up" "muovi in alto")
("n-th" "ennesima")
("n-th root" "radice ennesima")
("name" "nome")
("negation" "negazione")
("new" "nuovo")
("new buffer" "nuovo buffer")
("new column" "nuova colonna")
("new item" "nuovo elemento")
("new line" "nuova linea")
("new number" "nuovo numero")
("new page" "nuova pagina")
("new page before" "nuova pagina prima")
("new paragraph" "nuovo paragrafo")
("new roman" "new roman")
("new row" "nuova riga")
("new table" "nuova tabella")
("next line" "prossima linea")
("no changes need to be saved" "non ci sono cambiamenti da salvare")
("no dictionary for" "non c'č un dizionario per")
("no first indentation" "nessuna prima indentazione")
("no indentation" "nessuna indentazione")
("no last indentation" "nessuna ultima indentazione")
("no line break" "nessun salto di linea")
("no matches found" "non sono stati trovati risultati")
("no more matches for" "non ci sono altri risultati per la ricerca di")
("no more redo information available" "non ci sono pių informazioni per rifare")
("no more undo information available" "non ci sono pių informazioni per annullare'")
("no name" "senza nome")
("no page break after" "nessuna interruzione di pagina dopo")
("no page break before" "nessuna interruzione di pagina prima")
("no spelling errors found in text" "non sono stati trovati errori di ortografia nel testo")
("no style" "nessuno stile")
("none" "nessuno")
("normal" "normale")
("norwegian" "norvegese")
("not" "no")
("not equal" "non uguale")
("not found" "non trovato")
("notation" "notazione")
("note" "nota")
("november" "novembre")
("number" "numero")
("number equation" "equazione numerata")
("number of columns" "numero di colonne")
("numbered equation" "equazione numerata")
("numbered equations" "equazioni numerate")
("numerator" "numeratore")
("numeric" "numerico")
("objectives" "obiettivi")
("occurrences" "occorrenze")
("october" "ottobre")
("odd page footer" "pič di pagina (pagine dispari)")
("odd page header" "intestazione (pagine dispari)")
("odd page margin" "margine di pagine dispari")
("odd page text" "testo (pagine dispari)")
("off" "disattivato")
("ok" "ok")
("old english" "inglese antico")
("old slavonic" "slavo antico")
("old style" "vecchio stile")
("on" "attivato")
("on cells" "sulle celle")
("on columns" "sulle colonne")
("on entire table" "su tutta la tabella")
("on rows" "sulle righe")
("one spelling error has been corrected" "un errore di ortografia č stato corretto")
("online documentation" "documentazione in linea")
("online help" "aiuto in linea")
("open" "apri")
("open a new buffer" "apri un nuovo buffer")
("open new window" "apri una nuova finestra")
("opening" "apertura")
("operation on named selection" "operazione sulla selezione indicata")
("operation on primary selection" "operazione sulla selezione primaria")
("operation on secondary selection" "operazione sulla selezione secondaria")
("operation on ternary selection" "operazione sulla selezione terziaria")
("optical" "ottica")
("option" "opzione")
("options" "opzioni")
("or" "o")
("orange" "arancione")
("orientation" "orientazione")
("origin" "origine")
("original welcome message" "messaggio originale di benvenuto")
("other" "altro")
("other pages" "altre pagine")
("other projects" "altri progetti")
("other style" "altri stili")
("outlined roman" "outlined roman")
("over" "sopra")
("padding" "spaziatura")
("page" "pagina")
("page bottom margin" "margine inferiore di pagina")
("page break" "interruzione di pagina")
("page break before" "interruzione di pagina prima")
("page height" "altezza della pagina")
("page insertion" "inserimento di pagina")
("page layout" "imaginazione della pagina")
("page number text" "testo del numero di pagina")
("page numbering" "numerazione di pagina")
("page reference" "referenza di pagina")
("page right margin" "margine destro della pagina")
("page top margin" "margine superiore della pagina")
("page type" "tipo di pagina")
("page width" "larghezza della pagina")
("paper" "foglio")
("paper type" "tipo di foglio")
("papyrus" "papiro")
("paragraph" "paragrafo")
("paragraph mode" "modo del paragrafo")
("paragraph width" "larghezza del paragrafo")
("part" "parte")
("past changes" "cambiamenti precedenti")
("paste" "incolla")
("paste from" "incolla da")
("paste text" "incolla testo")
("path" "cammino")
("perform operations on cells" "esegui operazioni sulle celle")
("perform operations on columns" "esegui operazioni sulle colonne")
("perform operations on entire table" "esegui operazioni sull'intera tabella")
("perform operations on rows" "esegui operazioni sulle righe")
("permanent" "permanente")
("person" "persona")
("personal dictionary has been modified" "il dizionario personale č stato modificato")
("phantasy" "fantasia")
("philosophy" "filosofia")
("phoenician" "fenicio")
("phonetic" "fonetico")
("pink" "rosa")
("plain block" "blocco normale")
("plain tabular" "tabella normale")
("planned changes" "cambiamenti previsti")
("plans for the future" "programmi per il futuro")
("please" "per favore")
("please wait" "per favore, attendi")
("plus" "pių")
("polish" "polacco")
("porting TeXmacs to other platforms" "porting di TeXmacs ad altre piattaforme")
("portrait" "verticale")
("portuguese" "portoghese")
("position" "posizione")
("position float" "posiziona fluttuante")
("position floating object" "posiziona oggetto fluttuante")
("position the table" "posiziona la tabella")
("postscript" "postscript")
("postscript image" "immagine postscript")
("pre-alpha release" "versione pre-alfa")
("preamble" "preambolo")
("preamble mode" "modo preambolo")
("preferences" "preferenze")
("presentation mode" "modo presentazione")
("preview before printing" "anteprima prima di stampare")
("preview with ghostview" "anteprima con ghostview")
("primary" "primario")
("prime" "prima")
("print" "stampa")
("print all" "stampa tutto")
("print all to file" "stampa tutto su file")
("print page selection" "stampa le pagine selezionate")
("print page selection to file" "stampa le pagine selezionate su file")
("print to postscript file" "stampa su file postscript")
("printer" "stampante")
("printer dpi" "risoluzione della stampante (dpi)")
("printing command" "comando di stampa")
("problem" "problema")
("professional" "professionale")
("prog" "prog")
("prog font" "carattere prog")
("prog font family" "famiglia del carattere prog")
("prog font series" "stile del carattere prog")
("prog font shape" "forma del carattere prog")
("program" "programma")
("program mode" "modo programma")
("programming" "programmazione")
("progressive" "progressivo")
("project" "progetto")
("projects" "progetti")
("prologue" "prologo")
("prompt on scripts" "chiedi per script")
("proof" "dimostrazione")
("proportional" "proporzionale")
("proportional typewriter" "macchina da scrivere proporzionale")
("proposition" "proposizione")
("query replace" "sostituisci")
("quit" "esci")
("quotation" "citazione (pių paragrafi)")
("quote" "citazione")
("range" "rango")
("really close it" "chiudi per davvero")
("really huge" "gigante")
("really quit" "sicuro di voler uscire")
("red" "rosso")
("redo" "rifai")
("redo undone changes" "rifai azioni annullate")
("reduce bottom margin by" "riduci il margine superiore di")
("reduce left margin by" "riduci il margine sinistro di")
("reduce margins" "riduci i margini")
("reduce right margin by" "riduci il margine destro di")
("reduce top margin by" "riduci il margine superiore di")
("reduction even page margin" "riduci il margine delle pagine pari")
("reduction odd page margin" "riduci il margine delle pagine dispari")
("reduction page bottom margin" "riduci il margine inferiore della pagina")
("reduction page right margin" "riduci il margine destro della pagina")
("reduction page top margin" "riduci il margine superiore della pagina")
("reference" "referenza")
("references" "referenze")
("regular" "regolare")
("release" "versione")
("reload" "ricarica")
("remark" "importante")
("remove number" "rimuovi numero")
("remove this switch" "rimuovi questo switch")
("renumber this page" "rinumera questa pagina")
("repeat object" "ripeti oggetto")
("replace" "sustituisci")
("replace by" "sustituisci con")
("replaced" "sustituito")
("replaced one occurrence" "un'occorrenza č stata sostituita")
("reset" "azzera")
("resize object" "ridimensiona oggetto")
("resized" "oggetto ridimensionato")
("return" "invio")
("revert" "ritorna all'originale")
("revert buffer" "ripristina il buffer")
("right" "destra")
("right border" "bordo destro")
("right margin" "margine destro")
("right padding" "spaziatura destra")
("right ragged" "allineato a  destra")
("right subscript" "pedice a destra")
("right superscript" "apice a destra")
("right surrounding" "surrounding a destra")
("rigid" "rigido")
("rigid horizontal" "orizzontale rígida")
("rigid space" "spazio rígido")
("romanian" "rumeno")
("root" "radice")
("rotate backward" "ruota all'indietro")
("rotate forward" "ruota in avanti")
("row" "riga")
("row contains glued items" "la riga contiene elementi appiccicati")
("runic" "runico")
("running author" "autore attuale")
("running title" "titolo attuale")
("russian" "russo")
("russian and Ukrainian users" "utenti russi ed ucraini")
("sample" "esempio")
("sans serif" "sans serif")
("save" "salva")
("save TeXmacs file" "salva file TeXmacs")
("save TeXmacs style file" "salva file di stile TeXmacs")
("save as" "salva come")
("save as LaTeX" "salva come LaTeX")
("save as TeX" "salva come TeX")
("save buffer" "salva buffer")
("save buffer as" "salva il buffer come")
("save file" "salva file")
("save html file" "salva file html")
("save image" "salva immagine")
("save latex file" "salva file latex")
("save tex file" "salva file tex")
("save this buffer" "salva questo buffer")
("save verbatim file" "salva file di testo semplice")
("saved" "salvato")
("screen layout" "impaginazione su schermo")
("screen margins" "margini dello schermo")
("script" "script")
("script above" "script sopra")
("script below" "script sotto")
("script script size" "script script size")
("script size" "dimensione dello script")
("scripts not accepted" "gli script non sono accettati")
("search" "cerca")
("searching" "cercando")
("secondary" "secondario")
("section" "sezione")
("security" "sicurezza")
("see" "vedi")
("select a document style" "seleziona uno stile di documento")
("select a foreground color" "seleziona un colore di primo piano")
("select a language" "seleziona una lingua")
("selected region" "regione selezionata")
("selection" "selezione")
("selections" "selezioni")
("seminar" "seminario")
("separation between lines" "separazione tra le righe")
("separation between paragraphs" "separazione tra i paragrafi")
("separator" "separatore")
("september" "settembre")
("serb" "serbo")
("series" "serie")
("series|right" "serie|destra")
("session" "sessione")
("session name" "nome della sessione")
("session type" "tipo di sessione")
("sessions" "sessioni")
("set" "imposta")
("set background color of cell" "imposta colore di sfondo della cella")
("set cell mode" "imposta il modo della cella")
("set extension center" "imposta il centro dell'estensione")
("set height" "imposta altezza")
("set height of cell" "imposta l'altezza della cella")
("set margins" "imposta i margini")
("set n" "imposta n")
("set paragraph margins" "imposta i margini del paragrafo")
("set paragraph mode" "imposta il modo del paragrafo")
("set printer dpi" "imposta la risoluzione della stampante (dpi)")
("set printing command" "imposta il comando di stampa")
("set span" "imposta span")
("set special cell properties" "imposta proprietā speciali della cella")
("set special table properties" "imposta proprietā speciali della tabella")
("set text width" "imposta la larghezza del testo")
("set width" "imposta larghezza")
("set width of cell" "imposta la larghezza della cella")
("settings" "impostazioni")
("setup printer" "configura la stampante")
("shape" "forma")
("shift-down" "shift-gių")
("shift-right" "shift-destra")
("show" "mostra")
("show header and footer" "mostra intestazione e pič di pagina")
("show on screen" "mostra sullo schermo")
("shrinking factor" "fattore di riduzione")
("signature" "firma")
("size" "dimensioni")
("size limits" "limiti delle dimensioni")
("slanted" "inclinata")
("sloppy" "sloppy")
("small" "piccolo")
("small caps" "maiuscoletto")
("small figure" "figura piccola")
("small skip" "salto piccolo")
("small table" "tabella piccola")
("source code" "codice sorgente")
("space" "spazio")
("space between lines" "spazio tra le righe")
("space between paragraphs" "spazio tra i paragrafi")
("spacing" "spaziatura")
("spanish" "spagnolo")
("special" "speciale")
("special cell properties" "proprietā speciali della cella")
("special table properties" "proprietā speciali della tabella")
("specific" "specifico")
("spell" "ortografía")
("spell checking complete" "controllo ortografico completo")
("spelling error" "errore ortografico")
("spelling errors have been corrected" "gli errori ortografici sono stati corretti")
("split" "dividi")
("square root" "radice quadrata")
("start" "inizio")
("start a new chapter" "inizia un nuovo capitolo")
("start a new section" "inizia una nuova sezione")
("start an interactive session" "inizia una sessione interattiva")
("status bar" "barra di stato")
("stretchable" "estensibile")
("stretchable horizontal" "estensibile orizzontalmente")
("stretchable vertical" "estensibile verticalmente")
("stroke" "tratto")
("strokes" "tratti")
("strong" "enfatizzato")
("structural" "strutturale")
("style" "stile")
("styles" "stili")
("sub" "secondo livello")
("subparagraph" "sotto-paragrafo")
("subscript" "pedice")
("subsection" "sotto-sezione")
("subsub" "terzo livello")
("subsubsection" "sotto-sotto-sezione")
("summary" "sommario")
("summary of the principal TeXmacs tags" "sommario dei principali tag di TeXmacs")
("superpose" "sovrapponi")
("superposed" "sovrapposto")
("superscript" "apice")
("swedish" "svedese")
("switch" "switch")
("switch to first" "passa al primo")
("switch to formula style" "passa allo stile formula")
("switch to last" "passa all'ultimo")
("switch to next" "passa al successivo")
("switch to previous" "passa al precedente")
("switching and folding" "switch e arrotolamento")
("symbol" "simbolo")
("system environment variables" "variabili ambientali di sistema")
("tab" "tab")
("table" "tabella")
("table command" "comando di tabella")
("table of contents" "indice")
("tables" "tabelle")
("tabular" "tabellare")
("tabular material" "materiale tabellare")
("teXmacs as an interface" "TeXmacs come un'interfaccia")
("teXmacs fonts" "font di TeXmacs")
("teXmacs notice" "avviso di TeXmacs")
("teXmacs style files" "fogli di stile di TeXmacs")
("ternary" "ternario")
("test" "provare")
("tex" "tex")
("text" "testo")
("text font" "carattere del testo")
("text height correction" "correzione dell'altezza del testo")
("text width" "larghezza del testo")
("texts" "testi")
("th century bc" "-esimo secolo A.C.")
("th root" "radice -esima")
("thai" "tailandia")
("the TeXmacs authors" "gli autori di TeXmacs")
("the application did not send its usual startup banner" "l'applicazione non ha inviato il suo solito banner iniziale")
("the application does not reply" "l'applicazione non risponde")
("the application is already running" "l'applicazione č giā in esecuzione")
("the boxes produced by the typesetter" "i box prodotti dal sistema di composizione")
("the graphical user interface" "l'interfaccia grafica utente")
("the leaves of TeXmacs trees" "le foglie dell'albero di TeXmacs")
("the mycas example" "l'esempio di mycas")
("the primitive TeXmacs constructs" "i costrutti primitivi di TeXmacs")
("the standard TeXmacs styles" "gli stili standard di TeXmacs")
("theorem" "teorema")
("there are unsaved files" "ci sono file non salvati")
("this buffer has not been saved" "questo buffer non č stato salvato")
("this document has been produced using" "questo documento č stato realizzato usando")
("this line" "questa linea")
("this page" "questa pagina")
("tilda" "tilda")
("tilde" "tilde")
("times" "times")
("tiny" "diminuito")
("title" "titolo")
("to the left" "a sinistra")
("to the right" "a destra")
("today" "oggi")
("tools" "strumenti")
("top" "superiore")
("top border" "bordo superiore")
("top level" "livello superiore")
("top margin" "margine superiore")
("top padding" "spaziatura superiore")
("translate" "traduci")
("tree" "albero")
("tuple" "tupla")
("turkish" "turco")
("tutorial" "tutorial")
("type" "tipo")
("type a latex command followed by" "scrivi un comando latex seguito da")
("type body" "scrivi il corpo")
("type denominator" "scrivi il denominatore")
("type numerator" "scrivi il numeratore")
("type script" "scrivi l'indice")
("typesetted boxes" "composizione tipografica dei box")
("typewriter" "macchina da scrivere")
("typewriter small caps" "maiuscoletto macchina da scrivere")
("typing a simple text" "comporre un semplice testo")
("typing simple texts" "comporre semplici testi")
("ugaritic" "ugaritico")
("ukrainian" "ucraino")
("undo" "annulla")
("undo last changes" "annulla gli ultimi cambiamenti")
("up" "su")
("update" "aggiorna")
("update from web" "aggiorna dal web")
("update image links" "aggiorna i collegamenti alle immagini")
("upper border" "bordo superiore")
("use TeXmacs" "usa TeXmacs")
("use a black color" "usa colore nero")
("use a blue color" "usa colore blu")
("use a bold font" "usa un font grassetto")
("use a brown color" "usa colore marrone")
("use a calligraphic font" "usa un carattere calligrafico")
("use a green color" "usa colore verde")
("use a grey color" "usa colore grigio")
("use a magenta color" "usa colore magenta")
("use a orange color" "usa colore arancione")
("use a pink color" "usa colore rosa")
("use a red color" "usa colore rosso")
("use a sans serif font" "usa un carattere sans serif")
("use a typewriter font" "usa un carattere macchina da scrivere")
("use a white color" "usa colore bianco")
("use a yellow color" "usa colore giallo")
("use blackboard bold font" "usa carattere lavagna grassetto")
("use bold font series" "usa serie di caratteri grassetto")
("use calligraphical font" "usa carattere calligrafico")
("use footnote font size" "usa dimensione del carattere pič di pagina")
("use fraktur font" "usa carattere fraktur")
("use huge font size" "usa un carattere enorme")
("use italic font shape" "usa un carattere di forma corsiva")
("use large font size" "usa un carattere grande")
("use left hand column as border" "usa la colonna di sinistra come bordo")
("use medium font series" "usa serie di caratteri medio")
("use normal font size" "usa carattere di dimensioni normali")
("use package" "usa pacchetto")
("use paragraph width" "usa larghezza di paragrafo")
("use really huge font size" "usa un carattere gigante")
("use right font shape" "usa forma del carattere di destra")
("use right hand column as border" "usa la colonna di destra come bordo")
("use roman font family" "usa famiglia di caratteri roman")
("use row above as border" "usa la riga superiore come bordo")
("use row below as border" "usa la riga inferiore come bordo")
("use sans serif font family" "usa famiglia di caratteri sans serif")
("use script font size" "usa dimensione del carattere script")
("use slanted font shape" "usa forma di carattere inclinata")
("use small capitals" "usa maiuscoletto")
("use small font size" "usa carattere piccolo")
("use small-caps font shape" "usa forma di carattere maiuscoletto")
("use the blackboard bold font" "usa carattere lavagna grassetto")
("use the fraktur font" "usa il carattere fraktur")
("use tiny font font size" "usa dimensione del carattere piccolo")
("use typewriter font family" "usa famiglia di caratteri macchina da scrivere")
("use very large font size" "usa un carattere molto grande")
("user manual" "manuale utente")
("user provided icons" "icone definite dall'utente")
("value" "valore")
("variable" "variabile")
("variant" "variante")
("vector" "vettore")
("verbatim" "testo semplice")
("verse" "verso")
("vertical" "verticale")
("vertical after" "verticale dopo")
("vertical before" "verticale prima")
("vertical cell alignment" "allineamento verticale della cella")
("vertical part" "parte verticale")
("vertical position" "posizione verticale")
("vertical space" "spazio verticale")
("vertical table alignment" "alineamento verticale della tabella")
("vertically align cell to the base" "allinea verticalmente la cella alla base")
("vertically center the cell" "centra verticalmente la cella")
("very large" "molto grande")
("very small" "molto piccolo")
("vietnamese" "vietnamita")
("view" "visualizza")
("visible" "visibile")
("warning" "avviso")
("welcome" "benvenuto")
("welcome to GNU TeXmacs" "benvenuto in GNU TeXmacs")
("what is new" "che c'č di nuovo")
("when finished" "una volta terminato")
("while" "mentre")
("white" "bianco")
("wide accent" "accento largo")
("width" "larghezza")
("width is exactly the specified width" "la larghezza č esattamente la larghezza specificata")
("width is maximum of specified width and box width" "la larghezza č la massimo tra quella specificata e quella del box")
("width is minimum of specified width and box width" "la larghezza č la minima tra quella specificata e quella del box")
("width magnification" "ingrandimento della larghezza")
("window" "finestra")
("window height" "altezza della finestra")
("window width" "larghezza della finestra")
("windows key" "tasto Window")
("with" "con")
("write a name" "scrivi un nome")
("write bold mathematics" "scrivi matematica in grassetto")
("write bold text" "scrivi in grassetto")
("write italic text" "scrivi in italico")
("write sample text" "scrivi esempio")
("write strong text" "scrivi testo marcato")
("write typewriter text" "scrivi testo macchina da scrivere")
("write verbatim text" "scrivi testo a larghezza fissa")
("writing data converters" "scrivere convertitori di formato")
("writing your own style files" "scrivere il proprio file di stile")
("yellow" "giallo")
("you can also obtain" "puoi anche ottenere")
("your document is back in its original state" "il documento č stato riportato allo stato originale")
("your first interface" "la tua prima interfaccia")
