
/******************************************************************************
* MODULE     : ispell.hpp
* DESCRIPTION: interface with the ispell spell checker
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef ISPELL_H
#define ISPELL_H
#include "tree.hpp"

string ispell_start (string lan);
tree   ispell_check (string lan, string s);
void   ispell_accept (string lan, string s);
void   ispell_insert (string lan, string s);
void   ispell_done (string lan);

#endif // ISPELL_H
