;; Unicode characters which are imported specially in TeXmacs

;; (C) 2002-2003  David Allouche
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details. If
;; you don't have this file, write to the Free Software Foundation, Inc., 59
;; Temple Place - Suite 330, Boston, MA 02111-1307, USA.

("#00A0"	" ")		; no-break space = non-breaking space
("#2019"	"'")		; right single quotation mark
