("a modifier" "")
("abbreviation" "")
("about menu" "")
("accept" "")
("accept all scripts" "")
("accept no scripts" "")
("accept script" "")
("acknowledgment" "")
("acronym" "")
("action" "")
("add switch after" "")
("add switch before" "")
("address block" "")
("advanced layout features" "")
("albanian" "")
("algorithm" "")
("align cell to the bottom" "")
("align cell to the left" "")
("align cell to the right" "")
("align cell to the top" "")
("align table at base of left column" "")
("align table at base of middle column" "")
("align table at base of origin column" "")
("align table at base of right column" "")
("align table at fraction bar height" "")
("align table at the base of bottom row" "")
("align table at the base of middle row" "")
("align table at the base of origin row" "")
("align table at the base of top row" "")
("align table at the bottom" "")
("align table at the center" "")
("align table at the horizontal center" "")
("align table at the left" "")
("align table at the top" "")
("align table to the right" "")
("align text to the left" "")
("align text to the right" "")
("aligned" "")
("alignment" "")
("allow" "")
("allowed extension" "")
("allowed reduction" "")
("alt key" "")
("and" "")
("argument" "")
("arithmetic" "")
("arrows" "")
("at fraction bar height" "")
("at the baseline" "")
("at the bottom" "")
("at the left" "")
("at the middle" "")
("at the right" "")
("at the top" "")
("authorize" "")
("automatically close brackets" "")
("automatically generated content" "")
("autosave" "")
("background" "")
("background evaluations" "")
("backward replace" "")
("backward search" "")
("basic communication using pipes" "")
("basic data types" "")
("become a TeXmacs developer" "")
("big" "")
("big figure" "")
("big table" "")
("border" "")
("both" "")
("bottom border" "")
("bottom padding" "")
("breaking" "")
("british" "")
("broken white" "")
("browse" "")
("browse web" "")
("browser" "")
("bulgarian" "")
("bullet" "")
("bullets" "")
("cancel" "")
("caps-lock key" "")
("case" "")
("cc" "")
("cell" "")
("cell background color" "")
("cell border" "")
("cell height" "")
("cell operation mode" "")
("cell width" "")
("center text" "")
("centered block" "")
("centered tabular" "")
("change border of cell" "")
("change cell operation mode" "")
("change log" "")
("check text for spelling errors" "")
("chinese" "")
("choice" "")
("cite" "")
("clear selection" "")
("close session" "")
("closing" "")
("code" "")
("compact" "")
("compatibility with other formats" "")
("compile article" "")
("compile book" "")
("completion" "")
("complex" "")
("condition" "")
("configuration" "")
("contact" "")
("content" "")
("content tag" "")
("context dependent icons" "")
("continuation of" "")
("contribute" "")
("convention" "")
("converters to other data formats" "")
("copy text" "")
("copy to" "")
("correct text" "")
("custom tab" "")
("customizing TeXmacs" "")
("customizing the TeXmacs styles" "")
("custum tab" "")
("cut text" "")
("cut to" "")
("cyrillic input method" "")
("czech" "")
("danish" "")
("dark grey" "")
("dash" "")
("dashes" "")
("data conversion" "")
("data format" "")
("date" "")
("deactivate" "")
("deactivate table" "")
("decorate atoms" "")
("default padding" "")
("delete column" "")
("delete row" "")
("destination" "")
("determinant" "")
("developers guide" "")
("directory" "")
("disable" "")
("disable first indentation" "")
("disable first indentation after" "")
("disable first indentation before" "")
("disallow" "")
("distribute unused space" "")
("div" "")
("do not remap" "")
("document format" "")
("documentation" "")
("documents are trees" "")
("dots" "")
("download" "")
("duplicate" "")
("dynamic libraries" "")
("editing tools" "")
("email" "")
("emphasize" "")
("emphasize text" "")
("enable first indentation" "")
("enable first indentation after" "")
("enable first indentation before" "")
("encl" "")
("enter a title" "")
("equal" "")
("equivalent for" "")
("esperanto" "")
("evaluate" "")
("even page footer" "")
("even page header" "")
("even page text" "")
("exact mode" "")
("exam" "")
("exclusive" "")
("exclusive or" "")
("executable" "")
("expand" "")
("explained" "")
("export as" "")
("extern " "")
("extra" "")
("extract format" "")
("find next match" "")
("find text" "")
("finnish" "")
("first contact" "")
("first start" "")
("flexibility" "")
("flexibility factor for vertical spacing" "")
("floating figure" "")
("floating object" "")
("floating table" "")
("fold" "")
("fold input field" "")
("folded" "")
("font name" "")
("font variant" "")
("fonts" "")
("footer" "")
("footnote" "")
("for" "")
("foreground" "")
("format" "")
("formatted and structured output" "")
("formatted date" "")
("forward replace" "")
("forward search" "")
("full manuals" "")
("further customization" "")
("gNU TeXmacs comes without any form of legal warranty" "")
("gNU TeXmacs falls under the GNU general public license" "")
("general architecture of TeXmacs" "")
("generating font file" "")
("generic" "")
("glossary entry" "")
("glue decorations" "")
("go" "")
("greater" "")
("greater or equal" "")
("h modifier" "")
("happy typing" "")
("has been aborted" "")
("has completed its task" "")
("header" "")
("header and footer" "")
("height is exactly the specified height" "")
("height is maximum of specified height and box height" "")
("height is minimum of specified height and box height" "")
("help us" "")
("here" "")
("home" "")
("horizontal" "")
("horizontal cell alignment" "")
("horizontal part" "")
("horizontal table alignment" "")
("horizontally align cell to the base" "")
("horizontally center the cell" "")
("how much longer may pages become" "")
("how much shorter may pages become" "")
("hungarian" "")
("hybrid command" "")
("if" "")
("image links" "")
("improving the current implementation" "")
("include" "")
("include a document" "")
("include file" "")
("inclusion into TeXmacs" "")
("inclusions" "")
("index entry" "")
("input mode" "")
("insert a big operator" "")
("insert a binary operation" "")
("insert a binary relation" "")
("insert a blackboard bold character" "")
("insert a bold calligraphic character" "")
("insert a bold character" "")
("insert a calligraphic character" "")
("insert a fraction" "")
("insert a fraktur character" "")
("insert a greek character" "")
("insert a large left delimiter" "")
("insert a large right delimiter" "")
("insert a large separator" "")
("insert a link" "")
("insert a miscellaneous symbol" "")
("insert a new item" "")
("insert a picture" "")
("insert a square root" "")
("insert a table" "")
("insert a table or a matrix" "")
("insert a wide symbol above" "")
("insert a wide symbol below" "")
("insert an" "")
("insert an accent" "")
("insert an arrow" "")
("insert an environment" "")
("insert an n" "")
("insert automatically generated content" "")
("insert column" "")
("insert column to the left" "")
("insert column to the right" "")
("insert executable markup" "")
("insert fields" "")
("insert footnote" "")
("insert image" "")
("insert inactive markup" "")
("insert input field above" "")
("insert input field below" "")
("insert macro" "")
("insert mathematics" "")
("insert new rows or columns" "")
("insert or delete rows or columns" "")
("insert row" "")
("insert row above" "")
("insert row below" "")
("insert structural markup" "")
("insert text field" "")
("insert value" "")
("interfacing" "")
("interfacing TeXmacs with other systems" "")
("interjection" "")
("interline separation" "")
("internationalization" "")
("interrupt execution" "")
("invisible" "")
("is running" "")
("is tuple" "")
("ispell does not respond" "")
("ispell is already busy" "")
("italian" "")
("japanese" "")
("justify text" "")
("key" "")
("keyboard equivalent" "")
("keyboard modifiers" "")
("keyboard shorthand" "")
("keyword" "")
("keywords" "")
("left arrow" "")
("left padding" "")
("left prime" "")
("left surrounding" "")
("length" "")
("less" "")
("less or equal" "")
("light grey" "")
("limits" "")
("link image" "")
("load a file" "")
("load buffer" "")
("load file" "")
("load image" "")
("look and feel" "")
("look up" "")
("m modifier" "")
("macro" "")
("main" "")
("main icon bar" "")
("major changes" "")
("make a page insertion" "")
("make a subscript" "")
("make a superscript" "")
("make page reference" "")
("making donations" "")
("manual" "")
("map to" "")
("margins as on paper" "")
("master" "")
("master file" "")
("mathematical and customized input" "")
("mathematical formulas" "")
("mathematical input" "")
("mathematical typesetting" "")
("maximal number of columns" "")
("maximal number of rows" "")
("maximum mode" "")
("merge" "")
("meta key" "")
("minimal number of columns" "")
("minimal number of rows" "")
("minimum mode" "")
("miscellaneous features" "")
("mod" "")
("modifier" "")
("modify cell alignment" "")
("modify cell border" "")
("modify cell padding" "")
("modify cell size" "")
("modify table border" "")
("modify table padding" "")
("more information about GNU TeXmacs can be found in the help" "")
("multiline input" "")
("my documents" "")
("new buffer" "")
("new page before" "")
("next" "")
("no dictionary for" "")
("no indentation" "")
("no matches found" "")
("no matches found for" "")
("no more matches for" "")
("no more redo information available" "")
("no more undo information available" "")
("no spelling errors found in text" "")
("no style" "")
("norwegian" "")
("not" "")
("not equal" "")
("notation" "")
("number" "")
("number equation" "")
("number of columns" "")
("occurrences" "")
("odd page footer" "")
("odd page header" "")
("odd page text" "")
("ok" "")
("old style" "")
("on cells" "")
("on columns" "")
("on entire table" "")
("on rows" "")
("one spelling error has been corrected" "")
("online documentation" "")
("online help" "")
("open" "")
("open a new buffer" "")
("opening" "")
("operation on named selection" "")
("operation on primary selection" "")
("operation on secondary selection" "")
("operation on ternary selection" "")
("or" "")
("origin" "")
("original welcome message" "")
("other pages" "")
("other style" "")
("over" "")
("overbrace" "")
("overline" "")
("padding" "")
("page break before" "")
("page insertion" "")
("page number text" "")
("page numbering" "")
("page reference" "")
("page setup" "")
("paste from" "")
("paste text" "")
("perform operations on cells" "")
("perform operations on columns" "")
("perform operations on entire table" "")
("perform operations on rows" "")
("permanent" "")
("person" "")
("personal dictionary has been modified" "")
("philosophy" "")
("plain block" "")
("plain tabular" "")
("planned changes" "")
("plans for the future" "")
("please wait" "")
("plugin" "")
("plugins" "")
("plus" "")
("polish" "")
("porting TeXmacs to other platforms" "")
("portuguese" "")
("position float" "")
("position floating object" "")
("position the table" "")
("preamble mode" "")
("preferences" "")
("presentation" "")
("presentation mode" "")
("previous" "")
("primary" "")
("prime" "")
("printer" "")
("problem" "")
("programming" "")
("progressive" "")
("prompt on scripts" "")
("prompts and default input" "")
("proportional" "")
("query replace" "")
("range" "")
("really close it" "")
("redo" "")
("redo undone changes" "")
("regular" "")
("reload" "")
("remove all output fields" "")
("remove fields" "")
("remove input field" "")
("remove input field above" "")
("remove number" "")
("remove this switch" "")
("renumber this page" "")
("repeat object" "")
("replace" "")
("replace by" "")
("replaced" "")
("replaced one occurrence" "")
("results of the search for" "")
("revert" "")
("right arrow" "")
("right padding" "")
("right surrounding" "")
("rotate backward" "")
("rotate forward" "")
("running author" "")
("running title" "")
("russian" "")
("sample" "")
("save buffer" "")
("save buffer as" "")
("save file" "")
("save image" "")
("save this buffer" "")
("scripts not accepted" "")
("search" "")
("search words in my documents" "")
("search words in the documentation" "")
("search words in the source code" "")
("searching" "")
("secondary" "")
("security" "")
("see" "")
("select a document style" "")
("select a foreground color" "")
("select a language" "")
("selections" "")
("sending commands to TeXmacs" "")
("serb" "")
("session name" "")
("session type" "")
("sessions" "")
("set background color of cell" "")
("set cell mode" "")
("set extension center" "")
("set height" "")
("set height of cell" "")
("set paragraph margins" "")
("set paragraph mode" "")
("set span" "")
("set special cell properties" "")
("set special table properties" "")
("set width" "")
("set width of cell" "")
("settings" "")
("show" "")
("show on screen" "")
("signature" "")
("size limits" "")
("sloppy" "")
("slovene" "")
("small figure" "")
("small table" "")
("source code" "")
("space between lines" "")
("space between paragraphs" "")
("spanish" "")
("special cell properties" "")
("special table properties" "")
("spell" "")
("spell checking complete" "")
("spelling error" "")
("spelling errors have been corrected" "")
("split session" "")
("start a new chapter" "")
("start a new section" "")
("start an interactive session" "")
("status bar" "")
("stroke" "")
("strokes" "")
("strong" "")
("structural" "")
("sub" "")
("subparagraph" "")
("subsub" "")
("summary" "")
("summary of the principal TeXmacs tags" "")
("superpose" "")
("superposed" "")
("switch" "")
("switch to first" "")
("switch to last" "")
("switch to next" "")
("switch to previous" "")
("switching and folding" "")
("system environment variables" "")
("table command" "")
("tables" "")
("tabular material" "")
("teXmacs as an interface" "")
("teXmacs fonts" "")
("teXmacs notice" "")
("teXmacs plugins" "")
("teXmacs style files" "")
("ternary" "")
("text height correction" "")
("texts" "")
("the TeXmacs authors" "")
("the application did not send its usual startup banner" "")
("the application does not reply" "")
("the application is already running" "")
("the boxes produced by the typesetter" "")
("the graphical user interface" "")
("the leaves of TeXmacs trees" "")
("the mycas example" "")
("the primitive TeXmacs constructs" "")
("the standard TeXmacs styles" "")
("this buffer has not been saved" "")
("this document has been produced using" "")
("this page" "")
("times" "")
("to the left" "")
("to the right" "")
("today" "")
("tools" "")
("top border" "")
("top padding" "")
("translate" "")
("tuple" "")
("tutorial" "")
("typesetted boxes" "")
("typing a simple text" "")
("typing simple texts" "")
("ukrainian" "")
("underbrace" "")
("underline" "")
("undo last changes" "")
("update from web" "")
("update image links" "")
("use TeXmacs" "")
("use a bold font" "")
("use a calligraphic font" "")
("use a sans serif font" "")
("use a typewriter font" "")
("use calligraphical font" "")
("use left hand column as border" "")
("use paragraph width" "")
("use right hand column as border" "")
("use row above as border" "")
("use row below as border" "")
("use small capitals" "")
("use the blackboard bold font" "")
("use the fraktur font" "")
("user manual" "")
("user provided icons" "")
("users of Cyrillic languages" "")
("value" "")
("variable" "")
("variant" "")
("vertical" "")
("vertical cell alignment" "")
("vertical part" "")
("vertical table alignment" "")
("vertically align cell to the base" "")
("vertically center the cell" "")
("view" "")
("visible" "")
("welcome to GNU TeXmacs" "")
("what is new" "")
("while" "")
("wide bar" "")
("width is exactly the specified width" "")
("width is maximum of specified width and box width" "")
("width is minimum of specified width and box width" "")
("window" "")
("windows key" "")
("with" "")
("write a name" "")
("write bold mathematics" "")
("write bold text" "")
("write italic text" "")
("write sample text" "")
("write strong text" "")
("write typewriter text" "")
("write verbatim text" "")
("writing data converters" "")
("writing your own style files" "")
("your document is back in its original state" "")
("your first interface" "")
("your language" "")
