("a modifier" "")
("a series" "")
("abbreviation" "大略")
("about" "关於")
("about menu" "关於菜单")
("above" "以上")
("abstract" "大纲")
("accent" "")
("accept" "")
("accept all scripts" "")
("accept no scripts" "")
("accept script" "")
("acknowledgment" "感谢辞")
("acronym" "")
("action" "")
("activate" "")
("activate once" "")
("activate hidden" "")
("activation" "")
("acute" "")
("add package" "增加库")
("add switch after" "")
("add switch before" "")
("address" "住址")
("address block" "")
("advanced" "")
("advanced layout features" "")
("after" "之后")
("albanian" "")
("algorithm" "算法")
("align cell to the bottom" "")
("align cell to the left" "")
("align cell to the right" "")
("align cell to the top" "")
("align table at base of left column" "表格对齐左边")
("align table at base of middle column" "表格对齐中间")
("align table at base of origin column" "")
("align table at base of right column" "表格对齐右边")
("align table at fraction bar height" "")
("align table at the base of bottom row" "")
("align table at the base of middle row" "")
("align table at the base of origin row" "")
("align table at the base of top row" "")
("align table at the bottom" "")
("align table at the center" "")
("align table at the horizontal center" "")
("align table at the left" "")
("align table at the top" "")
("align table to the right" "")
("align text to the left" "")
("align text to the right" "")
("aligned" "")
("alignment" "整列")
("all" "")
("allow" "")
("allowed extension" "")
("allowed reduction" "")
("alpha release" "")
("alt key" "alt 键")
("alternative specification" "")
("american" "美语")
("animation" "动画")
("and" "")
("angular" "")
("appendix" "附录")
("apply" "使用")
("apply macro" "")
("apply macro once" "")
("april" "")
("arab" "")
("arch series" "")
("archaic" "")
("architecture" "")
("argument" "")
("arithmetic" "")
("armenian" "")
("arrow" "")
("arrows" "")
("article" "论文")
("as on paper" "")
("assign" "")
("at fraction bar height" "")
("at the baseline" "")
("at the bottom" "")
("at the left" "")
("at the middle" "")
("at the right" "")
("at the top" "")
("attach" "")
("attach buffer to which project" "")
("august" "")
("author" "作者")
("authorize" "")
("automatic" "自动")
("automatically close brackets" "")
("automatically generated content" "")
("automatic quotes" "")
("autosave" "自动保存")
("axis" "")
("axiom" "公理")
("b series" "")
("background" "")
("background color" "")
("background evaluations" "")
("backward replace" "")
("backward search" "")
("bad data format" "")
("bar" "")
("bard" "")
("base level" "")
("bashkirian" "")
("basic" "")
("basic communication using pipes" "")
("basic data types" "基本信息型态")
("basic types" "基本型态")
("be useful" "")
("become a TeXmacs developer" "成为TeXmacs的开发者")
("before" "")
("begin" "")
("below" "")
("beta release" "β版")
("bibliography" "参考文献")
("bibliography file" "参考文献文件")
("bibliography style" "参考文献的格式档")
("big" "大")
("big figure" "大图")
("big operator" "大的运算符号")
("big skip" "大的间距")
("big table" "大表格")
("binary operation" "2项运算")
("binary relation" "2项运算")
("black" "黑色")
("blackboard bold" "粗黑体")
("blackboard math" "")
("blackletter" "")
("blue" "蓝色")
("body" "")
("bold" "粗黑")
("book" "书籍")
("border" "")
("both" "")
("bottom" "")
("bottom border" "")
("bottom cell border" "")
("bottom cell padding" "")
("bottom margin" "")
("bottom padding" "")
("bottom table border" "")
("bottom table padding" "")
("branch" "分支")
("break" "断开")
("breaking" "")
("breve" "")
("british" "英语")
("broken white" "")
("brown" "")
("browse" "浏览")
("browse web" "")
("browser" "浏览程序")
("buffer" "")
("bulgarian" "保加力亚文")
("bullet" "")
("bullets" "")
("by" "")
("by typing" "")
("bytes free" "0 bytes ")
("bytes used" "使用 bytes 总数")
("calligraphic" "")
("cancel" "取消")
("caps-lock key" "")
("case" "")
("cc" "")
("cell" "")
("cell background color" "")
("cell border" "")
("cell height" "")
("cell operation mode" "")
("cell width" "")
("center" "中心")
("center text" "原文中间")
("centered" "")
("centered block" "")
("centered tabular" "")
("change border of cell" "")
("change cell operation mode" "")
("change log" "")
("changes" "")
("chapter" "章")
("check" "检查")
("check text for spelling errors" "检查拼写")
("chinese" "中文")
("choice" "选择")
("citation" "引用")
("cite" "")
("class" "")
("clear" "清除")
("clear selection" "")
("clipped" "")
("clipped and magnified" "")
("clipped and resized" "")
("clone" "")
("clone window" "一样的窗口")
("close" "退出")
("close TeXmacs" "退出 TeXmacs")
("close buffer" "退出暂存区")
("close session" "退出工作")
("close window" "关闭窗口")
("closing" "正在关闭中")
("closing style" "")
("code" "程序码")
("color" "颜色")
("column" "列")
("column contains glued items" "")
("common formats" "")
("compact" "")
("compactification" "")
("compactify" "")
("compatibility with other formats" "")
("compile article" "文章编译")
("compile book" "书籍编译")
("compile buffer" "")
("compile project" "")
("completion" "完成")
("complex" "")
("concrete" "")
("condensed" "")
("condition" "")
("configuration" "设置")
("contact" "")
("content" "属性")
("content tag" "属性标签")
("context dependent icons" "属性相关图标")
("continuation of" "延续")
("contribute" "参与")
("control the evaluation of expressions" "式子表现的控制")
("convention" "")
("converters to other data formats" "转换成其他格式")
("copy" "复制")
("copy text" "复制内文")
("copy to" "复制到")
("copying" "复制中")
("corollary" "系")
("correct text" "")
("create web site" "生成站点")
("cursors" "游标")
("custom tab" "")
("customizing TeXmacs" "调整TeXmacs ")
("customizing the TeXmacs styles" "调整TeXmacs格式")
("custum tab" "")
("cut" "剪切")
("cut text" "剪切属性")
("cut to" "剪切属性到")
("cypriot" "")
("cyrillic" "")
("cyrillic input method" "")
("cyrillic italic right" "")
("czech" "捷克语")
("dancers" "")
("danish" "丹麦语")
("dark grey" "")
("dash" "")
("dashes" "")
("data conversion" "")
("data format" "")
("date" "日期")
("deactivate" "")
("deactivate table" "")
("december" "")
("decorate atoms" "")
("default" "既定")
("default padding" "")
("definition" "定义")
("delete" "删除")
("delete column" "删除一行")
("delete row" "正在删除中")
("denominator" "分母")
("description" "描述")
("destination" "目标")
("detach" "")
("detailed" "详细")
("determinant" "行列式")
("devangari" "")
("developers guide" "开发者导引")
("directory" "目录")
("disable" "取消")
("disable first indentation" "")
("disable first indentation after" "")
("disable first indentation before" "")
("disable indentation" "")
("disable indentation after" "")
("disable indentation before" "")
("disactivate" "")
("disactivate once" "")
("disallow" "")
("distribute unused space" "")
("div" "")
("do not remap" "")
("document" "文书")
("document format" "文书格式")
("document style" "文书")
("document styles" "文书格式")
("documentation" "")
("documents are trees" "树状文书")
("dot" "")
("dots" "")
("double stroke" "")
("down" "")
("download" "下载")
("dpi" "")
("duplicate" "")
("dutch" "荷兰语")
("dynamic libraries" "")
("edit" "编辑")
("edit source tree" "编辑树状属性")
("editing tools" "编辑工具")
("email" "电子邮件")
("emphasize" "强调")
("emphasize text" "强调性属性")
("empty box" "空白的区域")
("enable first indentation" "")
("enable first indentation after" "")
("enable first indentation before" "")
("enable indentation" "")
("enable indentation after" "")
("enable indentation before" "")
("encl" "")
("end" "")
("english" "英语")
("enter a title" "输入标题")
("enumerate" "有序枚举")
("enumerate|Alpha" "")
("enumerate|Roman" "")
("enumerate|alpha" "")
("enumerate|roman" "")
("env" "")
("environment" "环境")
("epilogue" "")
("equal" "等於")
("equation" "方程序")
("equations" "方程序群")
("equivalent for" "")
("error" "错误")
("esperanto" "")
("etruscan" "")
("evaluate" "评价")
("evaluate scheme expression" "scheme表现评价")
("even page footer" "偶数页数注脚")
("even page header" "偶数页数标题")
("even page margin" "")
("even page left margin" "")
("even page shift" "")
("even page text" "")
("exact horizontal" "")
("exact mode" "")
("exam" "试卷")
("example" "例")
("exclusive" "排他的")
("exclusive or" "")
("executable" "可运行")
("execute" "运行")
("execute a TeXmacs command" "运行 TeXmacs 命令")
("execute system command" "运行系统命令")
("executive" "")
("exercise" "练习")
("expand" "展开")
("explained" "")
("export" "转换格式输出")
("export as" "转换成")
("extend or normal" "")
("extern " "")
("extra" "")
("extract format" "")
("family" "")
("february" "")
("figure" "图")
("file" "文件")
("file name" "文件名")
("file not found" "无此文件")
("find next match" "查找下一条信息")
("find text" "查找属性")
("finnish" "芬兰语")
("first contact" "")
("first indentation" "首次对齐位置")
("first item" "")
("first page" "")
("first start" "开始第一次")
("flat" "")
("flexibility" "弹性")
("flexibility factor for vertical spacing" "")
("floating figure" "浮动的图形")
("floating object" "浮动的对像")
("floating table" "浮动的表格")
("fold" "折叠")
("fold input field" "")
("folded" "可折叠的")
("font" "字体")
("font base size" "字体基本大小")
("font dpi" "")
("font family" "")
("font name" "")
("font series" "")
("font shape" "")
("font size" "")
("font type" "")
("font variant" "")
("fonts" "")
("footer" "")
("footnote" "注释")
("for" "")
("foreground" "")
("foreign" "")
("format" "格式")
("formatted and structured output" "")
("formatted date" "")
("formatting" "格式中")
("formula" "公式")
("formula style" "")
("forward replace" "")
("forward search" "")
("fraction" "分数")
("french" "法文")
("full manuals" "完全手册")
("full screen mode" "全屏方式")
("function" "")
("functional" "")
("further customization" "")
("future changes" "")
("gNU TeXmacs comes without any form of legal warranty" "GNU TeXmacs无任何保证")
("gNU TeXmacs falls under the GNU general public license" "GNU TeXmacs 使用 GNU 
general public license 授权.")
("general architecture of TeXmacs" "")
("general negation" "")
("general note" "")
("generating font file" "字体文件制作中")
("generic" "常规")
("georgian" "")
("german" "德文")
("getting started" "")
("glossary" "词汇")
("glossary entry" "")
("glue" "")
("glue decorations" "")
("go" "移到")
("gothic" "")
("graphical interface" "")
("grave" "")
("greater" "")
("greater or equal" "")
("greek" "")
("greek letter" "")
("green" "")
("grey" "")
("group" "")
("h modifier" "")
("happy typing" "")
("has been aborted" "")
("has completed its task" "")
("hat" "")
("header" "标题")
("header and footer" "标题与注释")
("hebrew" "")
("height" "高度")
("height is exactly the specified height" "")
("height is maximum of specified height and box height" "")
("height is minimum of specified height and box height" "")
("height magnification" "")
("help" "说明")
("help me" "")
("help us" "请帮助我们")
("here" "")
("hidden" "")
("home" "")
("homepage" "主页")
("horizontal" "")
("horizontal cell alignment" "")
("horizontal ink separation" "")
("horizontal part" "")
("horizontal position" "")
("horizontal space" "")
("horizontal table alignment" "")
("horizontally align cell to the base" "")
("horizontally center the cell" "")
("how much longer may pages become" "")
("how much shorter may pages become" "")
("huge" "")
("hungarian" "匈牙利语")
("hybrid command" "")
("hyperlink" "")
("hyphenation" "语法")
("icelandic" "")
("if" "")
("image" "图")
("image links" "")
("implementation" "")
("import" "输入其他格式文件")
("improving the current implementation" "")
("inactive" "")
("include" "")
("include a document" "")
("include file" "")
("inclusion into TeXmacs" "")
("inclusions" "")
("indent" "")
("indentation flag" "对齐标签")
("index" "索引")
("index entry" "")
("index level" "")
("informative flags" "")
("inline arguments" "")
("input mode" "")
("insert" "输入")
("insert Pr" "")
("insert a TeXmacs symbol" "")
("insert a big operator" "")
("insert a binary operation" "")
("insert a binary relation" "")
("insert a blackboard bold character" "")
("insert a bold calligraphic character" "")
("insert a bold character" "")
("insert a calligraphic character" "")
("insert a condition" "")
("insert a control flow instruction" "")
("insert a fraction" "")
("insert a fraktur character" "")
("insert a greek character" "")
("insert a large left delimiter" "")
("insert a large right delimiter" "")
("insert a large separator" "")
("insert a left subscript" "")
("insert a link" "")
("insert a miscellaneous symbol" "")
("insert a new branch" "")
("insert a new item" "")
("insert a numerical operation" "")
("insert a picture" "")
("insert a square root" "")
("insert a table" "")
("insert a table or a matrix" "")
("insert a textual operation" "")
("insert a wide symbol above" "")
("insert a wide symbol below" "")
("insert an" "")
("insert an accent" "")
("insert an arrow" "")
("insert an environment" "")
("insert an n" "")
("insert an operation on tuples" "")
("insert appendix" "")
("insert arccos" "")
("insert arcsin" "")
("insert arctan" "")
("insert arg" "")
("insert argument" "")
("insert author" "")
("insert automatically generated content" "")
("insert big contour integral" "")
("insert big coproduct" "")
("insert big direct sum" "")
("insert big dotted point" "")
("insert big integral" "")
("insert big intersection" "")
("insert big logical and" "")
("insert big logical or" "")
("insert big product" "")
("insert big square union" "")
("insert big summation" "")
("insert big tensor product" "")
("insert big union" "")
("insert big union sum" "")
("insert big vertical space" "")
("insert chapter" "")
("insert column" "")
("insert column to the left" "")
("insert column to the right" "")
("insert cos" "")
("insert cosh" "")
("insert cot" "")
("insert coth" "")
("insert csc" "")
("insert deg" "")
("insert det" "")
("insert dim" "")
("insert dollar" "")
("insert equation" "")
("insert executable markup" "")
("insert exp" "")
("insert fields" "")
("insert footnote" "")
("insert formula" "")
("insert gcd" "")
("insert hom" "")
("insert horizontal interword space" "")
("insert image" "")
("insert inactive markup" "")
("insert inf" "")
("insert input field above" "")
("insert input field below" "")
("insert ker" "")
("insert lg" "")
("insert lim" "")
("insert liminf" "")
("insert limsup" "")
("insert ln" "")
("insert log" "")
("insert macro" "")
("insert mathematics" "")
("insert max" "")
("insert medium horizontal space" "")
("insert medium vertical space" "")
("insert min" "")
("insert negative horizontal space" "")
("insert new item or number" "")
("insert new rows or columns" "")
("insert or delete rows or columns" "")
("insert row" "")
("insert row above" "")
("insert row below" "")
("insert sec" "")
("insert section" "输入节(section) ")
("insert sharp" "")
("insert sin" "")
("insert sinh" "")
("insert small horizontal space" "")
("insert small vertical space" "")
("insert structural markup" "")
("insert subsection" "")
("insert subsubsection" "")
("insert sup" "")
("insert tan" "")
("insert tanh" "")
("insert text field" "")
("insert thick horizontal space" "")
("insert title" "")
("insert value" "")
("interactive command" "")
("interfacing" "界面")
("interfacing TeXmacs with other systems" "")
("interjection" "")
("interline separation" "")
("interline space" "")
("internationalization" "")
("interparagraph space" "")
("interrupt execution" "")
("introduction" "")
("invisible" "")
("invisible citation" "")
("irish" "")
("is running" "")
("is tuple" "")
("ispell does not respond" "")
("ispell is already busy" "")
("italian" "意大利语")
("italic" "")
("italic right" "")
("itemize" "无序枚举")
("january" "")
("japanese" "日文")
("july" "")
("june" "")
("justified" "")
("justify text" "")
("key" "")
("keyboard" "键盘")
("keyboard equivalent" "")
("keyboard modifiers" "")
("keyboard shorthand" "")
("keymaps" "")
("keyword" "关键字")
("keywords" "关键语")
("kill" "")
("known bugs" "")
("label" "标签 (lebel)")
("landscape" "")
("language" "言语")
("large" "")
("large closing bracket" "")
("large delimiter" "")
("large opening bracket" "")
("large separator" "")
("last page" "")
("latex command" "")
("latin" "")
("layout" "")
("left" "")
("left arrow" "")
("left border" "左端")
("left cell border" "")
("left cell padding" "")
("left margin" "左端空白")
("left padding" "")
("left prime" "")
("left ragged" "")
("left slanted" "")
("left subscript" "")
("left superscript" "")
("left surrounding" "")
("left table border" "")
("left table padding" "")
("legal" "")
("lemma" "引理")
("length" "")
("less" "")
("less or equal" "")
("letter" "书信")
("license" "")
("light" "")
("light grey" "")
("limitations" "")
("limits" "")
("line" "")
("line break" "")
("line stretch" "")
("link" "链接")
("link image" "")
("list" "")
("list of figures" "图形目录")
("list of tables" "表格目录")
("load" "打开")
("load LaTeX file" "")
("load TeX file" "")
("load TeXmacs file" "")
("load TeXmacs style file" "")
("load a file" "")
("load buffer" "")
("load file" "")
("load html file" "")
("load image" "")
("load in new window" "")
("load latex file" "")
("load more recent autosave file" "打开最近自动保存的文件")
("load tex file" "打开 TeX 文件")
("load verbatim file" "")
("loaded" "")
("long" "")
("look and feel" "")
("look up" "")
("lower border" "下端")
("m modifier" "")
("macro" "宏命令")
("magenta" "")
("magic" "")
("magnification" "放大比率")
("magnified" "")
("main" "")
("main icon bar" "主菜单")
("major changes" "")
("make a page insertion" "")
("make a subscript" "")
("make a superscript" "")
("make acute" "")
("make bar" "")
("make check" "")
("make fraction" "")
("make grave" "")
("make hat" "")
("make label" "")
("make negation" "")
("make page reference" "")
("make reference" "")
("make square root" "")
("make tilde" "")
("make title" "表题制作")
("make vector" "")
("making donations" "")
("manual" "说明书")
("map to" "")
("march" "")
("margins" "空白端")
("margins as on paper" "")
("master" "")
("master file" "")
("math" "")
("math font" "")
("math font family" "")
("math font series" "")
("math font shape" "")
("mathematical" "")
("mathematical and customized input" "")
("mathematical font" "")
("mathematical formulas" "")
("mathematical input" "")
("mathematical typesetting" "")
("mathematics" "数学")
("matrix" "行列")
("maximal" "")
("maximal number of columns" "")
("maximal number of rows" "")
("maximum mode" "")
("may" "")
("medium" "")
("medium skip" "")
("meminfo" "")
("merge" "")
("messy" "")
("meta key" "")
("middle" "")
("minimal" "")
("minimal number of columns" "")
("minimal number of rows" "")
("minimum mode" "")
("minus" "")
("miscellaneous" "")
("miscellaneous features" "")
("mod" "")
("mode" "")
("modifier" "")
("modify cell alignment" "")
("modify cell border" "")
("modify cell padding" "")
("modify cell size" "")
("modify table border" "")
("modify table padding" "")
("mongolian" "")
("more information about GNU TeXmacs can be found in the Help" "进一步有关GNU 
TeXmacs 的信息怎可参考说明")
("mosaic" "")
("move down" "")
("move object" "")
("move to the right when finished" "")
("move up" "")
("multiline input" "")
("mutator tags" "")
("my documents" "")
("n-th" "")
("n-th root" "")
("name" "字体文件名")
("negation" "")
("new" "添加")
("new buffer" "")
("new column" "添加一行")
("new double page" "")
("new double page before" "")
("new item" "")
("new line" "添加一行")
("new number" "")
("new page" "添加一页")
("new page before" "")
("new paragraph" "")
("new roman" "")
("new row" "")
("new table" "")
("next" "下一个")
("next line" "")
("no changes need to be saved" "没有任何更改需要保存")
("no dictionary for" "")
("no first indentation" "")
("no indentation" "")
("no line break" "")
("no matches found" "")
("no matches found for" "")
("no more matches for" "")
("no more redo information available" "")
("no more undo information available" "")
("no name" "无标题文件")
("no page break after" "")
("no page break before" "")
("no spelling errors found in text" "")
("no style" "")
("none" "")
("normal" "")
("norwegian" "挪威文")
("not" "")
("not equal" "")
("not found" "")
("notation" "记法")
("note" "")
("november" "")
("number" "")
("number of columns" "列的数目")
("number equation" "")
("numbered equation" "")
("numbered equations" "")
("numerator" "分子")
("numeric" "")
("objectives" "")
("occurrences" "")
("october" "")
("odd page footer" "")
("odd page header" "")
("odd page margin" "")
("odd page left margin" "")
("odd page right margin" "")
("odd page shift" "")
("odd page text" "")
("off" "关")
("ok" "")
("old english" "")
("old slavonic" "")
("old style" "")
("on" "")
("on cells" "")
("on columns" "")
("on entire table" "")
("on rows" "")
("one spelling error has been corrected" "")
("online documentation" "")
("online help" "在线说明")
("only inline tags" "")
("open" "")
("open a new buffer" "")
("open new window" "新开窗口")
("opening" "")
("operation on named selection" "")
("operation on primary selection" "")
("operation on secondary selection" "")
("operation on ternary selection" "")
("optical" "")
("option" "")
("options" "")
("or" "")
("orange" "")
("orientation" "")
("origin" "")
("original welcome message" "")
("other" "其他")
("other pages" "")
("other projects" "")
("other style" "")
("outlined roman" "")
("over" "")
("overbrace" "")
("overline" "")
("padding" "")
("page" "页")
("page bottom margin" "")
("page break" "")
("page break before" "")
("page height" "")
("page insertion" "添加页")
("page layout" "")
("page number text" "")
("page numbering" "页数")
("page reference" "")
("page right margin" "")
("page setup" "设置页面")
("page top margin" "页面上縁")
("page type" "页面种类")
("page width" "页面宽长")
("paper" "")
("paper type" "")
("papyrus" "")
("paragraph" "段落")
("paragraph mode" "")
("paragraph width" "")
("part" "部分")
("past changes" "")
("paste" "粘贴")
("paste from" "从贴过来")
("paste text" "")
("path" "")
("perform operations on cells" "")
("perform operations on columns" "")
("perform operations on entire table" "")
("perform operations on rows" "")
("permanent" "")
("person" "")
("personal dictionary has been modified" "")
("phantasy" "")
("philosophy" "")
("phoenician" "")
("phonetic" "")
("pink" "")
("plain block" "")
("plain tabular" "")
("planned changes" "")
("plans for the future" "")
("please" "")
("please wait" "请稍后")
("plugin" "")
("plugins" "")
("plus" "")
("polish" "波兰语")
("porting TeXmacs to other platforms" "")
("portrait" "")
("portuguese" "葡萄牙语")
("position" "")
("position float" "")
("position floating object" "")
("position the table" "")
("postscript" "")
("postscript image" "")
("pre-alpha release" "")
("preamble" "")
("preamble mode" "")
("preferences" "设置(preferences)")
("presentation" "演讲")
("presentation mode" "演讲模式")
("presentation tag" "演讲标签")
("preview before printing" "")
("preview command" "")
("preview with ghostview" "")
("previous" "")
("primary" "")
("prime" "")
("print" "打印")
("print all" "全部打印")
("print all to file" "全部印成文件")
("print page selection" "选择打印")
("print page selection to file" "")
("print to postscript file" "印成 postscript 文件")
("printer" "打印机")
("printer dpi" "")
("printing command" "")
("problem" "问题")
("professional" "")
("prog" "")
("prog font" "")
("prog font family" "")
("prog font series" "")
("prog font shape" "")
("program" "")
("program font" "")
("program mode" "")
("programming" "")
("progressive" "")
("project" "项目")
("projects" "项目")
("prologue" "")
("prompt on scripts" "")
("prompts and default input" "")
("proof" "证明")
("proportional" "")
("proportional typewriter" "")
("proposition" "命题")
("query replace" "")
("quit" "")
("quotation" "引用")
("quote" "")
("range" "")
("really close it" "")
("really huge" "")
("really quit" "")
("recover autosave file" "复元自动保存文件")
("red" "")
("redo" "恢复")
("redo undone changes" "")
("reduce bottom margin by" "")
("reduce left margin by" "")
("reduce margins" "")
("reduce right margin by" "")
("reduce top margin by" "")
("reduction even page margin" "")
("reduction odd page margin" "")
("reduction page bottom margin" "")
("reduction page right margin" "")
("reduction page top margin" "")
("reference" "参考")
("reference guide" "参考导引")
("reference guide for the TeXmacs primitives" "")
("reference guide for the standard TeXmacs styles" "")
("references" "参考")
("regular" "")
("release" "")
("reload" "重新装入")
("remark" "注意")
("remove all output fields" "")
("remove fields" "")
("remove input field" "")
("remove input field above" "")
("remove number" "")
("remove package" "取消库")
("remove this switch" "")
("renumber this page" "")
("repeat" "")
("repeat object" "")
("replace" "替换")
("replace by" "被替换")
("replaced" "")
("replaced one occurrence" "")
("reset" "")
("resize object" "")
("resized" "")
("results of the search for" "")
("return" "")
("revert" "反向")
("revert buffer" "")
("right" "")
("right arrow" "")
("right border" "右端")
("right cell border" "")
("right cell padding" "")
("right margin" "右端")
("right padding" "")
("right ragged" "")
("right subscript" "")
("right superscript" "")
("right surrounding" "")
("right table border" "")
("right table padding" "")
("rigid" "")
("rigid horizontal" "")
("rigid space" "")
("romanian" "罗马尼亚语")
("root" "根")
("rotate backward" "")
("rotate forward" "")
("row" "")
("row contains glued items" "")
("runic" "")
("running author" "")
("running title" "")
("russian" "苏俄语")
("sample" "")
("sans serif" "")
("save" "存盘")
("save TeXmacs file" "存成 TeXmacs 档")
("save TeXmacs style file" "存成 TeXmacs 格式档")
("save as" "另存为")
("save as LaTeX" "另存为 LaTeX 文件")
("save as TeX" "另存为 TeX 文件")
("save buffer" "保存暂存盘")
("save buffer as" "另存暂存盘为")
("save file" "保存文件")
("save html file" "存为 html 文件")
("save image" "保存图形文件")
("save latex file" "保存 LaTeX 文件")
("save tex file" "保存TeX文件")
("save this buffer" "保存暂存区信息")
("save verbatim file" "存为  verbatim 文件")
("saved" "保存完成")
("scheme command" "")
("scheme extensions" "scheme 扩充")
("screen layout" "")
("screen margins" "")
("script" "直译程序")
("scripts" "直译程序")
("script above" "")
("script below" "")
("script script size" "")
("script size" "")
("scripts not accepted" "")
("search" "查找")
("search words in my documents" "")
("search words in the documentation" "")
("search words in the source code" "")
("searching" "")
("secondary" "")
("section" "节")
("security" "")
("see" "")
("select a document style" "")
("select a foreground color" "")
("select a language" "")
("selected region" "")
("selection" "")
("selections" "选择")
("seminar" "会议")
("sending commands to TeXmacs" "")
("separation between lines" "")
("separation between paragraphs" "")
("separator" "")
("september" "")
("serb" "")
("series" "种类")
("series|right" "")
("session" "进程")
("session name" "")
("session type" "")
("sessions" "进程")
("set" "")
("set background color of cell" "")
("set cell mode" "")
("set extension center" "")
("set height" "")
("set height of cell" "")
("set margins" "")
("set n" "")
("set or get environment variables" "")
("set paragraph margins" "")
("set paragraph mode" "")
("set printer dpi" "")
("set printing command" "")
("set span" "")
("set special cell properties" "")
("set special table properties" "")
("set text width" "")
("set width" "")
("set width of cell" "")
("settings" "")
("setup printer" "")
("shape" "型类")
("shift-down" "")
("shift-right" "")
("short" "")
("show" "")
("show header and footer" "")
("show hidden" "")
("show on screen" "")
("shrinking factor" "缩小比率")
("signature" "")
("size" "大小")
("size tag" "大小标签")
("size limits" "")
("slanted" "")
("sloppy" "")
("slovene" "")
("small" "")
("small caps" "")
("small figure" "小图")
("small skip" "")
("small table" "")
("source code" "属性码")
("source tags" "")
("space" "空白")
("space between lines" "")
("space between paragraphs" "")
("spacing" "空白作成法")
("spanish" "西班牙语")
("special" "特定")
("special cell properties" "")
("special table properties" "")
("specific" "指定")
("spell" "拼写")
("spell checking complete" "拼写检查完成")
("spelling error" "拼写错误")
("spelling errors have been corrected" "拼写错误更正完成")
("split" "")
("split session" "")
("square root" "平方根")
("standard environment variables" "")
("standard formats" "")
("standard TeXmacs styles" "")
("start" "开始")
("start a new chapter" "")
("start a new section" "")
("start an interactive session" "")
("status bar" "状态选栏")
("stretch" "")
("stretchable" "")
("stretchable horizontal" "")
("stretchable vertical" "")
("stretched" "")
("stroke" "")
("strokes" "")
("strong" "")
("structural" "")
("style" "风格")
("styles" "")
("stylesheet language" "")
("sub" "")
("subparagraph" "小段落")
("subscript" "下添字")
("subsection" "小节(sub)")
("subsub" "")
("subsubsection" "小小节(subsub)")
("subtitle" "")
("summary" "")
("summary of the principal TeXmacs tags" "")
("superpose" "")
("superposed" "")
("superscript" "上添字")
("swedish" "瑞典语")
("swiss" "")
("switch" "")
("switch to first" "")
("switch to formula style" "")
("switch to last" "")
("switch to next" "")
("switch to previous" "")
("switching and folding" "")
("symbol" "")
("system command" "")
("system environment variables" "")
("tab" "")
("table" "表")
("table command" "")
("table of contents" "目次")
("tables" "")
("tabular" "")
("tabular material" "")
("taiwanese" "台湾话")
("teXmacs as an interface" "")
("teXmacs fonts" "")
("teXmacs notice" "")
("teXmacs plugins" "")
("teXmacs style files" "")
("ternary" "")
("test" "捡测")
("tex" "")
("text" "文书")
("text font" "")
("text height correction" "")
("text width" "")
("texts" "")
("th century bc" "")
("th root" "")
("thai" "")
("the TeXmacs authors" "")
("the TeXmacs format" "")
("the application did not send its usual startup banner" "")
("the application does not reply" "")
("the application is already running" "")
("the boxes produced by the typesetter" "")
("the buffer has not been saved" "暂存区信息尚未保存")
("the graphical user interface" "")
("the leaves of TeXmacs trees" "")
("the mycas example" "")
("the primitive TeXmacs constructs" "")
("the standard TeXmacs styles" "")
("theorem" "定理")
("there are unsaved files" "尚有文件尚未保存保存")
("this buffer has not been saved" "现工作的暂存区信息尚未保存")
("this document has been produced using" "此文档由制作")
("this line" "")
("this page" "")
("this page footer" "")
("this page header" "")
("tilda" "")
("tilde" "")
("times" "")
("tiny" "")
("title" "标题")
("to the left" "")
("to the right" "")
("today" "本日")
("tools" "工具")
("top" "")
("top border" "")
("top cell border" "")
("top cell padding" "")
("top level" "")
("top margin" "")
("top padding" "")
("top table border" "")
("top table padding" "")
("translate" "")
("tree" "")
("tuple" "")
("turkish" "土耳其文")
("tutorial" "入门说明")
("type" "")
("type a latex command followed by" "")
("type body" "")
("type denominator" "")
("type numerator" "")
("type script" "")
("typesetted boxes" "")
("typewriter" "")
("typewriter small caps" "")
("typing a simple text" "")
("typing simple texts" "")
("ugaritic" "")
("ukrainian" "乌克兰语")
("underbrace" "")
("underline" "")
("undo" "放弃(undo)")
("undo last changes" "")
("up" "")
("update" "更新")
("update from web" "")
("update image links" "")
("upper border" "上端")
("upright" "")
("use TeXmacs" "")
("use a black color" "")
("use a blue color" "")
("use a bold font" "")
("use a brown color" "")
("use a calligraphic font" "")
("use a green color" "")
("use a grey color" "")
("use a magenta color" "")
("use a orange color" "")
("use a pink color" "")
("use a red color" "")
("use a sans serif font" "")
("use a typewriter font" "")
("use a white color" "")
("use a yellow color" "")
("use blackboard bold font" "")
("use bold font series" "")
("use calligraphical font" "")
("use footnote font size" "")
("use fraktur font" "")
("use huge font size" "")
("use italic font shape" "")
("use large font size" "")
("use left hand column as border" "")
("use medium font series" "")
("use normal font size" "")
("use package" "")
("use paragraph width" "")
("use really huge font size" "")
("use right font shape" "")
("use right hand column as border" "")
("use roman font family" "")
("use row above as border" "")
("use row below as border" "")
("use sans serif font family" "")
("use script font size" "")
("use slanted font shape" "")
("use small capitals" "")
("use small font size" "")
("use small-caps font shape" "")
("use the blackboard bold font" "")
("use the fraktur font" "")
("use tiny font font size" "")
("use typewriter font family" "")
("use very large font size" "")
("user manual" "")
("user provided icons" "用户提供图标")
("users of Cyrillic languages" "")
("value" "")
("variable" "")
("variant" "种类")
("vector" "")
("verbatim" "")
("verse" "韵文")
("vertical" "")
("vertical after" "")
("vertical before" "")
("vertical cell alignment" "")
("vertical part" "")
("vertical position" "")
("vertical space" "")
("vertical table alignment" "")
("vertically align cell to the base" "")
("vertically center the cell" "")
("very large" "")
("very small" "")
("vietnamese" "")
("view" "查看")
("visible" "")
("warning" "警告")
("web" "网络")
("welcome" "欢迎")
("welcome to GNU TeXmacs" "欢迎使用GNU TeXmacs")
("what is new" "")
("when finished" "")
("while" "")
("white" "")
("wide accent" "")
("wide bar" "")
("width" "宽")
("width is exactly the specified width" "")
("width is maximum of specified width and box width" "")
("width is minimum of specified width and box width" "")
("width magnification" "")
("window" "")
("window height" "")
("window width" "")
("windows key" "")
("with" "")
("write a macro" "")
("write a name" "")
("write bold mathematics" "")
("write bold text" "")
("write italic text" "")
("write sample text" "")
("write strong text" "")
("write typewriter text" "")
("write verbatim text" "")
("writing data converters" "")
("writing your own style files" "")
("written with TeXmacs" "使用 TeXmacs 制作")
("yellow" "")
("you can also obtain" "")
("your document is back in its original state" "")
("your first interface" "")
("your language" "使用语言")
