("GNU" "GNU")
("LaTeX" "LaTeX")
("TeX" "TeX")
("about" "σχετικά με τον TeXmacs")
("about menu" "μενού σχετικά")
("above" "παραπάνω")
("abstract" "περίληψη")
("accent" "τόνος")
("accept" "αποδοχή")
("accept all scripts" "αποδοχή όλων των σεναρίων")
("accept no scripts" "μη αποδοχή σεναρίων")
("accept script" "αποδοχή σεναρίου")
("action" "ενέργεια")
("activate" "ενεργοποίηση")
("acute" "οξεία")
("address" "διεύθυνση")
("address block" "τμήμα διεύθυνσης")
("advanced" "για προχωρημένους")
("after" "μετά")
("algorithm" "αλγόριθμος")
("align cell to the bottom" "στοίχιση κελιού κάτω")
("align cell to the left" "στοίχιση κελιού αριστερά")
("align cell to the right" "στοίχιση κελιού δεξιά")
("align cell to the top" "στοίχιση κελιού πάνω")
("align table at base of left column" "στοίχιση πίνακα στη βάση της αριστερής στήλης")
("align table at base of middle column" "στοίχιση πίνακα στη βάση της μεσαίας στήλης")
("align table at base of origin column" "στοίχιση πίνακα στη βάση της αρχικής στήλης")
("align table at base of right column" "στοίχιση πίνακα στη βάση της δεξιάς στήλης")
("align table at fraction bar height" "στοίχιση πίνακα στο ύψος της γραμμής κλάσματος")
("align table at the base of bottom row" "στοίχιση πίνακα στη βάση της κάτω σειράς")
("align table at the base of middle row" "στοίχιση πίνακα στη βάση της μεσαίας σειράς")
("align table at the base of origin row" "στοίχιση πίνακα στη βάση της αρχικής σειράς")
("align table at the base of top row" "στοίχιση πίνακα στη βάση της πάνω σειράς")
("align table at the bottom" "στοίχιση πίνακα κάτω")
("align table at the center" "στοίχιση πίνακα στο κέντρο")
("align table at the horizontal center" "στοίχιση πίνακα στο κέντρο οριζοντίως")
("align table at the left" "στοίχιση πίνακα αριστερά")
("align table at the top" "στοίχιση πίνακα πάνω")
("align table to the right" "στοίχιση πίνακα δεξιά")
("align text to the left" "στοίχιση κειμένου αριστερά")
("align text to the right" "στοίχιση κειμένου δεξιά")
("all" "όλα")
("allow" "αποδοχή")
("allowed extension" "επιτρεπτή επέκταση")
("allowed reduction" "επιτρεπτή αναγωγή")
("alpha release" "δοκιμαστική έκδοση alpha")
("and" "και")
("appendix" "παράρτημα")
("apply" "εφαρμογή")
("april" "απρίλιος")
("arab" "αραβικά")
("archaic" "archaic")
("architecture" "αρχιτεκτονική")
("arithmetic" "αριθμητικό")
("armenian" "αρμενικά")
("arrow" "βέλος")
("article" "άρθρο")
("as on paper" "όπως στο χαρτί")
("assign" "ανάθεση")
("at fraction bar height" "στο ύψος της γραμμής κλάσματος")
("at the baseline" "στη γραμμή βάσης") 
("at the bottom" "κάτω")
("at the left" "αριστερά")
("at the middle" "στη μέση")
("at the right" "δεξιά")
("at the top" "πάνω")
("attach" "επισύναψη")
("attach buffer to which project" "επισύναψη buffer στο project...")
("august" "αύγουστος")
("author" "συγγραφέας")
("authorize" "εξουσιοδότηση")
("automatic" "αυτοματοποιημένα")
("axiom" "αξίωμα")
("background color" "χρώμα παρασκηνίου")
("backward replace" "αντικατάσταση προς τα πίσω")
("backward search" "αναζήτηση προς τα πίσω")
("bad data format" "λανθασμένη μορφή δεδομένων")
("bar" "μπάρα")
("bard" "bard")
("base level" "επίπεδο βάσης")
("bashkirian" "bashkirian")
("basic" "βασικό")
("basic types" "βασικοί τύποι")
("be useful" "να'ναι χρήσιμο")
("before" "πριν")
("begin" "ξεκίνα")
("below" "από κάτω")
("beta release" "δοκιμαστική έκδοση beta")
("bibliography" "βιβλιογραφία")
("bibliography file" "αρχείο βιβλιογραφίας")
("bibliography style" "στυλ βιβλιογραφίας")
("big" "μεγάλο")
("big figure" "μεγάλο σχήμα")
("big operator" "μεγάλος τελεστής")
("big skip" "μεγάλο κενό")
("big table" "μεγάλος πίνακας")
("binary operation" "δυαδική πράξη")
("binary relation" "δυαδική σχέση")
("black" "μαύρο")
("blackboard bold" "έντονα μαυροπίνακα")
("blackboard math" "μαθηματικά μαυροπίνακα")
("blackletter" "blackletter")
("blue" "μπλε")
("body" "σώμα")
("bold" "έντονο")
("book" "βιβλίο")
("border" "περίγραμμα")
("both" "και τα δύο")
("bottom" "κάτω")
("bottom border" "κάτω περίγραμμα")
("bottom margin" "κάτω περιθώριο")
("bottom padding" "κάτω γέμισμα")
("branch" "κλάδος")
("break" "αλλαγή γραμμής")
("breaking" "αλλαγή γραμμής")
("breve" "βραχύ")
("brown" "καφέ")
("buffer" "buffer")
("by" "με")
("by typing" "με πληκτρολόγηση")
("bytes free" "bytes ελεύθερα")
("bytes used" "bytes χρησιμοποιημένα")
("calligraphic" "καλλιγραφικά")
("cancel" "ακύρωση")
("case" "πεζά/κεφαλαία")
("cell" "κελί")
("cell background color" "χρώμα παρασκηνίου κελιού")
("cell border" "περίγραμμα κελιού")
("cell height" "ύψος κελιού")
("cell operation mode" "λειτουργία κελιού")
("cell width" "πλάτος κελιού")
("center" "κέντρο")
("center text" "κεντράρισμα κειμένου")
("centered" "κεντραρισμένο")
("centered block" "κεντραρισμένο τμήμα")
("centered tabular" "κεντrαρισμένος πίνακας δεδομένων")
("change border of cell" "αλλαγή περιγράμματος κελιού")
("change cell operation mode" "αλλαγή λειτουργίας κελιού")
("changes" "αλλαγές")
("chapter" "κεφάλαιο")
("check" "έλεγχος")
("check text for spelling errors" "ορθογραφικός έλεγχος κειμένου")
("chinese" "κινέζικα")
("choice" "επιλογή")
("citation" "παραπομπή")
("class" "κλάση")
("clear" "εκκαθάριση")
("clear selection" "εκκαθάριση επιλογής")
("clipped" "αποκομμένο")
("clipped and magnified" "αποκομμένο και μεγεθυμένο")
("clipped and resized" "αποκομμένο και με αλλαγμένο μέγεθος")
("clone" "κλωνοποίηση")
("clone window" "κλωνοποίηση παραθύρου")
("close" "κλείσιμο")
("close TeXmacs" "κλείσιμο TeXmacs")
("close buffer" "κλείσιμο buffer")
("close session" "κλείσιμο συνόδου")
("close window" "κλείσιμο παραθύρου")
("code" "κώδικας")
("color" "χρώμα")
("column" "στήλη")
("column contains glued items" "η στήλη περιέχει εφαπτόμενα αντικείμενα") 
("compile buffer" "μεταγλώττιση buffer")
("compile project" "μεταγλώττιση project")
("complex" "μιγαδικός")
("concrete" "concrete")
("condensed" "συμπυκνωμένο")
("condition" "προϋπόθεση")
("context dependent icons" "εικονίδια εξαρτόμενα από τα συμφραζόμενα")
("continuation of" "συνέχεια")
("convention" "σύμβαση")
("copy" "αντιγραφή")
("copy text" "αντιγραφή κειμένου")
("copy to" "αντιγραφή σε")
("copying" "αντιγραφή")
("corollary" "πόρισμα")
("correct text" "διόρθωση κειμένου")
("cursors" "δείκτες")
("cut" "αποκοπή")
("cut text" "αποκοπή κειμένου")
("cypriot" "cypriot")
("cyrillic" "κυριλλικά")
("cyrillic italic right" "κυριλλικά κεκλιμένα")
("czech" "τσέχικα")
("dancers" "χορευτές")
("data conversion" "μετατροπή δεδομένων")
("data format" "μορφή δεδομένων")
("date" "ημερομηνία")
("deactivate table" "απενεργοποίηση πίνακα")
("december" "δεκέμβριος")
("default" "προεπιλεγμένο")
("default padding" "προεπιλεγμένο γέμισμα")
("definition" "ορισμός")
("delete" "διαγραφή")
("delete column" "διαγραφή στήλης")
("delete row" "διαγραφή σειράς")
("denominator" "παρονομαστής")
("description" "περιγραφή")
("detach" "απόσπαση")
("determinant" "προσδιοριστικός")
("devangari" "devangari")
("directory" "κατάλογος")
("disable first indentation" "απενεργοποίηση πρώτης εσοχής")
("disable first indentation after" "απενεργοποίηση πρώτης εσοχής μετά")
("disable first indentation before" "απενεργοποίηση πρώτης εσοχής πριν")
("disable indentation" "απενεργοποίηση εσοχής")
("disable indentation after" "απενεργοποίηση πρώτης εσοχής μετά")
("disable indentation before" "απενεργοποίηση πρώτης εσοχής πριν")
("disable last indentation" "απενεργοποίηση τελευταίας εσοχής")
("disable last indentation after" "απενεργοποίηση τελευταίας εσοχής μετά")
("disable last indentation before" "απενεργοποίηση τελευταίας εσοχής πριν")
("deactivate" "απενεργοποίηση")
("disallow" "απαγόρευση")
("distribute unused space" "διανομή αχρησιμοποίητου χώρου")
("div" "div")
("document" "έγγραφο")
("document style" "στυλ εγγράφου")
("document styles" "στυλ εγγράφου")
("dot" "κουκκίδα")
("double stroke" "διπλή γραμμή")
("down" "κάτω")
("dpi" "dpi")
("duplicate" "διπλασιασμός")
("dutch" "ολλανδικά")
("edit" "επεξεργασία")
("emphasize text" "έμφαση κειμένου")
("empty box" "άδειο κουτί")
("enable first indentation" "ενεργοποίηση πρώτης εσοχής")
("enable first indentation after" "ενεργοποίηση πρώτης εσοχής μετά")
("enable first indentation before" "ενεργοποίηση πρώτης εσοχής πριν")
("enable indentation" "ενεργοποίηση εσοχής")
("enable indentation after" "ενεργοποίηση εσοχής μετά")
("enable indentation before" "ενεργοποίηση εσοχής πριν")
("enable last indentation" "ενεργοποίηση τελευταίας εσοχής")
("enable last indentation after" "ενεργοποίηση τελευταίας εσοχής μετά")
("enable last indentation before" "ενεργοποίηση τελευταίας εσοχής πριν")
("end" "τέλος")
("english" "αγγλικά")
("enter a title" "εισαγωγή τίτλου")
("enumerate" "απαρίθμηση")
("enumerate|Alpha" "απαρίθμηση|Alpha")
("enumerate|Roman" "απαρίθμηση|Roman")
("enumerate|alpha" "απαρίθμηση|alpha")
("enumerate|roman" "απαρίθμηση|roman")
("env" "env")
("environment" "περιβάλλον")
("epilogue" "επίλογος")
("equal" "ίσον")
("equation" "εξίσωση")
("equations" "εξισώσεις")
("error" "σφάλμα")
("etruscan" "etruskan")
("evaluate" "υπολογισμός")
("evaluate scheme expression" "υπολογισμός έκφρασης scheme")
("even page margin" "περιθώριο άρτιας σελίδας") 
("exact horizontal" "ακριβώς οριζόντιο")
("exact mode" "ακριβής λειτουργία")
("example" "παράδειγμα")
("exclusive or" "αποκλειστικό ή")
("executable" "εκτελέσιμο")
("execute" "εκτέλεση")
("execute a TeXmacs command" "εκτέλεση εντολής TeXmacs") 
("execute system command" "εκτέλεση εντολής συστήματος")
("executive" "εκτελεστικός")
("exercise" "άσκηση")
("explained" "εξηγημένο")
("export" "εξαγωγή")
("export as" "εξαγωγή ως")
("extend or normal" "επέκταση ή κανονικό")
("extern " "extern")
("extract format" "εξαγωγή μορφής")
("family" "οικογένεια")
("february" "Φεβρουάριος")
("figure" "εικόνα")
("file" "αρχείο")
("file name" "όνομα αρχείου")
("file not found" "το αρχείο δεν βρέθηκε")
("find next match" "εύρεση επόμενης αντιστοιχίας")
("find text" "εύρεση κειμένου")
("first indentation" "πρώτη εσοχή")
("first item" "πρώτο αντικείμενο")
("first page" "πρώτη σελίδα")
("flat" "επίπεδο")
("flexibility" "προσαρμοστικότητα")
("flexibility factor for vertical spacing" "συντελεστής προσαρμοστικότητας για τα κάθετα περιθώρια")
("floating figure" "μετακινούμενο σχήμα")
("floating object" "μετακινούμενο αντικείμενο")
("floating table" "μετακινούμενος πίνακας")
("font" "γραμματοσειρά")
("font base size" "βασικό μέγεθος γραμματοσειράς")
("font dpi" "dpi γραμματοσειράς")
("font family" "οικογένεια γραμματοσειράς")
("font series" "σειρά γραμματοσειράς")
("font shape" "σχήμα γραμματοσειράς")
("font size" "μέγεθος γραμματοσειράς")
("fonts" "γραμματοσειρές")
("footnote" "υποσημείωση")
("for" "for")
("foreground" "προσκήνιο")
("foreign" "ξένες")
("formatted date" "μορφοποιημένη ημερομηνία")
("formula" "μαθηματικός τύπος")
("formula style" "στυλ μαθηματικού τύπου")
("forward replace" "αντικατάσταση προς τα εμπρός")
("forward search" "εύρεση προς τα εμπρός")
("fraction" "κλάσμα")
("french" "γαλλικά")
("function" "λειτουργία")
("future changes" "μελλοντικές αλλαγές")
("gNU TeXmacs comes without any form of legal warranty" "ο gNU TeXmacs δεν καλύπτεται από κάποια νόμιμη εγγύηση")
("gNU TeXmacs falls under the GNU general public license" "ο gNU TeXmacs υπόκειται στην GNU general public license")
("general negation" "γενική άρνηση")
("generating font file" "δημιουργία αρχείου γραμματοσειράς")
("georgian" "γεωργιανά")
("german" "γερμανικά")
("getting started" "αρχίζοντας...")
("glossary" "γλωσσάρι")
("glossary entry" "όρος γλωσσαρίου")
("glue" "κόλληση")
("glue decorations" "επικόλληση διακόσμησης")
("gothic" "γοτθικά")
("graphical interface" "γραφικό περιβάλλον")
("grave" "βαρεία")
("greater" "μεγαλύτερο")
("greater or equal" "μεγαλύτερο ή ίσο")
("greek" "ελληνικά")
("greek letter" "ελληνικό γράμμα")
("green" "πράσινο")
("grey" "γκρι")
("group" "ομάδα")
("happy typing" "καλή πληκτρολόγηση") καλή πληκτρολόγηση
("has been aborted" "έχει ακυρωθεί")
("has completed its task" "έχει ολοκληρώσει την εργασία του")
("hat" "καρρέ")
("header and footer" "κεφαλίδα και υποσέλιδο")
("hebrew" "εβραϊκά")
("height" "ύψος")
("height is exactly the specified height" "το ύψος ταυτίζεται με το δοσμένο")
("height is maximum of specified height and box height" "το ύψος είναι το μέγιστο από: δοσμένο και ύψος πλαισίου")
("height is minimum of specified height and box height" "το ύψος είναι το ελάχιστο από: δοσμένο και ύψος πλαισίου")
("height magnification" "πρόσθεση ύψους")
("help" "βοήθεια")
("help me" "βοήθησέ με")
("here" "εδώ")
("home" "αφετηρία")
("horizontal" "οριζόντιο")
("horizontal cell alignment" "στοίχιση κελιού οριζοντίως")
("horizontal ink separation" "διαχωρισμός ink οριζοντίως")
("horizontal part" "οριζόντιο τμήμα")
("horizontal position" "οριζόντια θέση")
("horizontal space" "οριζόντιος κενός χώρος")
("horizontal table alignment" "στοίχιση πίνακα οριζοντίως")
("horizontally align cell to the base" "στοίχιση κελιού οριζοντίως στη βάση")
("horizontally center the cell" "κεντράρισμα κελιού οριζοντίως")
("how much longer may pages become" "δυνατότητα επιμήκυνσης σελιδών")
("how much shorter may pages become" "δυνατότητα σμίκρυνσης σελιδών")
("huge" "τεράστιο")
("hungarian" "ουγγρικά")
("hyperlink" "υπερδεσμός")
("hyphenation" "συλλαβισμός")
("icelandic" "ισλανδικά")
("if" "if")
("image" "εικόνα")
("image links" "σύνδεσμοι με εικόνα")
("implementation" "υλοποίηση") υλοποίηση
("import" "εισαγωγή")
("inactive" "ανενεργό")
("include" "συμπερίληψη")
("include a document" "συμπερίληψη εγγράφου")
("include file" "συμπερίληψη αρχείου")
("inclusions" "συμπεριλήψεις")
("indent" "εσοχή")
("indentation flag" "σημαία εσοχής")
("index" "ευρετήριο")
("index entry" "λήμμα ευρετηρίου")
("index level" "επίπεδο ευρετηρίου")
("insert" "εισαγωγή")
("insert Pr" "εισαγωγή Pr")
("insert a TeXmacs symbol" "εισαγωγή συμβόλου του TeXmacs")
("insert a big operator" "εισαγωγή μεγάλου τελεστή")
("insert a binary operation" "εισαγωγή δυαδικής πράξης")
("insert a binary relation" "εισαγωγή δυαδικής σχέσης")
("insert a fraction" "εισαγωγή κλάσματος")
("insert a greek character" "εισαγωγή ελληνικού χαρακτήρα")
("insert a large left delimiter" "εισαγωγή μεγάλου αριστερού οροθέτη")
("insert a large right delimiter" "εισαγωγή μεγάλου δεξιού οροθέτη")
("insert a large separator" "εισαγωγή μεγάλου διαχωριστικού")
("insert a left subscript" "εισαγωγή αριστερού δείκτη")
("insert a link" "εισαγωγή συνδέσμου")
("insert a miscellaneous symbol" "εισαγωγή συμβόλου από τα διάφορα")
("insert a new branch" "εισαγωγή νέου κλάδου")
("insert a new item" "εισαγωγή νέου αντικειμένου")
("insert a picture" "εισαγωγή εικόνας")
("insert a square root" "εισαγωγή τετραγωνικής ρίζας")
("insert a table" "εισαγωγή πίνακα")
("insert a table or a matrix" ""εισαγωγή πίνακα ή μαθηματικού πίνακα)
("insert an" "εισαγωγή")
("insert an accent" "εισαγωγή τόνου")
("insert an arrow" "εισαγωγή βέλους")
("insert an environment" "εισαγωγή περιβάλλοντος")
("insert an n" "εισαγωγή ν") 
("insert appendix" "εισαγωγή παραρτήματος")
("insert arccos" "εισαγωγή arccos")
("insert arcsin" "εισαγωγή arcsin")
("insert arctan" "εισαγωγή arctan")
("insert arg" "εισαγωγή arg")
("insert argument" "εισαγωγή επιχειρήματος")
("insert automatically generated content" "εισαγωγή αυτοματοποιημένων περιεχομένων")
("insert big contour integral" "εισαγωγή μεγάλου επικαμπύλιου ολοκληρώματος")
("insert big coproduct" "εισαγωγή μεγάλου εσωτερικού συγγινομένου") 
("insert big direct sum" "εισαγωγή μεγάλου ευθέος αθροίσματος")
("insert big dotted point" "εισαγωγή μεγάλου σημείου") 
("insert big integral" "εισαγωγή μεγάλου ολοκληρώματος")
("insert big intersection" "εισαγωγή μεγάλης διχοτόμησης")
("insert big logical and" "εισαγωγή μεγάλου λογικού και")
("insert big logical or" "εισαγωγή μεγάλου λογικού ή")
("insert big product" "εισαγωγή μεγάλου γινομένου")
("insert big square union" "εισαγωγή μεγάλης τετραγωνική ένωσης") 
("insert big summation" "εισαγωγή μεγάλου αθροίσματος")
("insert big tensor product" "εισαγωγή μεγάλου τανυστικού γινομένου")
("insert big union" "εισαγωγή μεγάλης ένωσης")
("insert big union sum" "εισαγωγή μεγάλου αθροίσματος ενώσεων") 
("insert big vertical space" "εισαγωγή μεγάλου κατακόρυφου κενού") 
("insert chapter" "εισαγωγή κεφαλαίου")
("insert column" "εισαγωγή στήλης")
("insert column to the left" "εισαγωγή στήλης αριστερά")
("insert column to the right" "εισαγωγή στήλης δεξιά")
("insert cos" "εισαγωγή cos")
("insert cosh" "εισαγωγή cosh")
("insert cot" "εισαγωγή cot")
("insert coth" "εισαγωγή coth")
("insert csc" "εισαγωγή csc")
("insert deg" "εισαγωγή deg")
("insert det" "εισαγωγή det")
("insert dim" "εισαγωγή dim")
("insert dollar" "εισαγωγή συμβόλου δολλαρίου")
("insert equation" "εισαγωγή εξίσωσης")
("insert executable markup" "εισαγωγή εκτελέσιμου markup")
("insert exp" "εισαγωγή exp")
("insert footnote" "εισαγωγή υποσημείωσης")
("insert formula" "εισαγωγή μαθηματικού τύπου")
("insert gcd" "εισαγωγή gcd")
("insert hom" "εισαγωγή hom")
("insert horizontal interword space" "εισαγωγή οριζόντιου κενού μεταξύ των λέξεων")
("insert image" "εισαγωγή εικόνας")
("insert inactive markup" "εισαγωγή ανενεργού markup")
("insert inf" "εισαγωγή inf")
("insert ker" "εισαγωγή ker")
("insert lg" "εισαγωγή lg")
("insert lim" "εισαγωγή lim")
("insert liminf" "εισαγωγή liminf")
("insert limsup" "εισαγωγή limsup")
("insert ln" "εισαγωγή ln")
("insert log" "εισαγωγή log")
("insert mathematics" "εισαγωγή μαθηματικών")
("insert max" "εισαγωγή μεγίστου")
("insert medium horizontal space" "εισαγωγή μέσου κενού οριζοντίως")
("insert medium vertical space" "εισαγωγή μέσου κατακόρυφου κενού")
("insert min" "εισαγωγή ελαχίστου")
("insert negative horizontal space" "εισαγωγή αρνητικού κενού οριζοντίως")
("insert new item or number" "εισαγωγή νέου στοιχείου ή αρίθμησης")
("insert new rows or columns" "εισαγωγή νέων σειρών/στηλών")
("insert or delete rows or columns" "εισαγωγή ή διαγραφή σειρών/στηλών")
("insert row" "εισαγωγή σειράς")
("insert row above" "εισαγωγή σειράς πάνω")
("insert row below" "εισαγωγή σειράς κάτω")
("insert sec" "εισαγωγή sec")
("insert section" "εισαγωγή ενότητας")
("insert sharp" "εισαγωγή δίεσης")
("insert sin" "εισαγωγή sin")
("insert sinh" "εισαγωγή sinh")
("insert small horizontal space" "εισαγωγή μικρού κενού οριζοντίως")
("insert small vertical space" "εισαγωγή μικρού κατακόρυφου κενού") 
("insert structural markup" "εισαγωγή structural markup")
("insert subsection" "εισαγωγή υποενότητας")
("insert subsubsection" "εισαγωγή υπό-υποενότητας")
("insert sup" "εισαγωγή sup")
("insert tan" "εισαγωγή tan")
("insert tanh" "εισαγωγή tanh")
("insert thick horizontal space" "εισαγωγή χοντρού κενού οριζοντίως")
("interjection" "παρεμβολή")
("interline separation" "διάστιχο")
("interline space" "απόσταση γραμμών") 
("interparagraph space" "απόσταση παραγράφων")
("interrupt execution" "εκτέλεση διακοπής")
("introduction" "εισαγωγή")
("invisible" "αόρατο")
("invisible citation" "αόρατη παραπομπή")
("irish" "ιρλανδικά")
("is running" "τρέχει")
("is tuple" "είναι tuple")
("ispell does not respond" "το ispell δεν ανταποκρίνεται")
("ispell is already busy" "το ispell είναι απασχολημένο")
("italian" "ιταλικά")
("italic" "πλάγια")
("italic right" "πλάγια δεξιά")
("itemize" "σημειακή απαρίθμηση")
("january" "ιανουάριος")
("japanese" "ιαπωνικά")
("july" "ιούλιος")
("june" "ιούνιος")
("justified" "πλήρως ευθυγραμισμένο")
("justify text" "πλήρης ευθυγράμμιση κειμένου")
("keyboard" "πληκτρολόγιο")
("keyboard equivalent" "αντίστοιχο πλήκτρο")
("keyboard shorthand" "πλήκτρο συντόμευσης")
("keymaps" "keymaps")
("kill" "εξουδετέρωση")
("known bugs" "γνωστά bugs")
("label" "ετικέτα")
("landscape" "τοπίο")
("language" "γλώσσα")
("large" "μεγάλο")
("large closing bracket" "μεγάλη δεξιά αγκύλη")
("large delimiter" "μεγάλος οροθέτης")
("large opening bracket" "μεγάλη αριστερή αγκύλη")
("large separator" "μεγάλο διαχωριστικό")
("last indentation" "τελευταία εσοχή")
("last page" "τελευταία σελίδα")
("latex command" "εντολή latex")
("latin" "latin")
("layout" "προσχέδιο")
("left" "αριστερά")
("left border" "αριστερό περίγραμμα")
("left margin" "περιθώριο αριστερά")
("left padding" "αριστερό γέμισμα")
("left prime" "left prime")
("left ragged" "τραβηγμένη αριστερά")
("left slanted" "κλίση προς αριστερά")
("left subscript" "αριστερός δείκτης")
("left superscript" "αριστερός εκθέτης")
("left surrounding" "αριστερό περιστοίχισμα")
("legal" "legal")
("lemma" "λήμμα")
("length" "μήκος")
("less" "μικρότερο")
("less or equal" "μικρότερο ή ίσο")
("letter" "letter") letter % είναι μέγεθος χαρτιού
("license" "άδεια")
("light" "ελαφρύ")
("limitations" "περιορισμοί")
("limits" "όρια")
("line" "γραμμή")
("line break" "αλλαγή γραμμής")
("line stretch" "επέκταση γραμμής")
("link" "σύνδεσμος")
("link image" "σύνδεσμος με εικόνα")
("list" "λίστα")
("list of figures" "λίστα με εικόνες")
("list of tables" "λίστα με πίνακες")
("load" "φόρτωμα")
("load LaTeX file" "φόρτωμα αρχείου LaTeX")
("load TeX file" "φόρτωμα αρχείου TeX")
("load TeXmacs file" "φόρτωμα αρχείου TeXmacs")
("load TeXmacs style file" "φόρτωμα αρχείου στυλ TeXmacs")
("load a file" "φόρτωμα αρχείου")
("load buffer" "φόρτωμα buffer")
("load file" "φόρτωμα αρχείου")
("load html file" "φόρτωμα αρχείου html")
("load image" "φόρτωμα εικόνας")
("load in new window" "φόρτωμα σε νέο παράθυρο")
("load latex file" "φόρτωμα αρχείου latex")
("load more recent autosave file" "φόρτωμα πιο πρόσφατου αυτοαποθηκευμένου αρχείου")
("load tex file" "φόρτωμα αρχείου tex")
("load verbatim file" "φόρτωμα αρχείου verbatim")
("loaded" "φορτώθηκε")
("long" "μακρύ")
("look up" "ανεύρεση")
("lower border" "κάτω περίγραμμα")
("macro" "μακροεντολή")
("magenta" "μωβ")
("magic" "μαγικό")
("magnification" "magnification")
("magnified" "μεγεθυμένο")
("main" "κύριο")
("main icon bar" "κύρια εργαλειοθήκη")
("make a page insertion" "εισαγωγή σελίδας")
("make a subscript" "δείκτης")
("make a superscript" "εκθέτης")
("make acute" "εισαγωγή οξείας")
("make bar" "μπάρα")
("make check" "έλεγχος")
("make fraction" "κλάσμα")
("make grave" "βαρεία")
("make hat" "καρρέ")
("make label" "ετικέτα")
("make negation" "άρνηση")
("make page reference" "αναφορά σε σελίδα")
("make reference" "αναφορά")
("make square root" "τετραγωνική ρίζα")
("make tilde" "περισπωμένη")
("make title" "τίτλος")
("make vector" "διάνυσμα")
("march" "Μάρτιος")
("margins" "περιθώρια")
("master" "κύριο")
("master file" "κύριο αρχείο")  
("math" "μαθηματικά")
("math font" "μαθηματική γραμματοσειρά")
("math font family" "οικογένεια μαθηματικής γραμματοσειράς")
("math font series" "σειρά μαθηματικής γραμματοσειράς")
("math font shape" "σχήμα μαθηματικής γραμματοσειράς")
("mathematical" "μαθηματικό")
("mathematics" "μαθηματικά")
("matrix" "μαθηματικός πίνακας")
("maximal number of columns" "μέγιστος αριθμός στηλών")
("maximal number of rows" "μέγιστος αριθμός σειρών")
("maximum mode" "μέγιστη λειτουργία")
("may" "μάιος")
("medium" "μεσαίο")
("medium skip" "κενό μεσαίου μεγέθους")
("meminfo" "meminfo")
("merge" "συγχώνευση")
("messy" "messy")
("middle" "μεσαίο")
("minimal number of columns" "ελάχιστος αριθμός στηλών")
("minimal number of rows" "ελάχιστος αριθμός σειρών")
("minimum mode" "ελάχιστη λειτουργία")
("minus" "μείον")
("miscellaneous" "διάφορα")
("mod" "mod")
("mode" "λειτουργία")
("modify cell alignment" "τροποποίηση στοίχισης κελιού")
("modify cell border" "τροποποίηση περιγράμματος κελιού")
("modify cell padding" "τροποποίηση γεμίσματος κελιού")
("modify cell size" "τροποποίηση μεγέθους κελιού")
("modify table border" "τροποποίηση περιγράμματος πίνακα")
("modify table padding" "τροποποίηση γεμίσματος πίνακα")
("mongolian" "μογγολικά")
("more information about GNU TeXmacs can be found in the help" "περισσότερες πληροφορίες για τον GNU TeXmacs στην βοήθεια")
("mosaic" "mosaic") 
("move down" "μετακίνηση κάτω")
("move object" "μετακίνηση αντικειμένου")
("move to the right when finished" "μετακίνηση δεξιά μετά την ολοκλήρωση")
("move up" "μετακίνηση πάνω")
("n-th" "ν-οστή")
("n-th root" "ν-οστή ρίζα")
("name" "όνομα")
("negation" "άρνηση")
("new" "νέο")
("new buffer" "νέος buffer") νέος buffer
("new column" "νέα στήλη")
("new item" "νέο στοιχείο")
("new line" "νέα γραμμή")
("new number" "νέος αριθμός")
("new page" "νέα σελίδα")
("new paragraph" "νέα παράγραφος")
("new roman" "new roman")
("new row" "νέα σειρά")
("new table" "νέος πίνακας")
("next line" "επόμενη γραμμή")
("no changes need to be saved" "καμμία αλλαγή προς αποθήκευση")
("no dictionary for" "απουσία λεξικού")
("no first indentation" "χωρίς πρώτη εσοχή")
("no last indentation" "χωρίς τελευταία εσοχή")
("no line break" "χωρίς αλλαγή της γραμμής")
("no matches found" "δεν βρέθηκαν αντιστοιχίες")
("no more matches for" "δεν βρέθηκαν άλλες αντιστοιχίας για το")
("no more redo information available" "το ιστορικό επαναλήψεων εξαντλήθηκε")
("no more undo information available" "το ιστορικό αναιρέσεων εξαντλήθηκε")
("no name" "χωρίς όνομα")
("no page break after" "χωρίς αλλαγή σελίδας μετά")
("no page break before" "χωρίς αλλαγή σελίδας πριν")
("no spelling errors found in text" "δεν βρέθηκαν ορθογραφικά λάθη στο κείμενο")
("no style" "χωρίς στυλ")
("none" "κανένα")
("normal" "κανονικό")
("not" "όχι")
("not equal" "όχι ίσο")
("not found" "δεν βρέθηκε")
("note" "σημείωση")
("november" "νοέμβριος")
("number" "αριθμός")
("number of columns" "αριθμός στηλών")
("numbered equation" "αριθμημένη εξίσωση")
("numbered equations" "αριθμημένες εξισώσεις")
("numerator" "αριθμητής")
("numeric" "αριθμητικό")
("objectives" "στόχοι")
("occurrences" "συμβάντα")
("october" "Οκτώβριος")
("odd page margin" "περιθώριο σελιδών με περιττό αριθμό")
("off" "μη ενεργό")
("ok" "εντάξει")
("old english" "αρχαία αγγλικά")
("old slavonic" "αρχαία σλαβικά")
("on" "ενεργό")
("on cells" "στα κελιά")
("on columns" "στις στήλες")
("on entire table" "σε όλο τον πίνακα")
("on rows" "στις σειρές")
("one spelling error has been corrected" "ένα ορθογραφικό λάθος διορθώθηκε")
("open" "άνοιγμα")
("open a new buffer" "νέο buffer")
("open new window" "νέο παράθυρο")
("operation on named selection" "λειτουργία επί καθορισμένης επιλογής")
("operation on primary selection" "λειτουργία επί πρωτεύουσας επιλογής")
("operation on secondary selection" "λειτουργία επί δευτερεύουσας επιλογής")
("operation on ternary selection" "λειτουργία επί τριτεύουσας επιλογής")
("optical" "οπτικό")
("option" "επιλογή")
("options" "επιλογές")
("or" "ή")
("orange" "πορτοκαλί")
("orientation" "προσανατολισμός")
("origin" "προέλευση")
("other" "άλλο")
("other pages" "άλλες σελίδες")
("other projects" "άλλα projects")
("other style" "άλλο στυλ")
("outlined roman" "outlined roman")
("over" "επί")
("padding" "γέμισμα")
("page" "σελίδα")
("page bottom margin" "κάτω περιθώριο σελίδας")
("page break" "αλλαγή σελίδας")
("page height" "ύψος σελίδας")
("page insertion" "εισαγωγή σελίδας")
("page layout" "μορφή σελίδας")
("page reference" "αναφορά σε σελίδα")
("page right margin" "δεξί περιθώριο σελίδας")
("page top margin" "πάνω περιθώριο σελίδας")
("page type" "τύπος σελίδας")
("page width" "πλάτος σελίδας")
("paper" "χαρτί")
("paper type" "τύπος χαρτιού")
("papyrus" "πάπυρος")
("paragraph" "παράγραφος")
("paragraph mode" "λειτουργία παραγράφου")
("paragraph width" "πλάτος παραγράφου")
("part" "μέρος")
("past changes" "προηγούμενες αλλαγές")
("paste" "επικόλληση")
("paste from" "επικόλληση από")
("paste text" "επικόλληση κειμένου")
("path" "path")
("perform operations on cells" "εφαρμογή τροποποιήσεων στα κελιά")
("perform operations on columns" "εφαρμογή τροποποιήσεων στις στήλες")
("perform operations on entire table" "εφαρμογή τροποποιήσεων σε όλο τον πίνακα")
("perform operations on rows" "εφαρμογή τροποποιήσεων στις σειρές")
("personal dictionary has been modified" "το προσωπικό σας λεξικό τροποποιήθηκε ")
("phantasy" "phantasy")
("phoenician" "phoenician")
("phonetic" "phonetic")
("pink" "ροζ")
("plain block" "απλό πλαίσιο/τμήμα")
("plain tabular" "απλός πίνακας δεδομένων")
("please" "παρακαλώ")
("please wait" "παρακαλώ περιμένετε")
("plus" "συν")
("polish" "πολωνικά")
("portrait" "πορτραίτο")
("portuguese" "πορτογαλικά")
("position" "θέση")
("position float" "τοποθέτηση μετακινούμενου")
("position floating object" "τοποθέτηση μετακινούμενου αντικειμένου"
("position the table" "τοποθέτηση του πίνακα")
("postscript" "postscript")
("postscript image" "εικόνα postscript")
("pre-alpha release" "δοκιμαστική έκδοση pre-alpha")
("preamble" "προοίμιο")
("preview before printing" "προεπισκόπηση εκτύπωσης")
("preview with ghostview" "προεπισκόπηση μέσω ghostview")
("primary" "πρωτεύον")
("prime" "πρώτο")
("print" "εκτύπωση")
("print all" "εκτύπωση όλων")
("print all to file" "εκτύπωση όλων σε αρχείο")
("print page selection" "εκτύπωση επιλεγμένων σελίδων")
("print page selection to file" "εκτύπωση επιλεγμένων σελίδων σε αρχείο")
("print to postscript file" "εκτύπωση σε αρχείο postscript")
("printer dpi" "ανάλυση εκτυπωτή σε dpi")
("printing command" "εντολή εκτύπωσης")
("professional" "επαγγελματικό")
("prog" "prog")
("prog font" "prog γραμματοσειρά")
("prog font family" "οικογένεια prog γραμματοσειράς")
("prog font series" "σειρά prog γραμματοσειράς")
("prog font shape" "σχήμα prog γραμματοσειράς")
("program" "πρόγραμμα")
("program mode" "λειτουργία προγράμματος")
("programming" "προγραμματισμός")
("project" "project")
("projects" "projects")
("prologue" "πρόλογος")
("prompt on scripts" "προτροπή για την εκτέλεση scripts")
("proof" "απόδειξη")
("proportional" "ανάλογο")
("proportional typewriter" "proportional typewriter")
("proposition" "πρόταση")
("query replace" "αντικατάσταση κειμένου")
("quit" "έξοδος")
("quotation" "παράθεμα")
("quote" "παράθεση")
("range" "εύρος")
("really huge" "τερατεράστιο")
("really quit" "επιβεβαίωση εξόδου")
("red" "κόκκινο")
("redo" "επανάληψη ενέργειας")
("redo undone changes" "επανάληψη μη γενόμενων αλλαγών")
("reduce bottom margin by" "μείωση κάτω περιθωρίου κατά")
("reduce left margin by" "μείωση αριστερού περιθωρίου κατά")
("reduce margins" "μείωση περιθωρίων")
("reduce right margin by" "μείωση δεξιού περιθωρίου κατά")
("reduce top margin by" "μείωση πάνω περιθωρίου κατά")
("reduction even page margin" "μείωση περιθωρίου άρτιων σελίδων")
("reduction odd page margin" "μείωση περιθωρίου περιττών σελίδων")
("reduction page bottom margin" "μείωση κάτω περιθωρίου σελίδας")
("reduction page right margin" "μείωση δεξιού περιθωρίου σελίδας")
("reduction page top margin" "μείωση πάνω περιθωρίου σελίδας")
("reference" "αναφορά")
("references" "αναφορές")
("regular" "κανονικό")
("release" "έκδοση")
("remark" "σχόλιο")
("repeat object" "επανάληψη αντικειμένου")
("replace" "αντικατάσταση")
("replace by" "αντικατάσταση από")
("replaced" "αντικαταστάθηκε")
("replaced one occurrence" "αντικαταστάθηκε ένα συμβάν")
("reset" "επαναφορά")
("resize object" "αναπροσδιορισμός μεγέθους αντικειμένου")
("resized" "αναπροσδιορίστηκε το μέγεθος")
("return" "επιστροφή")
("revert buffer" "επαναφορά buffer")
("right" "δεξιά")
("right border" "δεξί περίγραμμα")
("right margin" "δεξί περιθώριο")
("right padding" "δεξί γέμισμα")
("right ragged" "τραβηγμένο δεξιά")
("right subscript" "δεξιός δείκτης")
("right superscript" "δεξιός εκθέτης")
("right surrounding" "δεξί περιστοίχισμα")
("rigid" "άκαμπτο")
("rigid horizontal" "άκαμπτο οριζόντια")
("rigid space" "άκαμπτο κενό")
("romanian" "ρουμανικά")
("root" "ρίζα")
("row" "σειρά")
("row contains glued items" "η σειρά περιέχει εφαπτόμενα αντικείμενα")
("runic" "ρουνικά")
("running author" "τρέχων συγγραφέας")
("running title" "τρέχων τίτλος")
("russian" "ρωσικά")
("sans serif" "sans serif")
("save" "αποθήκευση")
("save TeXmacs file" "αποθήκευση αρχείου TeXmacs")
("save TeXmacs style file" "αποθήκευση αρχείου στυλ TeXmacs")
("save as" "αποθήκευση ως")
("save as LaTeX" "αποθήκευση ως LaTeX")
("save as TeX" "αποθήκευση ως TeX")
("save buffer" "αποθήκευση buffer")
("save buffer as" "αποθήκευση buffer ως")
("save file" "αποθήκευση αρχείου")
("save html file" "αποθήκευση αρχείου html")
("save image" "αποθήκευση εικόνας")
("save latex file" "αποθήκευση αρχείου latex")
("save tex file" "αποθήκευση αρχείου tex")
("save this buffer" "αποθήκευση buffer")
("save verbatim file" "αποθήκευση αρχείου verbatim")
("saved" "αποθηκεύτηκε")
("screen layout" "εμφάνιση οθόνης")
("screen margins" "περιθώρια οθόνης")
("script" "script")
("script above" "παραπάνω script")
("script below" "παρακάτω script")
("script script size" "")
("script size" "μέγεθος script")
("scripts not accepted" "μη αποδοχή scripts")
("search" "αναζήτηση")
("searching" "αναζήτηση...")
("secondary" "δευτερεύον")
("section" "ενότητα")
("security" "ασφάλεια")
("select a document style" "στυλ εγγράφου")
("select a foreground color" "επιλογή χρώματος προσκηνίου")
("select a language" "επιλογή γλώσσας")
("selected region" "επιλεγμένη περιοχή")
("selection" "επιλογή")
("seminar" "σεμινάριο")
("separation between lines" "διαχωρισμός γραμμών")
("separation between paragraphs" "διαχωρισμός παραγράφων")
("separator" "διαχωριστικό")
("september" "Σεπτέμβριος")
("series" "σειρά")
("series|right" "σειρά|δεξιά")
("session" "σύνοδος")
("session name" "όνομα συνόδου")
("session type" "τύπος συνόδου")
("set" "ρύθμιση")
("set background color of cell" "ρύθμιση χρώματος παρασκηνίου κελιού")
("set cell mode" "ρύθμιση λειτουργίας κελιού")
("set extension center" "ρύθμιση extension center")
("set height" "ρύθμιση ύψους")
("set height of cell" "ρύθμιση ύψους κελιού")
("set margins" "ρύθμιση περιθωρίων")
("set n" "τιμή του ν")
("set paragraph margins" "περιθώρια παραγράφου")
("set paragraph mode" "μορφή παραγράφου")
("set printer dpi" "dpi εκτυπωτή")
("set printing command" "εντολή εκτύπωσης")
("set span" "ρύθμιση ανοίγματος")
("set special cell properties" "ρύθμιση ειδικών ιδιοτήτων κελιού")
("set special table properties" "ρύθμιση ειδικών ιδιοτήτων πίνακα")
("set text width" "ρύθμιση πλάτους κειμένου")
("set width" "ρύθμιση πλάτους")
("set width of cell" "ρύθμιση πλάτους κελιού")
("setup printer" "ρύθμιση εκτυπωτή")
("shape" "σχήμα")
("shift-down" "shift-down")
("shift-right" "shift-right")
("show header and footer" "εμφάνιση κεφαλίδας και υποσελίδου")
("shrinking factor" "παράγοντας σμίκρυνσης")
("size" "μέγεθος")
("size limits" "όρια μεγέθους")
("slanted" "γυρτά")
("sloppy" "ατημέλητα")
("small" "μικρό")
("small caps" "καπιταλάκια") 
("small figure" "μικρή εικόνα")
("small skip" "μικρό κενό")
("small table" "μικρός πίνακας")
("space" "διάστημα")
("space between lines" "διάστημα γραμμών")
("space between paragraphs" "διάστημα παραγράφων")
("spacing" "διάστημα")
("spanish" "ισπανικά")
("special" "ειδικό")
("special cell properties" "ειδικές ιδιότητες κελιού")
("special table properties" "ειδικές ιδιότητες πίνακα")
("specific" "ιδιαίτερο")
("spell" "ορθογραφία")
("spell checking complete" "ο ορθογραφικός έλεγχος ολοκληρώθηκε")
("spelling error" "ορθογραφικό λάθος")
("spelling errors have been corrected" "τα ορθογραφικά λάθη διορθώθηκαν")
("split" "χωρισμός")
("square root" "τετραγωνική ρίζα")
("start" "έναρξη")
("start a new chapter" "έναρξη νέου κεφαλαίου")
("start a new section" "έναρξη νέας ενότητας")
("start an interactive session" "έναρξη αλληλεπίδρασης")
("stretchable" "τανυόμενο") 
("stretchable horizontal" "τανυόμενο οριζοντίως")
("stretchable vertical" "τανυόμενο κατακορύφως")
("style" "στυλ")
("styles" "στυλ")
("sub" "υπό")
("subparagraph" "υποπαράγραφος")
("subscript" "δείκτης")
("subsection" "υποενότητα")
("subsub" "υπό-υπό")
("subsubsection" "υπό-υποενότητα")
("superscript" "εκθέτης")
("swedish" "σουηδικά")
("switch to formula style" "αλλαγή σε στυλ μαθηματικού τύπου")
("symbol" "σύμβολο")
("tab" "tab")
("table" "πίνακας")
("table command" "εντολή πίνακα")
("table of contents" "πίνακας περιεχομένων")
("tables" "πίνακες")
("tabular" "πίνακας δεδομένων")
("ternary" "τριτεύον")
("test" "δοκιμή")
("tex" "tex")
("text" "κείμενο")
("text font" "γραμματοσειρά κειμένου")
("text height correction" "διόρθωση ύψους κειμένου")
("text width" "πλάτος κειμένου")
("texts" "κείμενα")
("th century bc" "th century bc")
("th root" "-οστή ρίζα")
("thai" "ταϋλανδικά")
("the application did not send its usual startup banner" "η εφαρμογή δεν έστειλε το συνηθισμένο banner εκκίνησης")
("the application does not reply" "η εφαρμογή δεν ανταποκρίνεται")
("the application is already running" "η εφαρμογή είναι ήδη σε λειτουργία")
("theorem" "θεώρημα")
("there are unsaved files" "υπάρχουν μη αποθηκευμένα αρχεία")
("this line" "αυτή η γραμμή")
("tilda" "tilda")
("tilde" "περισπωμένη")
("times" "επί")
("tiny" "μικροσκοπικό")
("title" "τίτλος")
("to the left" "στα αριστερά")
("to the right" "στα δεξιά")
("top" "κορυφή")
("top border" "πάνω περίγραμμα")
("top level" "πρώτο επίπεδο")
("top margin" "πάνω περιθώριο")
("top padding" "πάνω γέμισμα")
("translate" "μετάφραση")
("tree" "δέντρο")
("tuple" "πολλαπλότητα")
("turkish" "τουρκικά")
("type" "πληκτρολόγηση")
("type a latex command followed by" "πληκτρολόγηση εντολής latex ακολουθούμενης από")
("type body" "πληκτρολόγηση σώματος")
("type denominator" "πληκτρολόγηση παρονομαστή")
("type numerator" "πληκτρολόγηση ονομαστή")
("type script" "πληκτρολόγηση script")
("typesetted boxes" "στοιχειοθετημένα πλαίσια")
("typewriter" "γραφομηχανή")
("typewriter small caps" "καπιταλάκια γραφομηχανής")
("ugaritic" "ugaritic")
("undo" "αναίρεση ενέργειας")
("undo last changes" "αναίρεση τελευταίων αλλαγών")
("up" "πάνω")
("update" "ανανέωση")
("update image links" "ανανέωση συνδέσμων σε εικόνες")
("upper border" "ανώτερο περίγραμμα")
("use a black color" "χρήση μαύρου χρώματος")
("use a blue color" "χρήση μπλε χρώματος")
("use a brown color" "χρήση καφέ χρώματος")
("use a calligraphic font" "χρήση καλλιγραφικής γραμματοσειράς")
("use a green color" "χρήση πράσινου χρώματος")
("use a grey color" "χρήση γκρι χρώματος")
("use a magenta color" "χρήση μωβ χρώματος")
("use a orange color" "χρήση πορτοκαλιού χρώματος")
("use a pink color" "χρήση ροζ χρώματος")
("use a red color" "χρήση κόκκινου χρώματος")
("use a sans serif font" "χρήση sans serif γραμματοσειράς")
("use a typewriter font" "χρήση γραμματοσειράς γραφομηχανής")
("use a white color" "χρήση λευκού χρώματος")
("use a yellow color" "χρήση κίτρινου χρώματος")
("use blackboard bold font" "χρήση έντονης γραμματοσειράς μαυροπίνακα")
("use bold font series" "χρήση έντονης γραμματοσειράς")
("use calligraphical font" "χρήση καλλιγραφικής γραμματοσειράς")
("use footnote font size" "χρήση μεγέθους γραμματοσειράς υποσημείωσης")
("use fraktur font" "χρήση fraktur γραμματοσειράς")
("use huge font size" "χρήση τεράστιου μεγέθους γραμματοσειράς")
("use italic font shape" "χρήση πλάγιου σχήματος γραμματοσειράς")
("use large font size" "χρήση μεγάλου μεγέθους γραμματοσειράς")
("use left hand column as border" "χρήση αριστερής στήλης ως περίγραμμα")
("use medium font series" "χρήση μεσαίας γραμματοσειράς")
("use normal font size" "χρήση κανονικού μεγέθους γραμματοσειράς")
("use package" "χρήση πακέτου")
("use paragraph width" "χρήση πλάτους παραγράφου")
("use really huge font size" "χρήση τερατεραστίου μεγέθους γραμματοσειράς")
("use right font shape" "χρήση ορθού σχήματος γραμματοσειράς")
("use right hand column as border" "χρήση δεξιάς στήλης ως περίγραμμα")
("use roman font family" "χρήση γραμματοσειράς οικογένειας roman")
("use row above as border" "χρήση παραπάνω σειράς ως περίγραμμα")
("use row below as border" "χρήση παρακάτω σειράς ως περίγραμμα")
("use sans serif font family" "χρήση γραμματοσειράς οικογένειας sans serif")
("use script font size" "χρήση μεγέθους γραμματοσειράς σεναρίου")
("use slanted font shape" "χρήση ατημέλητου σχήματος γραμματοσειράς")
("use small capitals" "χρήση μικρών κεφαλαίων")
("use small font size" "χρήση μικρού μεγέθους γραμματοσειράς")
("use small-caps font shape" "χρήση σχήματος γραμματοσειράς μικρών κεφαλαίων")
("use the blackboard bold font" "χρήση έντονης γραμματοσειράς μαυροπίνακα")
("use the fraktur font" "χρήση fraktur γραμματοσειράς")
("use tiny font font size" "χρήση μικροσκοπικού μεγέθους γραμματοσειράς")
("use typewriter font family" "χρήση γραμματοσειράς οικογένειας γραφομηχανής")
("use very large font size" "χρήση πολύ μεγάλου μεγέθους γραμματοσειράς")
("user provided icons" "διαθέσιμα εικονίδια")
("value" "τιμή")
("vector" "διάνυσμα")
("verbatim" "verbatim")
("verse" "στροφή")
("vertical" "καθέτως")
("vertical after" "καθέτως μετά")
("vertical before" "καθέτως πριν")
("vertical cell alignment" "στοίχιση κελιού καθέτως")
("vertical part" "κάθετο τμήμα")
("vertical position" "κάθετη θέση")
("vertical space" "κάθετο διάστημα")
("vertical table alignment" "καθετη στοίχιση πίνακα")
("vertically align cell to the base" "στοίχιση κελιού καθέτως στη βάση")
("vertically center the cell" "κεντράρισμα του κελιού οριζοντίως")
("very large" "πολύ μεγάλο")
("very small" "πολύ μικρό")
("vietnamese" "βιετναμέζικα")
("view" "επισκόπηση")
("visible" "ορατό")
("warning" "προειδοποίηση")
("welcome" "καλωσήλθατε")
("welcome to GNU TeXmacs" "καλωσήλθατε στον GNU TeXmacs")
("when finished" "όταν τελείωσει")
("while" "while")
("white" "λευκό")
("wide accent" "πλατύς τόνος")
("width" "πλάτος")
("width is exactly the specified width" "το πλάτος είναι ακριβώς το δοσμένο")
("width is maximum of specified width and box width" "το πλάτος είναι το μέγιστο από: δοσμένο και πλάτος πλαισίου")
("width is minimum of specified width and box width" "το πλάτος είναι το ελάχιστο από: δοσμένο και πλάτος πλαισίου")
("width magnification" "μεγέθυνση πλάτους")
("window height" "ύψος παραθύρου")
("window width" "πλάτος παραθύρου")
("with" "με")
("write a name" "όνομα")
("write bold mathematics" "έντονα μαθηματικά")
("write bold text" "έντονο κείμενο")
("write italic text" "πλαγιαστό κείμενο")
("write sample text" "κείμενο-δείγμα")
("write strong text" "έντονο κείμενο") 
("write typewriter text" "κείμενο γραφομηχανής")
("write verbatim text" "κείμενο κώδικα/εντολής")
("yellow" "κίτρινο")
("you can also obtain" "μπορείτε επίσης να αποκτήσετε")
("your document is back in its original state" "το έγγραφό σας επανήλθε στην αρχική του κατάσταση")
