("GNU" "GNU")
("LaTeX" "LaTeX")
("TeX" "TeX")
("a modifier" "modificateur A")
("a series" "série A")
("abbreviation" "abréviation")
("about" "sur")
("about menu" "sur")
("above" "en haut")
("abstract" "résumé")
("accent" "accent")
("accept" "accepter")
("accept all scripts" "accepter tous les scripts")
("accept no scripts" "ne jamais accepter de scripts")
("accept script" "accepter script")
("acknowledgment" "remerciements")
("acronym" "sigle")
("action" "action")
("activate" "activer")
("activate once" "activer une fois")
("activate hidden" "activer et cacher")
("activation" "activation")
("acute" "aigu")
("add package" "ajouter packetage")
("add switch after" "ajouter commutateur après")
("add switch before" "ajouter commutateur avant")
("address" "adresse")
("address block" "bloc d'adresse")
("advanced" "niveau supérieur")
("advanced layout features" "mise en page avancée")
("after" "après")
("albanian" "albanais")
("algorithm" "algorithme")
("align cell to the bottom" "aligner la cellule en bas")
("align cell to the left" "aligner la cellule à gauche")
("align cell to the right" "aligner la cellule à droite")
("align cell to the top" "aligner la cellule en haut")
("align table at base of left column" "aligner tableau à la base de la colonne à gauche")
("align table at base of middle column" "aligner tableau à la base de la colonne au centre")
("align table at base of origin column" "aligner tableau à la base de la colonne d'origine")
("align table at base of right column" "aligner tableau à la base de la colonne à droite")
("align table at fraction bar height" "aligner tableau à la hauteur des barres des fractions")
("align table at the base of bottom row" "aligner tableau à la base de la ligne du bas")
("align table at the base of middle row" "aligner tableau à la base de la ligne du milieu")
("align table at the base of origin row" "aligner tableau à la base de la ligne d'origine")
("align table at the base of top row" "aligner tableau à la base de la ligne du haut")
("align table at the bottom" "aligner tableau en bas")
("align table at the center" "aligner tableau au centre")
("align table at the horizontal center" "aligner tableau au centre horizontal")
("align table at the left" "aligner tableau à gauche")
("align table at the top" "aligner tableau en haut")
("align table to the right" "aligner tableau à droite")
("align text to the left" "aligner le texte à gauche")
("align text to the right" "aligner le texte à droite")
("aligned" "aligné")
("alignment" "alignement")
("all" "le tout")
("allow" "permettre")
("allow for macro definitions in preamble" "permettre des définitions de macros
dans la préambule")
("allowed extension" "extension permise")
("allowed reduction" "réduction permise")
("alpha release" "version alpha")
("alt key" "touche alt")
("alternative specification" "spécification alternative")
("always" "toujours")
("american" "américain")
("and" "et")
("angular" "angulaire")
("appendix" "annexe")
("apply" "appliquer")
("apply macro" "appliquer macro")
("apply macro once" "appliquer macro une fois")
("april" "avril")
("arab" "arabe")
("arc cosine" "arc cosinus")
("arc sine" "arc sinus")
("arc tangent" "arc tangente")
("arch series" "série arch")
("archaic" "archaïque")
("architecture" "architecture")
("argument" "argument")
("arithmetic" "arithmétique")
("armenian" "arménien")
("arrow" "flèche")
("arrows" "flèches")
("article" "article")
("as on paper" "comme sur le papier")
("assign" "changer")
("at fraction bar height" "à l'hauteur des barres des fractions")
("at the baseline" "à la ligne de base")
("at the bottom" "en bas")
("at the left" "à gauche")
("at the middle" "au milieu")
("at the right" "à droite")
("at the top" "en haut")
("attach" "attacher")
("attach buffer to which project" "attacher tampon à quel projet")
("august" "août")
("author" "auteur")
("authorize" "authoriser")
("automatic" "automatique")
("automatically close brackets" "fermer automatiquement les parenthèses")
("automatically generated content" "génération automatique de contenu")
("automatic quotes" "guillemets automatiques")
("autosave" "sauvegarde automatique")
("axis" "axe")
("axiom" "axiome")
("b series" "série B")
("background" "arrière-plan")
("background color" "couleur de fond")
("background evaluations" "évaluations en arrière-plan")
("backward replace" "remplacement en arrière")
("backward search" "recherche en arrière")
("bad data format" "mauvais format de fichier")
("bar" "barre")
("bard" "bard")
("base level" "niveau inférieur")
("baseline" "ligne de base")
("bashkirian" "bashkirien")
("basic" "base")
("basic communication using pipes" "communication de base par tubes")
("basic data types" "types de base")
("basic types" "types de base")
("be useful" "soyez utile")
("become a TeXmacs developer" "devenez un développeur TeXmacs")
("before" "avant")
("begin" "début")
("below" "en bas")
("beta release" "version bèta")
("bibliography" "bibliographie")
("bibliography file" "fichier de bibliographie")
("bibliography style" "style de bibliographie")
("big" "grand")
("big figure" "grande figure")
("big operator" "grand opérateur")
("big skip" "grand saut")
("big table" "grand tableau")
("binary operation" "opération binaire")
("binary relation" "relation binaire")
("black" "noir")
("blackboard bold" "tableau noir gras")
("blackboard math" "tableau noir math")
("blackletter" "lettre noir")
("blank column" "colonne vide")
("blank row" "ligne vide")
("block content" "contenu de bloc")
("blue" "bleu")
("body" "corps")
("bold" "gras")
("book" "livre")
("border" "bordure")
("both" "les deux")
("bottom" "en bas")
("bottom border" "bordure en bas")
("bottom cell border" "bordure en bas de la cellule")
("bottom cell padding" "espacement en bas de la cellule")
("bottom margin" "marge en bas")
("bottom padding" "espacement en bas")
("bottom table border" "bordure en bas du tableau")
("bottom table padding" "espacement en bas du tableau")
("branch" "branche")
("break" "saut")
("breaking" "sauts")
("breve" "demi-cercle")
("british" "britannique")
("broken white" "blanc cassé")
("brown" "brun")
("browse" "naviguer")
("browse web" "naviguer sur le réseau")
("browser" "navigateur")
("buffer" "tampon")
("bulgarian" "bulgare")
("bullet" "puce")
("bullets" "puces")
("by" "par")
("by typing" "en tapant")
("bytes free" "octets de libre")
("bytes used" "octets utilisés")
("calculus" "calcul différentiel")
("calligraphic" "calligraphique")
("cancel" "annuler")
("caps-lock key" "touche majuscules")
("case" "choisir")
("cc" "copies à")
("cell" "cellule")
("cell background color" "couleur de la cellule")
("cell border" "bordure de la cellule")
("cell height" "hauteur de la cellule")
("cell operation mode" "mode d'opération sur les cellules")
("cell width" "largeur de la cellule")
("cells" "cellules")
("center" "centré")
("center text" "centrer le texte")
("centered" "centré")
("centered block" "bloc centré")
("centered tabular" "tableau centré")
("change border of cell" "changer bordure de la cellule")
("change cell operation mode" "changer mode d'opération")
("change log" "tous les changements")
("change the border of the table" "changer la bordure du tableau")
("changes" "changements")
("chapter" "chapitre")
("check" "tchèche")
("check text for spelling errors" "vérifier le texte pour des fautes d'orthographe")
("chinese" "chinois")
("choice" "choix")
("citation" "citation")
("cite" "citer")
("class" "classe")
("clear" "effacer")
("clear selection" "effacer sélection")
("clipped" "tronqué")
("clipped and magnified" "tronqué et agrandi")
("clipped and resized" "tronqué et redimensionné")
("clone" "cloner")
("clone window" "cloner fenêtre")
("close" "fermer")
("close TeXmacs" "fermer TeXmacs")
("close buffer" "fermer tampon")
("close document" "fermer document")
("close session" "fermer la session")
("close window" "fermer fenêtre")
("closing" "fermeture")
("closing style" "style de fermeture")
("code" "code")
("color" "couleur")
("column" "colonne")
("column contains glued items" "la colonne contient des parties collées")
("column to the left" "colonne vers la gauche")
("column to the right" "colonne vers la droite")
("columns" "colonnes")
("common formats" "formats courants")
("compact" "compact")
("compactification" "compactification")
("compactify" "compactifier")
("compatibility with other formats" "compatibilité avec les autres formats")
("compile article" "compiler un article")
("compile book" "compiler un livre")
("compile buffer" "compiler tampon")
("compile project" "compiler projet")
("completion" "complétion")
("complex" "complexe")
("concrete" "concret")
("condensed" "condensé")
("condition" "condition")
("configuration" "configuration")
("contact" "contact")
("content" "contenu")
("content tag" "balise")
("context dependent icons" "icônes dépendantes du contexte")
("continuation of" "continuation de")
("contract logarithms" "contracter les logarithmes")
("contribute" "contribuer")
("control the evaluation of expressions" "contrôler l'évaluation des expressions")
("convention" "convention")
("converters" "convertisseurs")
("converters to other data formats" "convertisseurs pour d'autres formats")
("cosine" "cosinus")
("copy" "copier")
("copy text" "copier du texte")
("copy to" "copier vers")
("copying" "copier")
("corollary" "corollaire")
("correct text" "corrige texte")
("create a new document" "créer un nouveau document")
("create web site" "création d'un site web")
("cursors" "curseurs")
("curve" "courbe")
("custom tab" "tabulation personnalisé")
("customizing TeXmacs" "personnalisation")
("customizing the TeXmacs styles" "personnaliser les styles TeXmacs")
("custum tab" "tabulation personnalisée")
("cut" "couper")
("cut text" "couper du texte")
("cut to" "couper vers")
("cypriot" "cypriote")
("cyrillic" "cyrillique")
("cyrillic input method" "cyrillique par le clavier")
("cyrillic italic right" "cyrillique italique droit")
("czech" "tchèque")
("dancers" "danseurs")
("danish" "danois")
("dark grey" "gris foncé")
("dash" "tiret")
("dashes" "tirets")
("data conversion" "conversion des données")
("data format" "format des données")
("date" "date")
("deactivate" "desactiver")
("deactivate table" "desactiver tableau")
("december" "décembre")
("decimal comma" "virgule décimal")
("decimal dot" "point décimal")
("decorate atoms" "décorations atomiques")
("default" "valeur par défaut")
("default padding" "espacement par défaut")
("definition" "définition")
("delete" "supprimer")
("delete column" "supprimer colonne")
("delete row" "supprimer ligne")
("denominator" "dénominateur")
("description" "description")
("destination" "destination")
("detach" "détacher")
("detailed" "détaillé")
("detailed menus" "menus détaillés")
("details in menus" "détails dans les menus")
("determinant" "déterminant")
("devangari" "dévangari")
("developers guide" "guide du développeur")
("differentiate" "dériver")
("directory" "répertoire")
("disable" "désactiver")
("disable first indentation" "désactiver première indentation")
("disable first indentation after" "désactiver première indentation après")
("disable first indentation before" "désactiver première indentation avant")
("disable indentation" "désactiver indentation")
("disable indentation after" "désactiver indentation après")
("disable indentation before" "désactiver indentation avant")
("disable last indentation" "désactiver dernière indentation")
("disable last indentation after" "désactiver dernière indentation après")
("disable last indentation before" "désactiver dernière indentation avant")
("disactivate" "disactiver")
("disactivate once" "disactiver une fois")
("disallow" "interdire")
("distribute unused space" "distribuer espace non utilisé")
("div" "div")
("do not remap" "ne pas renvoyer")
("document" "document")
("document format" "format de document")
("document style" "style de document")
("document styles" "styles de document")
("documentation" "documentation")
("documents are trees" "les documents sont des arbres")
("dot" "rond")
("dots" "ronds")
("double stroke" "lignes doubles")
("down" "en bas")
("download" "télécharger")
("dpi" "points par pouce")
("draw image" "dessiner image")
("draw over selection" "dessiner sur sélection")
("duplicate" "en double")
("dutch" "néerlandais")
("dynamic libraries" "librairies dynamiques")
("edit" "éditer")
("edit source tree" "éditer l'arbre source")
("editing tools" "outils d'édition")
("eigenvalues" "valeurs propres")
("email" "email")
("emphasize" "mis en relief")
("emphasize text" "mettre en relief du texte")
("empty box" "boîte vide")
("enable first indentation" "activer première indentation")
("enable first indentation after" "activer première indentation après")
("enable first indentation before" "activer première indentation avant")
("enable indentation" "activer indentation")
("enable indentation after" "activer indentation après")
("enable indentation before" "activer indentation avant")
("enable last indentation" "activer dernière indentation")
("enable last indentation after" "activer dernière indentation après")
("enable last indentation before" "activer dernière indentation avant")
("encl" "inclus")
("encoding" "encodage")
("end" "fin")
("english" "anglais")
("enter a title" "entrer un titre")
("entire table" "tableau entier")
("enumerate" "énumération")
("enumerate|Alpha" "Lettres")
("enumerate|Roman" "Romain")
("enumerate|alpha" "lettres")
("enumerate|roman" "romain")
("env" "environnement")
("environment" "environnement")
("epilogue" "épilogue")
("equal" "égal à")
("equation" "équation")
("equations" "équations")
("equivalent for" "équivalent pour")
("error" "erreur")
("esperanto" "espéranto")
("etruscan" "étrusque")
("evaluate" "évaluer")
("evaluate scheme expression" "évaluer expression scheme")
("evaluation switch" "commutateur d'évaluation")
("evaluation tag" "balise d'évaluation")
("even page footer" "en bas pour pages paires")
("even page header" "en-tête pour pages paires")
("even page margin" "marge pour pages paires")
("even page left margin" "marge à gauche pour pages paires")
("even page shift" "décalage pour les pages paires")
("even page text" "texte pour les pages paires")
("exact horizontal" "horizontal exact")
("exact mode" "mode exact")
("exam" "examen")
("example" "exemple")
("exclusive" "exclusif")
("exclusive or" "ou exclusif")
("executable" "exécutable")
("execute" "exécuter")
("execute a TeXmacs command" "executer une commande TeXmacs")
("execute system command" "executer commande système")
("executive" "exécutif")
("exercise" "exercice")
("expand" "développer")
("expand logarithms" "développer les logarithmes")
("expand trigonometric functions" "développer les fonctions trigonométriques")
("expand unrecognized macros" "développer les macros non reconnus")
("expand user-defined macros" "développer les macros des utilisateurs")
("explained" "détaillé")
("exponential" "exponentielle")
("export" "exporter")
("export as" "exporter en tant que")
("export bibliographies as links" "exporter les bibliographies comme des liens")
("export formulas as images" "exporter les formules comme des images")
("extend or normal" "'extend' ou 'normal'")
("extern " "externe")
("extra" "extra")
("extract format" "extraire format")
("extras" "extra")
("factor" "factoriser")
("family" "famille")
("february" "février")
("figure" "figure")
("file" "fichier")
("file name" "nom de fichier")
("file not found" "fichier non trouvé")
("find next match" "chercher l'occurrence suivante")
("find text" "chercher du texte")
("finnish" "finlandais")
("first contact" "premier contact")
("first indentation" "première indentation")
("first item" "premier élément")
("first page" "première page")
("first start" "démarrage")
("flat" "plat")
("flexibility" "flexibilité")
("flexibility factor for vertical spacing" "facteur de flexibilité pour l'espacement vertical")
("floating figure" "figure flottante")
("floating object" "objet flottant")
("floating table" "tableau flottant")
("fold" "plier")
("fold input field" "plier champ d'entrée")
("folded" "plié")
("font" "police")
("font base size" "taille de police de base")
("font dpi" "points par pouce")
("font family" "famille de police")
("font name" "nom de police")
("font series" "espèce de police")
("font shape" "forme de police")
("font size" "taille de police")
("font type" "type de police")
("font variant" "variante de police")
("fonts" "polices")
("footer" "en bas")
("footnote" "note en bas de page")
("for" "pour")
("foreground" "premier-plan")
("foreign" "étranger")
("format" "format")
("formatted and structured output" "sorties formattées et structurées")
("formatted date" "date formatée")
("formatting" "formattage")
("formula" "formule")
("formula style" "style formules")
("forward replace" "remplacement en avant")
("forward search" "recherche en avant")
("fraction" "fraction")
("french" "français")
("full manuals" "manuels complets")
("function" "fonction")
("functional" "fonctionnel")
("further customization" "personnalisation")
("future changes" "changements à venir")
("gNU TeXmacs comes without any form of legal warranty" "GNU TeXmacs n'offre aucune garantie juridique")
("gNU TeXmacs falls under the GNU general public license" "GNU TeXmacs tombe sous la ``GNU general public license''")
("gcd" "pgcd")
("general architecture of TeXmacs" "architecture générale de TeXmacs")
("general negation" "négation générale")
("general note" "note générale")
("generating font file" "Génération du fichier de police")
("generic" "générique")
("georgian" "géorgien")
("german" "allemand")
("getting started" "commencer")
("glossary" "glossaire")
("glossary entry" "entrée dans le glossaire")
("glue" "coller")
("glue decorations" "coller décorations")
("go" "aller")
("gothic" "gothique")
("graphical interface" "interface graphique")
("grave" "grave")
("greater" "supérieur à")
("greater or equal" "supérieur ou égal à")
("greek" "grecque")
("greek letter" "lettre grecque")
("green" "vert")
("grey" "gris")
("group" "groupe")
("h modifier" "modificateur H")
("happy typing" "bonne rédaction")
("has been aborted" "a été arrêtée")
("has completed its task" "a effectué sa tâche")
("hat" "circonflexe")
("header" "en-tête")
("header and footer" "en-tête et bas de page")
("hebrew" "hébreu")
("height" "hauteur")
("height is exactly the specified height" "utiliser la hauteur spécifiée")
("height is maximum of specified height and box height" "utiliser maximum de la hauteur spécifiée et la hauteur de la boîte")
("height is minimum of specified height and box height" "utiliser minimum de la hauteur spécifiée et la hauteur de la boîte")
("height magnification" "agrandissement de la hauteur")
("help" "aide")
("help me" "aidez-moi")
("help us" "aidez-nous")
("here" "ici")
("hidden" "caché")
("home" "racine")
("homepage" "page d'acceuil")
("horizontal" "horizontal")
("horizontal alignment" "alignement horizontal")
("horizontal cell alignment" "alignement horizontal de la cellule")
("horizontal ink separation" "séparation d'encre horizontale")
("horizontal part" "partie horizontale")
("horizontal position" "position horizontale")
("horizontal space" "espace horizontal")
("horizontal table alignment" "alignement horizontal du tableau")
("horizontally align cell to the base" "alignement horizontale de la cellule sur la base")
("horizontally center the cell" "centrer la cellule horizontalement")
("how much longer may pages become" "par combien les pages peuvent-elles se rallonger")
("how much shorter may pages become" "par combien les pages peuvent-elles se raccourcir")
("huge" "énorme")
("hungarian" "hongrois")
("hybrid command" "commande hybride")
("hyperlink" "hyperlien")
("hyphenation" "césure")
("icelandic" "islandais")
("if" "si")
("if hyphenated" "en mode césure")
("image" "image")
("image links" "images liés")
("implementation" "implantation")
("import" "importer")
("improving the current implementation" "amélioration de l'implantation présente")
("in popup windows" "dans des fenêtres à part")
("inactive" "inactif")
("include" "inclure")
("include a document" "inclure un document")
("include file" "inclure un fichier")
("inclusion into TeXmacs" "inclusion dans TeXmacs")
("inclusions" "inclusions")
("indent" "indenter")
("indentation flag" "indentation")
("index" "index")
("index entry" "entrée dans l'index")
("index level" "niveau d'indices")
("informative flags" "drapeaux informatifs")
("inline arguments" "arguments en ligne")
("input mode" "mode des entrées")
("insert" "insérer")
("insert Pr" "insérer Pr")
("insert a TeXmacs symbol" "insérer un symbole TeXmacs")
("insert a big operator" "insérer un grand opérateur")
("insert a binary operation" "insérer une opération binaire")
("insert a binary relation" "insérer une relation binaire")
("insert a blackboard bold character" "insérer un caractère gras de tableau noir")
("insert a bold calligraphic character" "insérer un caractère calligraphique gras")
("insert a bold character" "insérer un caractère gras")
("insert a calligraphic character" "insérer un caractère calligraphique")
("insert a condition" "insérer une condition")
("insert a control flow instruction" "insérer une instruction de flux de contrôle")
("insert a fraction" "insérer une fraction")
("insert a fraktur character" "insérer un caractère Gothique")
("insert a greek character" "insérer un caractère grec")
("insert a large left delimiter" "insérer un grand délimiteur à gauche")
("insert a large right delimiter" "insérer un grand délimiteur à droite")
("insert a large separator" "insérer un grand séparateur")
("insert a left subscript" "insérer un indice à gauche")
("insert a link" "insérer un lien")
("insert a miscellaneous symbol" "insérer un symbole divers")
("insert a new branch" "insère une nouvelle branche")
("insert a new item" "insérer un nouvel élément")
("insert a numerical operation" "insérer une opération numérique")
("insert a picture" "insérer une image")
("insert a square root" "insérer une racine carrée")
("insert a table" "insérer un tableau")
("insert a table or a matrix" "insérer un tableau ou une matrice")
("insert a textual operation" "insérer une opération textuelle")
("insert a wide symbol above" "insérer un accent large au dessus")
("insert a wide symbol below" "insérer un accent large en dessous")
("insert an" "insérer un")
("insert an accent" "insérer un accent")
("insert an arrow" "insérer une flèche")
("insert an environment" "insérer un environnement")
("insert an n" "insérer une racine n")
("insert an operation on tuples" "insérer une opération sur des tuples")
("insert appendix" "insérer appendice")
("insert arccos" "insérer arccos")
("insert arcsin" "insérer arcsin")
("insert arctan" "insérer arctan")
("insert arg" "insérer arg")
("insert argument" "insérer argument")
("insert author" "insérer auteur")
("insert automatically generated content" "insérer du contenu automatiquement engendré")
("insert big contour integral" "insérer une grande intégrale de contour ")
("insert big coproduct" "insérer un grand coproduit")
("insert big direct sum" "insérer une grande somme directe")
("insert big dotted point" "insérer un grand point dans un cercle")
("insert big integral" "insérer une grande intégrale")
("insert big intersection" "insérer une grande intersection")
("insert big logical and" "insérer un grand et logique")
("insert big logical or" "insérer un grand ou logique")
("insert big product" "insérer un grand produit")
("insert big square union" "insérer une grande réunion angulaire")
("insert big summation" "insérer une grande somme")
("insert big tensor product" "insérer un grand produit tensoriel")
("insert big union" "insérer une grande réunion")
("insert big union sum" "insérer une grande union somme")
("insert big vertical space" "insérer une grande espace verticale")
("insert chapter" "insérer chapitre")
("insert column" "insérer colonne")
("insert column to the left" "insérer colonne vers la gauche")
("insert column to the right" "insérer colonne vers la droite")
("insert cos" "insérer cos")
("insert cosh" "insérer cosh")
("insert cot" "insérer cot")
("insert coth" "insérer coth")
("insert csc" "insérer csc")
("insert deg" "insérer deg")
("insert det" "insérer det")
("insert dim" "insérer dim")
("insert dollar" "insérer dollar")
("insert equation" "insérer équation")
("insert executable markup" "insérer du contenu exécutable")
("insert exp" "insérer exp")
("insert fields" "insérer des champs")
("insert footnote" "insérer note en bas de page")
("insert formula" "insérer une formule")
("insert gcd" "insérer gcd")
("insert hom" "insérer hom")
("insert horizontal interword space" "insérer une espace")
("insert image" "insérer image")
("insert inactive markup" "insérer du contenu inactif")
("insert inf" "insérer inf")
("insert input field above" "insérer champ d'entrée au dessus")
("insert input field below" "insérer champ d'entrée en dessous")
("insert ker" "insérer ker")
("insert large" "insérer un grand")
("insert lg" "insérer lg")
("insert lim" "insérer lim")
("insert liminf" "insérer liminf")
("insert limsup" "insérer limsup")
("insert ln" "insérer ln")
("insert log" "insérer log")
("insert macro" "insérer macro")
("insert mathematics" "insérer des mathématiques")
("insert max" "insérer max")
("insert medium horizontal space" "insérer une espace horizontale moyenne")
("insert medium vertical space" "insérer une espace verticale moyenne")
("insert min" "insérer min")
("insert negative horizontal space" "insérer une petite espace négative")
("insert new item or number" "insérer un nouvel élément ou numéro")
("insert new rows or columns" "insérer nouvelles lignes ou colonnes")
("insert or delete rows or columns" "insérer ou supprimer lignes ou colonnes")
("insert row" "insérer ligne")
("insert row above" "insérer ligne au dessus")
("insert row below" "insérer ligne en dessous")
("insert sec" "insérer sec")
("insert section" "insérer section")
("insert sharp" "insérer dièse")
("insert sin" "insérer sin")
("insert sinh" "insérer sinh")
("insert small horizontal space" "insérer un petit espace horizontal")
("insert small vertical space" "insérer une petit espace verticale")
("insert structural markup" "insérer du contenu structurel")
("insert subsection" "insérer sous-section")
("insert subsubsection" "insérer sous-sous-section")
("insert sup" "insérer sup")
("insert tan" "insérer tan")
("insert tanh" "insérer tanh")
("insert text field" "insérer champ de texte")
("insert thick horizontal space" "insérer une espace horizontale épaisse")
("insert title" "insérer titre")
("insert value" "insérer valeur")
("integrate" "intégrer")
("interactive command" "commande interactive")
("interactive questions" "questions interactives")
("interfacing" "interfaçage")
("interfacing TeXmacs with other systems" "interfaçage de TeXmacs avec d'autres systèmes")
("interjection" "interjection")
("interline separation" "séparation entre les lignes")
("interline space" "espace d'interligne")
("internationalization" "internationalisation")
("interparagraph space" "espace d'interparagraphe")
("interrupt execution" "interrompre l'éxécution")
("introduction" "introduction")
("invert" "inverser")
("invisible" "invisible")
("invisible citation" "citation invisible")
("irish" "irlandais")
("is running" "est en marche")
("is tuple" "est tuplet")
("ispell does not respond" "ispell ne répond pas")
("ispell is already busy" "ispell a déjà été lancé")
("italian" "italien")
("italic" "italique")
("italic right" "italique droit")
("itemize" "liste")
("january" "janvier")
("japanese" "japonais")
("july" "juillet")
("june" "juin")
("justified" "justifié")
("justify text" "justifier le texte")
("keep evaluated expressions" "garder les expressions évaluées")
("key" "touche")
("keyboard" "clavier")
("keyboard equivalent" "raccourci clavier")
("keyboard modifiers" "modificateurs de clavier")
("keyboard shorthand" "raccourci clavier")
("keymaps" "claviers")
("keyword" "mot-clef")
("keywords" "mots-clefs")
("kill" "tuer")
("known bugs" "bogues connues")
("label" "étiquette")
("landscape" "paysage")
("language" "langue")
("large" "grand")
("large closing bracket" "grande parenthèse fermante")
("large delimiter" "grand délimiteur")
("large opening bracket" "grande parenthèse ouvrante")
("large separator" "grand séparateur")
("last indentation" "dernière indentation")
("last page" "dernière page")
("latex command" "commande latex")
("latin" "latin")
("layout" "mise en page")
("lcm" "ppcm")
("left" "à gauche")
("left arrow" "flèche vers la gauche")
("left border" "limite à gauche")
("left cell border" "bordure de la cellule à gauche")
("left margin" "marge à gauche")
("left padding" "espacement à gauche")
("left prime" "prime à gauche")
("left ragged" "centré à gauche")
("left slanted" "italique vers la gauche")
("left subscript" "indice à gauche")
("left superscript" "exposant à gauche")
("left surrounding" "entourage à gauche")
("left table border" "bordure du tableau à gauche")
("legal" "légal")
("lemma" "lemme")
("length" "longueur")
("less" "inférieur à")
("less or equal" "inférieur ou égal à")
("letter" "lettre")
("license" "licence")
("light" "léger")
("light grey" "gris clair")
("limitations" "limitations")
("limits" "limites")
("line" "droite")
("line break" "saut de ligne")
("line stretch" "étirabilité de lignes")
("linear algebra" "algèbre linéaire")
("link" "lien")
("link image" "lier image")
("list" "liste")
("list of figures" "liste de figures")
("list of tables" "liste de tableaux")
("load" "charger")
("load LaTeX file" "charger fichier LaTeX")
("load TeX file" "charger fichier TeX")
("load TeXmacs file" "charger fichier TeXmacs")
("load TeXmacs style file" "charger fichier de style TeXmacs")
("load a file" "charger un fichier")
("load buffer" "charger tampon")
("load file" "charger fichier")
("load html file" "charger fichier html")
("load image" "charger image")
("load in new window" "charger dans nouvelle fenêtre")
("load latex file" "charger fichier latex")
("load more recent autosave file" "charger le fichier plus récent de sauvegarde automatique")
("load tex file" "charger fichier tex")
("load verbatim file" "charger fichier verbatim")
("loaded" "chargé")
("logarithm" "logarithme")
("logarithms" "logarithmes")
("long" "long")
("look and feel" "aspect de l'interface")
("look up" "rechercher")
("lower border" "limite en bas")
("m modifier" "modificateur M")
("macro" "macro")
("magenta" "mauve")
("magic" "magique")
("magnification" "agrandissement")
("magnified" "agrandi")
("main" "principal")
("main icon bar" "icônes principales")
("major changes" "changements majeurs")
("make a page insertion" "créer une insertion de page")
("make a subscript" "insérer un indice")
("make a superscript" "insérer un exposant")
("make acute" "créer un accent aigu")
("make bar" "créer une barre")
("make check" "créer un tchèche")
("make fraction" "créer une fraction")
("make grave" "créer un accent grave")
("make hat" "créer un accent circonflexe")
("make label" "créer une étiquette")
("make negation" "créer une négation")
("make page reference" "créer une référence de page")
("make reference" "créer une référence")
("make square root" "créer une racine carrée")
("make tilde" "créer un tilde")
("make title" "construire titre")
("make vector" "créer un vecteur")
("making donations" "effectuer des dons")
("manual" "manuel")
("map to" "envoyer vers")
("march" "mars")
("margins" "marges")
("margins as on paper" "marges comme sur le papier")
("master" "maître")
("master file" "fichier maître")
("math" "maths")
("math font" "police mathématique")
("math font family" "famille de police mathématique")
("math font series" "espèce de police mathématique")
("math font shape" "forme de police mathématique")
("mathematical" "mathématique")
("mathematical and customized input" "entrées mathématiques et personnalisées")
("mathematical font" "police mathématique")
("mathematical formulas" "formules mathématiques")
("mathematical input" "entrées sous forme mathématique")
("mathematical typesetting" "mise en page mathématique")
("mathematics" "mathématiques")
("matrices" "matrices")
("matrix" "matrice")
("maximal" "maximal")
("maximal cell height" "hauteur maximale de la cellule")
("maximal cell width" "largeur maximale de la cellule")
("maximal number of columns" "nombre minimal de colonnes")
("maximal number of rows" "nombre maximal de lignes")
("maximal table height" "hauteur maximale du tableau")
("maximal table width" "largeur maximale du tableau")
("maximum mode" "mode maximum")
("may" "mai")
("medium" "moyen")
("medium skip" "saut moyen")
("meminfo" "info mémoire")
("merge" "fusion")
("messy" "embrouillé")
("meta key" "touche meta")
("middle" "au milieu")
("minimal" "minimal")
("minimal cell height" "hauteur minimale de la cellule")
("minimal cell width" "largeur minimale de la cellule")
("minimal number of columns" "nombre minimal de colonnes")
("minimal number of rows" "nombre minimal de lignes")
("minimal table height" "hauteur minimale du tableau")
("minimal table width" "largeur minimale du tableau")
("minimum mode" "mode minimum")
("minus" "moins")
("miscellaneous" "divers")
("miscellaneous features" "fonctionnalités diverses")
("mod" "mod")
("mode" "mode")
("modifier" "modificateur")
("modify cell alignment" "modifier alignement de la cellule")
("modify cell border" "bordure de la cellule")
("modify cell padding" "espacement autour de la cellule")
("modify cell size" "modifier taille de la cellule")
("modify table border" "bordure du tableau")
("modify table padding" "espacement autour du tableau")
("mongolian" "mongol")
("more information about GNU TeXmacs can be found in the Help" "pour plus de renseignements sur GNU TeXmacs, veuillez consulter le menu Aide")
("mosaic" "mosaïque")
("move down" "aller vers le bas")
("move object" "déplacer objet")
("move to the right when finished" "aller à droite pour terminer")
("move up" "aller en haut")
("multiline input" "entrées multi-lignes")
("mutator tags" "balises mutantes")
("my documents" "mes documents")
("n-th" "n-ième")
("n-th root" "racine n-ième")
("name" "nom")
("negation" "négation")
("never" "jamais")
("new" "nouveau")
("new buffer" "nouveau tampon")
("new column" "nouvelle colonne")
("new document" "nouveau document")
("new double page" "nouvelle page double")
("new double page before" "nouvelle page double avant")
("new item" "nouvel élément")
("new line" "nouvelle ligne")
("new number" "nouveau nombre")
("new page" "nouvelle page")
("new page before" "nouvelle page avant")
("new paragraph" "nouveau paragraphe")
("new roman" "nouveau romain")
("new row" "nouvelle ligne")
("new table" "nouveau tableau")
("next" "suivant")
("next line" "ligne suivante")
("no changes need to be saved" "il n'y a pas de changements à sauvegarder")
("no dictionary for" "pas de dictionnaire pour")
("no first indentation" "pas de première indentation")
("no indentation" "pas d'indentation")
("no line break" "pas de saut de ligne")
("no matches found" "je n'ai pas trouvé d'occurrences")
("no matches found for" "pas d'occurrences pour")
("no more matches for" "pas d'autres occurrences de")
("no more redo information available" "tout a déjà été refait")
("no more undo information available" "les renseignements pour défaire ont été épuisés")
("no name" "sans nom")
("no page break after" "pas de saut de page après")
("no page break before" "pas de saut de page avant")
("no spelling errors found in text" "le texte ne contient pas de fautes d'orthographe")
("no style" "sans style")
("none" "néant")
("normal" "normal")
("norwegian" "norvégien")
("not" "non")
("not equal" "différent de")
("not found" "non trouvé")
("notation" "notation")
("note" "note")
("november" "novembre")
("number" "nombre")
("number equation" "numéroter l'équation")
("number of columns" "nombre de colonnes")
("numbered equation" "équation numérotée")
("numbered equations" "équations numérotées")
("numerator" "numérateur")
("numeric" "numérique")
("objectives" "objectifs")
("occurrences" "occurrences")
("october" "octobre")
("odd page footer" "en bas pour pages impaires")
("odd page header" "en-tête pour pages impaires")
("odd page margin" "marge pour pages impaires")
("odd page left margin" "marge à gauche pour pages impaires")
("odd page right margin" "marge à droite pour pages impaires")
("odd page shift" "décalage pour les pages impaires")
("odd page text" "texte pour les pages impaires")
("off" "désactiver")
("ok" "ok")
("old english" "vieil anglais")
("old slavonic" "vieux slave")
("old style" "ancien style")
("on" "activer")
("on cells" "par cellules")
("on columns" "par colonnes")
("on entire table" "par tableau entier")
("on rows" "par lignes")
("on the footer" "sur la barre d'état")
("one spelling error has been corrected" "une faute d'orthographe a été corrigée")
("online documentation" "documentation en ligne")
("online help" "aide en ligne")
("only inline tags" "balises en ligne seulement")
("open" "ouvrir")
("open a new buffer" "ouvrir un nouveau tampon")
("open new window" "ouvrir nouvelle fenêtre")
("opening" "ouverture")
("operation mode" "mode d'opération")
("operation on named selection" "opération sur sélection à spécifier")
("operation on primary selection" "opération sur sélection primaire")
("operation on secondary selection" "opération sur sélection secondaire")
("operation on ternary selection" "opération sur sélection ternaire")
("optical" "optique")
("option" "option")
("options" "options")
("or" "ou")
("orange" "orange")
("orientation" "orientation")
("origin" "origine")
("original welcome message" "message original de bienvenue")
("other" "autre")
("other pages" "autres pages")
("other projects" "autres projets")
("other style" "autre style")
("outlined roman" "romain dédoublé")
("over" "sur")
("overbrace" "accolade au dessus")
("overline" "surligné")
("padding" "espacement")
("page" "page")
("page bottom margin" "marge de page en bas")
("page break" "saut de page")
("page break before" "saut de page avant")
("page height" "hauteur de page")
("page insertion" "insertion de page")
("page layout" "mise en page")
("page number text" "texte pour les numéros de page")
("page numbering" "numérotation des pages")
("page reference" "référence de page")
("page right margin" "marge de page à droite")
("page setup" "paramètres d'impression")
("page top margin" "marge de page en haut")
("page type" "type de page")
("page width" "largeur de page")
("paper" "papier")
("paper type" "type de papier")
("papyrus" "papyrus")
("paragraph" "paragraphe")
("paragraph mode" "type de paragraphe")
("paragraph width" "largeur du paragraphe")
("parametric curve" "courbe paramétrisée")
("parametric surface" "surface paramétrisée")
("part" "partie")
("past changes" "changements effectués")
("paste" "coller")
("paste from" "coller de")
("paste text" "coller du texte")
("path" "chemin")
("perform operations on cells" "opérer cellule par cellule")
("perform operations on columns" "opérer sur des colonnes")
("perform operations on entire table" "opérer sur le tableau entier")
("perform operations on rows" "opérer sur des lignes")
("permanent" "permanent")
("person" "personne")
("personal dictionary has been modified" "votre dictionnaire personnel a été modifié")
("phantasy" "fantaisie")
("philosophy" "philosophie")
("phoenician" "phénicien")
("phonetic" "phonétique")
("pink" "rose")
("plain block" "bloc standard")
("plain tabular" "tableau standard")
("planned changes" "changements prévus")
("plans for the future" "projets pour l'avenir")
("please" "de grâce")
("please wait" "patientez un instant s.v.p.")
("plot" "graphique")
("plugin" "greffon")
("plugins" "greffons")
("plus" "plus")
("polish" "polonais")
("porting TeXmacs to other platforms" "porter TeXmacs vers d'autres plate-formes")
("portrait" "portrait")
("portuguese" "portugais")
("position" "position")
("position float" "positionner flottant")
("position floating object" "positionner l'objet flottant")
("position the table" "positionner le tableau")
("postscript" "postscript")
("postscript image" "image postscript")
("pre-alpha release" "version pré-alpha")
("preamble" "préambule")
("preamble mode" "mode de préambule")
("preferences" "préférences")
("presentation" "présentation")
("presentation mode" "mode de présentation")
("presentation tag" "balise de présentation")
("preview before printing" "aperçu avant l'impression")
("preview command" "commande de prévisualisation")
("preview with ghostview" "prévisualiser avec ghostview")
("previous" "précédant")
("primary" "primaire")
("prime" "prime")
("print" "imprimer")
("print all" "imprimer le tout")
("print all to file" "imprimer le tout vers un fichier")
("print page selection" "imprimer sélection de pages")
("print page selection to file" "imprimer sélection de pages vers un fichier")
("print to postscript file" "imprimer vers fichier postscript")
("printer" "imprimante")
("printer dpi" "résolution de l'imprimante en points par pouce")
("printing command" "commande pour imprimer")
("problem" "problème")
("professional" "professionnel")
("prog" "prog")
("prog font" "police pour programmer")
("prog font family" "famille de police de programmation")
("prog font series" "espèce de police de programmation")
("prog font shape" "forme de police de programmation")
("program" "programme")
("program font" "police de programmation")
("program mode" "mode de programmation")
("programming" "programmation")
("progressive" "progressif")
("project" "projet")
("projects" "projets")
("prologue" "prologue")
("prompt on scripts" "demander l'acceptation de chaque script")
("prompts and default input" "prompts et entrées par défaut")
("proof" "démonstration")
("proportional" "proportionnel")
("proportional typewriter" "proportionnel machine à écrire")
("proposition" "proposition")
("query replace" "remplacement contrôlé")
("quit" "quitter")
("quotation" "citation")
("quote" "citer")
("range" "partie")
("rank" "rang")
("really close it" "vraiment fermer")
("really huge" "vraiment énorme")
("really quit" "voulez-vous vraiment quitter")
("really tiny" "vraiment minuscule")
("recover autosave file" "recupérer fichier de sauvegarde automatique")
("red" "rouge")
("redo" "répéter")
("redo undone changes" "refaire des changements annulés")
("reduce bottom margin by" "réduire marge en bas de")
("reduce left margin by" "réduire marge à gauche de")
("reduce margins" "réduire les marges")
("reduce right margin by" "réduire marge à droite de")
("reduce top margin by" "réduire marge en haut de")
("reduce trigonometric functions" "réduire les fonctions trigonométriques")
("reduction bottom margin" "réduction marge en bas")
("reduction even page margin" "réduction marge de page impaire")
("reduction odd page margin" "réduction marge de page paire")
("reduction page bottom margin" "réduction de la marge de page en bas")
("reduction page right margin" "réduction de la marge de page à droite")
("reduction page top margin" "réduction de la marge de page en haut")
("reduction right margin" "réduction marge à droite")
("reduction top margin" "réduction marge en haut")
("reference" "référence")
("reference guide" "guide de référence")
("reference guide for the TeXmacs primitives" "guide de référence pour les primitives TeXmacs")
("reference guide for the standard TeXmacs styles" "guide de référence pour les
styles standards de TeXmacs")
("references" "références")
("regular" "normal")
("release" "version")
("reload" "recharger")
("remark" "remarque")
("remove" "enlever")
("remove all output fields" "enlever tous les champs de sortie")
("remove fields" "enlever des champs")
("remove input field" "enlever champ d'entrée")
("remove input field above" "enlever champ d'entrée au dessus")
("remove number" "enlever le numéro")
("remove this switch" "enlever ce commutateur")
("renumber this page" "renuméroter cette page")
("repeat" "répéter")
("repeat object" "répéter objet")
("replace" "remplacer")
("replace by" "remplacer par")
("replace unrecognized styles" "remplacer des styles non reconnus")
("replaced" "j'ai remplacé")
("replaced one occurrence" "j'ai remplacé une occurrence")
("reset" "remettre")
("resize object" "redimensionner objet")
("resized" "redimensionné")
("results of the search for" "resultats de la recherche pour")
("return" "entrée")
("revert" "réactualiser")
("revert buffer" "actualiser tampon")
("right" "à droite")
("right arrow" "flèche vers la droite")
("right border" "limite à droite")
("right cell border" "bordure de la cellule à droite")
("right margin" "marge à droite")
("right padding" "espacement à droite")
("right page margin" "marge de page à droite")
("right ragged" "centré à droite")
("right subscript" "indice à droite")
("right superscript" "exposant à droite")
("right surrounding" "entourage à droite")
("right table border" "bordure du tableau à droite")
("rigid" "fixe")
("rigid horizontal" "horizontal fixe")
("rigid space" "espace fixe")
("roman" "romain")
("romanian" "roumain")
("root" "racine")
("rotate backward" "tourner en arrière")
("rotate forward" "tourner en avant")
("row" "ligne")
("row above" "ligne au dessus")
("row below" "ligne en dessous")
("row contains glued items" "la ligne contient des parties collées")
("rows" "lignes")
("runic" "runes")
("running author" "auteur courant")
("running title" "titre courant")
("russian" "russe")
("russian and Ukrainian users" "utilisateurs Russes et Ukrainiens")
("sample" "échantillon")
("sans serif" "sans sérifs")
("save" "sauvegarder")
("save TeXmacs file" "sauvegarder fichier TeXmacs")
("save TeXmacs style file" "sauvegarder fichier de style TeXmacs")
("save as" "sauvegarder sous")
("save as LaTeX" "sauvegarder comme LaTeX")
("save as TeX" "sauvegarder comme TeX")
("save buffer" "sauvegarder tampon")
("save buffer as" "sauvegarder tampon sous")
("save file" "sauvegarder fichier")
("save html file" "sauvegarder fichier html")
("save image" "sauvegarder image")
("save latex file" "sauvegarder fichier latex")
("save tex file" "sauvegarder fichier tex")
("save this buffer" "sauvegarder ce tampon")
("save verbatim file" "sauvegarder fichier verbatim")
("saved" "sauvegardé")
("scheme command" "commande Scheme")
("screen layout" "apparence sur écran")
("screen margins" "marges sur l'écran")
("script" "indice")
("script above" "indice en haut")
("script below" "indice en bas")
("script script size" "taille d'indices d'indices")
("script size" "taille d'indices")
("scripts not accepted" "des scripts ne sont jamais acceptés")
("search" "chercher")
("search words in my documents" "chercher des mots dans mes documents")
("search words in the documentation" "chercher des mots dans la documentation")
("search words in the source code" "chercher des mots dans le code source")
("searching" "cherche")
("secondary" "secondaire")
("section" "section")
("security" "sécurité")
("see" "voir")
("select a document style" "sélectionner un style de document")
("select a foreground color" "sélectionner une couleur")
("select a language" "sélectionner une langue")
("selected region" "région sélectionnée")
("selection" "sélection")
("selections" "sélections")
("seminar" "séminaire")
("sending commands to TeXmacs" "envoyer des commandes à TeXmacs")
("separation between lines" "espacement entre les lignes")
("separation between paragraphs" "espacement entre les paragraphes")
("separator" "séparateur")
("september" "septembre")
("serb" "serbe")
("series" "espèce")
("series|right" "droit")
("session" "session")
("session name" "nom de la session")
("session type" "type de la session")
("sessions" "sessions")
("set" "mettre")
("set background color of cell" "changer la couleur de la cellule")
("set cell mode" "mode d'opération sur les cellules")
("set extension center" "centre d'extension")
("set height" "changer hauteur")
("set height of cell" "changer la hauteur de la cellule")
("set margins" "changer indentations")
("set n" "entrer n")
("set or get environment variables" "changer ou obtenir des variables d'environnement")
("set paragraph margins" "changer les marges du paragraphe")
("set paragraph mode" "changer le mode d'alignement du paragraphe")
("set printer dpi" "spécifier résolution de l'imprimante")
("set printing command" "commande pour imprimer")
("set span" "changer portée")
("set special cell properties" "changer propriétés spéciales de la cellule")
("set special table properties" "changer propriétés spéciales du tableau")
("set text width" "changer largeur du texte")
("set width" "changer largeur")
("set width of cell" "changer la largeur de la cellule")
("settings" "réglage")
("setup printer" "configurer imprimante")
("shape" "forme")
("shift-down" "shift-bas")
("shift-right" "shift-droite")
("short" "court")
("show" "afficher")
("show header and footer" "afficher en-tête et bas de page")
("show hidden" "montrer partie cachée")
("show on screen" "montrer sur l'écran")
("shrinking factor" "facteur de réduction")
("signature" "signature")
("simplify" "simplifier")
("simplification" "simplification")
("simplified evaluation of formulas" "évaluation simplifiée de formules")
("simplified menus" "menus simplifiés")
("sine" "sinus")
("size" "taille")
("size limits" "limitations de taille")
("size tag" "balise de taille")
("slanted" "oblique")
("sloppy" "relâché")
("slovene" "slovène")
("small" "petit")
("small caps" "petites majuscules")
("small figure" "petite figure")
("small skip" "petit saut")
("small table" "petit tableau")
("source code" "code source")
("source tags" "balises du code source")
("space" "espace")
("space between lines" "espace d'interligne")
("space between paragraphs" "espace d'interparagraphe")
("spacing" "espacement")
("spanish" "espagnol")
("special" "spécial")
("special cell properties" "propriétés spéciales de la cellule")
("special functions" "fonctions spéciales")
("special properties" "propriétés spéciales")
("special table properties" "propriétés spéciales du tableau")
("specific" "spécifique")
("specify the size of the table" "spécifier la taille du tableau")
("spell" "orthographe")
("spell checking complete" "le correcteur d'orthographe a terminé")
("spelling error" "faute d'orthographe")
("spelling errors have been corrected" "fautes d'orthographe ont été corrigées")
("split" "division")
("split session" "diviser session")
("square root" "racine carrée")
("standard environment variables" "variables d'environnement standards")
("standard formats" "formats standards")
("standard TeXmacs styles" "styles TeXmacs standards")
("start" "début")
("start a new chapter" "commencer un nouveau chapitre")
("start a new section" "commencer une nouvelle section")
("start an interactive session" "commencer une session interactive")
("status bar" "barre d'état")
("stretch" "étirer")
("stretchable" "étirable")
("stretchable horizontal" "horizontal étirable")
("stretchable vertical" "vertical étirable")
("stretched" "étiré")
("stroke" "tiret")
("strokes" "tirets")
("strong" "important")
("structural" "structurel")
("style" "style")
("styles" "styles")
("stylesheet language" "language des feuilles de style")
("sub" "sous")
("subparagraph" "sous-paragraphe")
("subscript" "indice")
("subsection" "sous-section")
("subsub" "sous-sous")
("subsubsection" "sous-sous-section")
("subtable" "sous-tableau")
("subtitle" "sous-titre")
("summary" "résumé")
("summary of the principal TeXmacs tags" "les principaux marqueurs de contenu")
("superpose" "superposer")
("superposed" "superposé")
("superscript" "exposant")
("surface" "surface")
("swedish" "suédois")
("swiss" "suisse")
("switch" "commutateur")
("switch to first" "passer au premier")
("switch to formula style" "basculer vers le style de formules")
("switch to last" "passer au dernier")
("switch to next" "passer au suivant")
("switch to previous" "passer au précédant")
("switching and folding" "commuter et plier")
("symbol" "symbole")
("system command" "commande système")
("system environment variables" "variables d'environnement systèmes")
("tab" "tabulation")
("table" "tableau")
("table command" "commande de tableau")
("table height" "hauteur du tableau")
("table of contents" "table des matières")
("table width" "largeur du tableau")
("tables" "tableaux")
("tabular" "tabuler")
("tabular material" "tableaux")
("tangent" "tangente")
("teXmacs as an interface" "teXmacs en tant qu'interface")
("teXmacs fonts" "les polices TeXmacs")
("teXmacs notice" "avis sur TeXmacs")
("teXmacs style files" "fichiers de style TeXmacs")
("ternary" "tertiaire")
("test" "test")
("tex" "tex")
("text" "texte")
("text font" "police en mode texte")
("text height correction" "correction de l'hauteur du texte")
("text width" "largeur du texte")
("texts" "textes")
("th century bc" "ième siècle avant J.C.")
("th root" "ième racine")
("thai" "thaï")
("the TeXmacs authors" "les auteurs de TeXmacs")
("the TeXmacs format" "le format TeXmacs")
("the application did not send its usual startup banner" "l'application n'a pas envoyé son message habituel de lancement")
("the application does not reply" "l'application ne répond pas")
("the application is already running" "l'application tourne déjà")
("the boxes produced by the typesetter" "les boîtes produites par le metteur en page")
("the buffer has not been saved" "ce document n'a pas été sauvegardé")
("the graphical user interface" "interface graphique d'utilisateur")
("the leaves of TeXmacs trees" "les feuilles des arbres TeXmacs")
("the mycas example" "l'exemple 'mycas'")
("the primitive TeXmacs constructs" "constructeurs TeXmacs primitifs")
("the standard TeXmacs styles" "les styles TeXmacs standards")
("theorem" "théorème")
("there are unsaved files" "tous les fichiers n'ont pas été sauvegardés")
("this buffer has not been saved" "ce document n'a pas été sauvegardé")
("this column" "cette colonne")
("this document has been produced using" "ce document a été conçu avec")
("this line" "cette ligne")
("this page" "cette page")
("this page footer" "en bas de cette page")
("this page header" "en-tête pour cette page")
("this row" "cette ligne")
("thumbnails" "onglets d'images")
("tilda" "tilde")
("tilde" "tilde")
("times" "fois")
("tiny" "minuscule")
("title" "titre")
("to the left" "vers la gauche")
("to the right" "vers la droite")
("today" "aujourd'hui")
("tools" "outils")
("top" "en haut")
("top border" "bordure en haut")
("top cell border" "bordure en haut de la cellule")
("top cell padding" "espacement en haut de la cellule")
("top level" "niveau supérieur")
("top margin" "marge en haut")
("top padding" "espacement en haut")
("top table border" "bordure en haut du tableau")
("top table padding" "espacement en haut du tableau")
("translate" "traduire")
("transpose" "transposer")
("tree" "arbre")
("triangularize" "trianguler")
("trigonometric" "trigonométrique")
("trigonometry" "trigonométrie")
("tuple" "tuplet")
("turkish" "turque")
("tutorial" "tutoriel")
("type" "type")
("type a latex command followed by" "tapez une commande latex, puis")
("type body" "entrer corps")
("type denominator" "entrer dénominateur")
("type numerator" "entrer numérateur")
("type script" "entrer indice")
("typesetted boxes" "boîtes typographiées")
("typewriter" "machine à écrire")
("typewriter small caps" "petites majuscules machine à écrire")
("typing a simple text" "saisie d'un texte simple")
("typing simple texts" "saisie de textes simples")
("ugaritic" "ougaritique")
("ukrainian" "ukrainien")
("underbrace" "accolade en dessous")
("underline" "sousligné")
("undo" "annuler")
("undo last changes" "annuler les derniers changements")
("up" "en haut")
("update" "actualiser")
("update from web" "réactualiser à partir du réseau")
("update image links" "mise à jour des images liés")
("upper border" "limite en haut")
("upright" "droit")
("use MathML" "utiliser MathML")
("use TeXmacs" "utilisez TeXmacs")
("use a black color" "utiliser une couleur noire")
("use a blue color" "utiliser une couleur bleue")
("use a bold font" "utiliser une police grasse")
("use a brown color" "utiliser une couleur brune")
("use a calligraphic font" "utiliser une police calligraphique")
("use a green color" "utiliser une couleur verte")
("use a grey color" "utiliser une couleur grise")
("use a magenta color" "utiliser une couleur mauve")
("use a orange color" "utiliser une couleur orange")
("use a pink color" "utiliser une couleur rose")
("use a red color" "utiliser une couleur rouge")
("use a sans serif font" "utiliser une police sans sérifs")
("use a typewriter font" "utiliser une police de machine à écrire")
("use a white color" "utiliser une couleur blanche")
("use a yellow color" "utiliser une couleur jaune")
("use blackboard bold font" "utiliser la police pour tableau noir")
("use bold font series" "utiliser une police grasse")
("use calligraphic font" "utiliser la police calligraphique")
("use calligraphical font" "utiliser une police calligraphique")
("use catcode definitions in preamble" "utiliser des catcodes dans la préambule")
("use footnote font size" "utiliser une très petite police")
("use fraktur font" "utiliser la police Gothique")
("use huge font size" "utiliser une police énorme")
("use italic font shape" "utiliser une police italique")
("use large font size" "utiliser une grande police")
("use left hand column as border" "utiliser colonne à gauche comme bordure")
("use medium font series" "utiliser une police d'épaisseur normale")
("use normal font size" "utiliser une police de taille normale")
("use package" "utiliser paquetage")
("use paragraph width" "utiliser largeur du paragraphe")
("use really huge font size" "utiliser une police vraiment énorme")
("use right font shape" "utiliser une police droite")
("use right hand column as border" "utiliser colonne à droite comme bordure")
("use roman font family" "utiliser la police romain")
("use row above as border" "utiliser ligne en haut comme bordure")
("use row below as border" "utiliser ligne en bas comme bordure")
("use sans serif font family" "utiliser la police sans sérifs")
("use script font size" "utiliser une très petite police")
("use slanted font shape" "utiliser une police oblique")
("use small capitals" "utiliser des petites majuscules")
("use small font size" "utiliser une petite police")
("use small-caps font shape" "utiliser une police de petites majuscules")
("use the blackboard bold font" "utiliser la police de tableau noir gras")
("use the fraktur font" "utiliser la police fraktur")
("use tiny font font size" "utiliser une police minuscule")
("use typewriter font family" "utiliser la police machine à écrire")
("use very large font size" "utiliser une très grande police")
("user manual" "manuel d'utilisation")
("user provided icons" "icônes définies par l'utilisateur")
("users of Cyrillic languages" "utilisateurs de langues Cyrilliques")
("value" "valeur")
("variable" "variable")
("variant" "variante")
("vector" "vecteur")
("verbatim" "verbatim")
("verse" "vers")
("vertical" "vertical")
("vertical after" "vertical après")
("vertical alignment" "alignement vertical")
("vertical before" "vertical avant")
("vertical cell alignment" "alignement vertical de la cellule")
("vertical part" "partie verticale")
("vertical position" "position verticale")
("vertical space" "espace vertical")
("vertical table alignment" "alignmement vertical du tableau")
("vertically align cell to the base" "alignement verticale de la cellule à la base")
("vertically center the cell" "centrer la cellule verticalement")
("very large" "très grand")
("very small" "très petite")
("vietnamese" "vietnamien")
("view" "vue")
("visible" "visible")
("warning" "avertissement")
("web" "toile")
("welcome" "bienvenue")
("welcome to GNU TeXmacs" "bienvenue à GNU TeXmacs")
("what is new" "nouveautés")
("when finished" "pour terminer")
("while" "tant que")
("white" "blanc")
("wide accent" "accent large")
("wide bar" "barre large")
("width" "largeur")
("width is exactly the specified width" "utiliser la largeur spécifiée")
("width is maximum of specified width and box width" "utiliser maximum de la largeur spécifiée et la largeur de la boîte")
("width is minimum of specified width and box width" "utiliser minimum de la largeur spécifiée et la largeur de la boîte")
("width magnification" "agrandissement de la largeur")
("window" "fenêtre")
("window height" "hauteur de fenêtre")
("window width" "largeur de fenêtre")
("windows key" "touche windows")
("with" "avec")
("wrap lines" "couper les lignes")
("write a macro" "écrire un macro")
("write a name" "écrire un nom")
("write bold mathematics" "écrire des mathématiques en gras")
("write bold text" "écrire en gras")
("write italic text" "écrire en italique")
("write sample text" "écrire du texte illustratif")
("write strong text" "écrire du texte important")
("write typewriter text" "écrire du texte de machine à écrire")
("write verbatim text" "écrire en verbatim")
("writing data converters" "écrire des convertisseurs de données")
("writing your own style files" "écrire vos propres feuilles de style")
("written with TeXmacs" "conçu avec TeXmacs")
("yellow" "jaune")
("you can also obtain" "vous pouvez également obtenir")
("your document is back in its original state" "votre document est revenu dans son état original")
("your first interface" "votre première interface")
("your language" "votre langue")
