;; Cork characters which have no unicode equivalent

;; (C) 2003  David Allouche
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details. If
;; you don't have this file, write to the Free Software Foundation, Inc., 59
;; Temple Place - Suite 330, Boston, MA 02111-1307, USA.


("#18"	"0")            ; perthousand zero
("#1A"  "j")		; dotless j
("#DF"	"SS")           ; latin capital letter sharp s
